/* Copyright (C) 2011 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */
 
namespace Ella
{
    public enum ItemType
    {
        RECTANGLE,
        CIRCLE,
        IMAGE,
        PATH,
        TEXT
    }
}
public struct Ella.Rect
{
    public double x;
    public double y;
    public double width;
    public double height;
}

public class Ella.Point : GLib.Object
{
    public double x;
    public double y;
    public Point(double x, double y)
    {
        this.x = x;
        this.y = y;
    }
    public Rect rect
    {
        get
        {
            Rect rect = Rect();
            rect.x = x - 5;
            rect.y = y - 5;
            rect.width = 10;
            rect.height = 10;
            return rect;
        }
    }
}

public bool RectIsIn(double x, double y, Ella.Rect rect1)
{
    if(x > rect1.x && x < rect1.x + rect1.width)
    {
        if(y > rect1.y && y < rect1.y + rect1.height)
        {
            return true;
        }
    }
    return false;
}
public abstract class Ella.Item : GLib.Object
{
    public Vala.ArrayList<Keyframe> keyframes;
    private Vala.ArrayList<Keyframe> kf_link;

    public Keyframe current_keyframe;
    public Keyframe kf_right;
    public Keyframe kf_left;
    public int frame_number = 0;
    public int point_selected;
    internal double m_text_width = 1.0;
    internal double m_text_height = 1.0;
    internal string m_text;
    public string text
    {
        set
        {
            m_text = value;
        }
        get
        {
            return m_text;
        }
    }

    private double _font_size = 1;
    public double font_size
    {
        get { return _font_size; }
        set { _font_size = value; }
    }

    internal string _name;
    public string name
    {
        get { return _name; }
        set { _name = value; }
    }

    public double width
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.width;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.width, kf_right.width);
            }
            else
                return 10;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].width = value;
                }
                current_keyframe.width = value;
            }
        }
    }

    public double height
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.height;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.height, kf_right.height);
            }
            else
                return 10;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].height = value;
                }
                current_keyframe.height = value;
            }
        }
    }
    public double x
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.x;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.x, kf_right.x);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].x = value;
                }
                current_keyframe.x = value;
            }
        }
    }
    public double y
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.y;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.y, kf_right.y);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].y = value;
                }
                current_keyframe.y = value;
            }
        }
    }

    public double red
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.red;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.red, kf_right.red);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].red = value;
                }
                current_keyframe.red = value;
            }
        }
    }
    public double green
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.green;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.green, kf_right.green);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].green = value;
                }
                current_keyframe.green = value;
            }
        }
    }
    public double blue
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.blue;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.blue, kf_right.blue);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].blue = value;
                }
                current_keyframe.blue = value;
            }
        }
    }
    public double alpha
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.alpha;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.alpha, kf_right.alpha);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].alpha = value;
                }
                current_keyframe.alpha = value;
            }
        }
    }

    public double red_stroke
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.red_stroke;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.red_stroke, kf_right.red_stroke);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].red_stroke = value;
                }
                current_keyframe.red_stroke = value;
            }
        }
    }
    public double green_stroke
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.green_stroke;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.green_stroke, kf_right.green_stroke);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].green_stroke = value;
                }
                current_keyframe.green_stroke = value;
            }
        }
    }
    public double blue_stroke
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.blue_stroke;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.blue_stroke, kf_right.blue_stroke);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].blue_stroke = value;
                }
                current_keyframe.blue_stroke = value;
            }
        }
    }
    public double alpha_stroke
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.alpha_stroke;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.alpha_stroke, kf_right.alpha_stroke);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].alpha_stroke = value;
                }
                current_keyframe.alpha_stroke = value;
            }
        }
    }
    
    double _line_width = 0;
    
    /* Uncomment the code to re-enable this animation, but don't forget to add
     * the necessary code to load and write the animations in svg then ;) */
    public double line_width
    {
        get
        {
            /*if(current_keyframe != null)
                return current_keyframe.line_width;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.line_width, kf_right.line_width);
            }
            else
                return 0;*/
            return _line_width;
        }
        set
        {
            /*if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].line_width = value;
                }
                current_keyframe.line_width = value;
            }*/
            _line_width = value;
        }
    }
    public double rotation
    {
        get
        {
            if(current_keyframe != null)
                return current_keyframe.rotation;
            else if (kf_right != null && kf_left != null)
            {
                return relativize_kf(kf_left.rotation, kf_right.rotation);
            }
            else
                return 0;
        }
        set
        {
            if(current_keyframe != null)
            {
                for(int i = 0; i < kf_link.size; i++)
                {
                    kf_link[i].rotation = value;
                }
                current_keyframe.rotation = value;
            }
        }
    }

    internal ItemType _type_item;
    public ItemType type_item
    { get {return _type_item;}
      set {_type_item = value; }
    }

    public Item()
    {
        /* Init the list which will contains all keyframes of our object.*/
        keyframes = new Vala.ArrayList<Keyframe>();
        kf_link = new Vala.ArrayList<Keyframe>();

        /* FIXME: and if we don't want to have a keyframe in the position 1? */

        /* And add a keyframe in position 1 to it, to be able to use it. (else,
         * we can't modify properties as width/height/color)
         */
        Keyframe kf = new Keyframe();
        kf.time = 0;
        keyframes.add(kf);

        /* Go to the keyframe we have just created. */
        change_keyframe(0);
    }

    /**
     * Just a function to get the period between the two frame where we are.
     */
    public double get_period()
    {
        return kf_right.time - kf_left.time;
    }

    internal double relativize_kf(double first, double last)
    {
        double period = get_period();

        if(period == 0)
            return first;
        double fr_nb = frame_number - kf_left.time;
        switch(kf_right.type)
        {
        case KeyframeType.SMOOTH:
            double accel = (Math.cos((fr_nb/period + 1)*Math.PI) + 1)/2;
            return ( (1-accel) * first + accel * last );
        case KeyframeType.NORMAL:
            return ( (1 - fr_nb/period) * first + fr_nb/period*last);
        }
        return 0.0;
    }

    public bool change_keyframe(int time)
    {
        frame_number = time;
        current_keyframe = null;
        if(kf_right == null)
            kf_right = keyframes[0];
        if(kf_left == null)
            kf_left = keyframes[0];
        for(int i = 0; i < keyframes.size; i ++)
        {
            if(keyframes[i].time > time && (keyframes[i].time < kf_right.time || kf_right.time <= time))
            {
                kf_right = keyframes[i];
            }
            if(keyframes[i].time == time)
            {
                current_keyframe = keyframes[i];
                kf_link.remove(current_keyframe);
            }
            if(keyframes[i].time < time && (keyframes[i].time > kf_left.time || kf_left.time >= time))
            {
                kf_left = keyframes[i];
            }
        }
        return current_keyframe != null;
    }

    public void move_keyframe(int old_kf, int new_kf)
    {
        for(int i = 0; i < keyframes.size; i ++)
        {
            if(keyframes[i].time == old_kf)
            {
                keyframes[i].time = new_kf;
                return;
            }
        }
    }

    public abstract void draw_editing(Cairo.Context cr, double x_root, double y_root);

    public void translate_points(ref Point[] points)
    {
        for(int i = 0; i < points.length; i++)
        {
            double temp = points[i].x;
            points[i].x = get_real_coords_x(points[i].x, points[i].y);
            points[i].y = get_real_coords_y(temp, points[i].y);
        }
    }

    internal double get_real_coords_y(double x, double y)
    {
        double default_angle;
        double r;
        r = Math.sqrt(y*y + x*x);
        if(y < 0)
            default_angle = Math.acos(x/r) + Math.PI;
        else
            default_angle = Math.acos(x/r);
        default_angle += rotation;
        return Math.sin(default_angle)*r + Math.fabs(y);
    }

    internal double get_real_coords_x(double x, double y)
    {
        double default_angle;
        double r;
        r = Math.sqrt(y*y + x*x);
        if(y < 0)
            default_angle = - Math.acos(x/r);
        else
            default_angle = Math.acos(x/r);
        default_angle += rotation;
        return Math.cos(default_angle)*r + Math.fabs(x);
    }

    internal double get_largest_value(Point[] points, int j)
    {
        double max_value = 0;
        if(j == 0)
        {
            max_value = points[0].x;
        }
        else if(j == 1)
        {
            max_value = points[0].y;
        }
        for(int i = 0; i < points.length; i++)
        {
            if(j == 0)
            {
                if(points[i].x > max_value)
                    max_value = points[i].x;
            }
            else if(j == 1)
            {
                if(points[i].y > max_value)
                    max_value = points[i].y;
            }
        }
        return max_value;
    }

    internal double get_smallest_value(Point[] points, int j)
    {
        double max_value = 0;
        if(j == 0)
        {
            max_value = points[0].x;
        }
        else if(j == 1)
        {
            max_value = points[0].y;
        }
        for(int i = 0; i < points.length; i++)
        {
            if(j == 0)
            {
                if(points[i].x < max_value)
                    max_value = points[i].x;
            }
            else if(j == 1)
            {
                if(points[i].y < max_value)
                    max_value = points[i].y;
            }
        }
        return max_value;
    }

    public void add_keyframe(int time)
    {
        for(int i = 0; i < keyframes.size; i++)
        {
            if(keyframes[i].time == time)
            {
                return;
            }
        }
        Keyframe kf = new Keyframe();
        kf.x = x;
        kf.y = y;
        kf.width = width;
        kf.height = height;
        kf.red = red;
        kf.green = green;
        kf.blue = blue;
        kf.alpha = alpha;
        kf.red_stroke = red_stroke;
        kf.green_stroke = green_stroke;
        kf.blue_stroke = blue_stroke;
        kf.alpha_stroke = alpha_stroke;
        kf.line_width = line_width;
        kf.rotation = rotation;
        kf.time = time;
        keyframes.add(kf);
    }

    public void add_keyframe_linked(int time, KeyframeType type)
    {
        for(int i = 0; i < keyframes.size; i++)
        {
            if(keyframes[i].time == time)
            {
                return;
            }
        }
        Keyframe kf = new Keyframe();
        kf.x = x;
        kf.y = y;
        kf.width = width;
        kf.height = height;
        kf.red = red;
        kf.green = green;
        kf.blue = blue;
        kf.alpha = alpha;
        kf.red_stroke = red_stroke;
        kf.green_stroke = green_stroke;
        kf.blue_stroke = blue_stroke;
        kf.alpha_stroke = alpha_stroke;
        kf.line_width = line_width;
        kf.rotation = rotation;
        kf.time = time;
        kf.type = type;
        keyframes.add(kf);
        kf_link.add(kf);
    }

    public double[] get_center()
    {
        switch(type_item)
        {
        case ItemType.TEXT:
            return {x + m_text_width/2, y + m_text_height/2};
        default:
            return {x + width/2, y + height/2};
        }
    }

    public void parse_color(string rule)
    {
        Gdk.Color color = Gdk.Color();
        Gdk.Color.parse(rule, out color);
        red = ((double)color.red)/0xFFFF;
        green = ((double)color.green)/0xFFFF;
        blue = ((double)color.blue)/0xFFFF;
    }

    public void parse_color_stroke(string rule)
    {
        Gdk.Color color = Gdk.Color();
        Gdk.Color.parse(rule, out color);
        red_stroke = ((double)color.red)/0xFFFF;
        green_stroke = ((double)color.green)/0xFFFF;
        blue_stroke = ((double)color.blue)/0xFFFF;
    }

    public abstract void write_to_xml(Xml.TextWriter writer, int fps);

    public abstract Rect Area();

    public abstract void draw(Cairo.Context cr, double x_root, double y_root);

    internal void start_xml_element(Xml.TextWriter writer, Ella.Keyframe kf, int fps)
    {
        writer.start_element("animate");
        writer.write_attribute("attributeType", "XML");
        writer.write_attribute("begin", (((double)frame_number)/(double)fps).to_string());
        writer.write_attribute("dur", ((double)(kf.time - frame_number)/(double)fps).to_string());
        writer.write_attribute("end", (((double)kf.time) / (double) fps).to_string());
    }
    
    internal void start_xml_element_color(Xml.TextWriter writer, Ella.Keyframe kf, int fps)
    {
        writer.start_element("animateColor");
        writer.write_attribute("attributeType", "CSS");
        writer.write_attribute("begin", (((double)frame_number)/(double)fps).to_string());
        writer.write_attribute("dur", ((double)(kf.time - frame_number)/(double)fps).to_string());
        writer.write_attribute("end", (((double)kf.time) / (double) fps).to_string());
    }

    internal void write_xml_attribute(Xml.TextWriter writer, string attrib, double first, double last)
    {
        writer.write_attribute("attributeName", attrib);
        writer.write_attribute("from", first.to_string());
        writer.write_attribute("to", last.to_string());
        writer.end_element();
    }

    internal void write_xml_attribute_transform(Xml.TextWriter writer, string attrib, double first, double last)
    {
        writer.write_attribute("type", attrib);
        writer.write_attribute("from", first.to_string());
        writer.write_attribute("to", last.to_string());
        writer.end_element();
    }

    internal void write_xml_attribute_str(Xml.TextWriter writer, string attrib, string first, string last)
    {
        writer.write_attribute("attributeName", attrib);
        writer.write_attribute("from", first);
        writer.write_attribute("to", last);
        writer.end_element();
    }

    internal void start_xml_element_transform(Xml.TextWriter writer, Ella.Keyframe kf, int fps)
    {
        writer.start_element("animateTransform");
        writer.write_attribute("attributeType", "XML");
        writer.write_attribute("attributeName", "transform");
        writer.write_attribute("begin", (((double)frame_number)/(double)fps).to_string());
        writer.write_attribute("dur", ((double)(kf.time - frame_number)/(double)fps).to_string());
        writer.write_attribute("end", (((double)kf.time) / (double) fps).to_string());
    }
    
    internal void write_general_infos_xml(Xml.TextWriter writer)
    {
        string transform = "rotate(" + rotation.to_string().replace(",", ".") + ") ";
        if(this is Path)
        {
            transform += "translate(" + x.to_string().replace(",", ".") + "," + y.to_string().replace(",", ".") + ") ";
        }
        if(this is Path || this is Text)
        {
            transform += "scale(" + width.to_string().replace(",", ".") + "," + height.to_string().replace(",", ".") + ") ";
        }
        writer.write_attribute("transform", transform);
        
        Ella.Color color = Ella.Color();
        color.red = red;
        color.green = green;
        color.blue = blue;
        Ella.Color color_stroke = Ella.Color();
        color_stroke.red = red_stroke;
        color_stroke.green = green_stroke;
        color_stroke.blue = blue_stroke;
        writer.write_attribute("style",
            "fill: %s;fill-opacity:%f;stroke-opacity:%f;stroke:%s;stroke-width:%f;".
            printf(color_to_string(color),
                   alpha,
                   alpha_stroke,
                   color_to_string(color_stroke),
                   line_width)
         );
    }
    
    internal void write_x_y(Xml.TextWriter writer, Keyframe kf, int fps)
    {
        start_xml_element(writer, kf, fps);
        write_xml_attribute(writer, "x", x, kf.x);

        start_xml_element(writer, kf, fps);
        write_xml_attribute(writer, "y", y, kf.y);
    }
    
    internal void write_color(Xml.TextWriter writer, Keyframe kf, int fps)
    {
        Ella.Color color = Ella.Color();
        Ella.Color color2 = Ella.Color();

        color.red = red;
        color.green = green;
        color.blue = blue;
        color2.red = kf.red;
        color2.green = kf.green;
        color2.blue = kf.blue;
        
        start_xml_element_color(writer, kf, fps);
        write_xml_attribute_str(writer, "fill", color_to_string(color), color_to_string(color2));

        color.red = red_stroke;
        color.green = green_stroke;
        color.blue = blue_stroke;
        color2.red = kf.red_stroke;
        color2.green = kf.green_stroke;
        color2.blue = kf.blue_stroke;
        
        start_xml_element_color(writer, kf, fps);
        write_xml_attribute_str(writer, "stroke", color_to_string(color), color_to_string(color2));

        start_xml_element(writer, kf, fps);
        write_xml_attribute(writer, "fill-opacity", alpha, kf.alpha);

        start_xml_element(writer, kf, fps);
        write_xml_attribute(writer, "stroke-opacity", alpha_stroke, kf.alpha_stroke);
    }
    
    public void print_kf()
    {
        foreach(Keyframe kf in keyframes)
        {
            print("%d\n", kf.time);
        }
    }
}

public class Ella.Circle : Ella.Item
{

    public override void draw(Cairo.Context cr, double x_root, double y_root)
    {
        cr.translate(x - x_root + width/2, y - y_root + height/2);
        cr.rotate(rotation);
        cr.scale(width/2, height/2);
        cr.set_source_rgba(red, green, blue, alpha);
        cr.arc(0, 0, 1, 0.0, 2.0 * Math.PI); // full circle
        cr.fill_preserve();
        cr.set_line_width(line_width);
        cr.set_source_rgba(red_stroke, green_stroke, blue_stroke, alpha_stroke);
        cr.stroke();
        cr.scale(1/(width/2), 1/(height/2));
        cr.rotate (-rotation);
        cr.translate(-x + x_root - width/2, -y + y_root - height/2);
    }
    
    
    public override Rect Area()
    {
        /* TODO: this is a rectangle area, not a circle one! */
        Rect area = Rect();
        area.x = x;
        area.y = y;
        area.width = 0;
        area.height = 0;
        
        Point[] points = {new Point(-width/2, height/2),
                            new Point(width/2, height/2),
                            new Point(-width/2, -height/2),
                            new Point(width/2, -height/2)};
        translate_points(ref points);

        double max_value = get_smallest_value(points, 1);

        area.height -= max_value;
        area.y += max_value;

        max_value = get_largest_value(points, 1);
        area.height += max_value;

        max_value = get_smallest_value(points, 0);

        area.width -= max_value;
        area.x += max_value;

        max_value = get_largest_value(points, 0);
        area.width += max_value;

        return area;
    }
    
    public override void draw_editing(Cairo.Context cr, double x_root, double y_root)
    {
    }
    
    public override void write_to_xml(Xml.TextWriter writer, int fps)
    {
        int frame_tmp = frame_number;
        change_keyframe(0);
        writer.start_element("ellipse");
        
        writer.write_attribute("rx", (width/2).to_string());
        writer.write_attribute("ry", (height/2).to_string());
        writer.write_attribute("cx", (x + width/2).to_string());
        writer.write_attribute("cy", (y + height/2).to_string());
        write_general_infos_xml(writer);

        
        /* The animations */
        for(int i = 0; i < keyframes.size; i++)
        {
            Ella.Keyframe kf = keyframes[i];

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "cx", x + width/2, kf.x + kf.width/2);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "cy", y + height/2, kf.y + kf.height/2);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "ry", height, kf.height/2);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "rx", width, kf.width/2);

            start_xml_element_transform(writer, kf, fps);
            write_xml_attribute_transform(writer, "rotate", rotation*180/Math.PI, kf.rotation*180/Math.PI);

            change_keyframe(kf.time);
        }

        writer.end_element();
        change_keyframe(frame_tmp);
    }
}


public class Ella.Rectangle : Ella.Item
{

    public override void draw(Cairo.Context cr, double x_root, double y_root)
    {
        cr.translate(x - x_root + width/2, y - y_root + height/2);
        cr.rotate(rotation);
        cr.rectangle(-width/2, -height/2, width, height);
        cr.set_source_rgba(red, green, blue, alpha);
        cr.fill_preserve();
        cr.set_line_width(line_width);
        cr.set_source_rgba(red_stroke, green_stroke, blue_stroke, alpha_stroke);
        cr.stroke();
        cr.rotate (-rotation);
        cr.translate(-x + x_root - width/2, -y + y_root - height/2);
    }
    
    
    public override Rect Area()
    {
        Rect area = Rect();
        area.x = x;
        area.y = y;
        area.width = 0;
        area.height = 0;
        
        Point[] points = {new Point(-width/2, height/2),
                            new Point(width/2, height/2),
                            new Point(-width/2, -height/2),
                            new Point(width/2, -height/2)};
        translate_points(ref points);

        double max_value = get_smallest_value(points, 1);

        area.height -= max_value;
        area.y += max_value;

        max_value = get_largest_value(points, 1);
        area.height += max_value;

        max_value = get_smallest_value(points, 0);

        area.width -= max_value;
        area.x += max_value;

        max_value = get_largest_value(points, 0);
        area.width += max_value;

        return area;
    }
    
    public override void draw_editing(Cairo.Context cr, double x_root, double y_root)
    {
    }
    
    public override void write_to_xml(Xml.TextWriter writer, int fps)
    {
        int frame_tmp = frame_number;
        change_keyframe(0);
        writer.start_element("rect");

        writer.write_attribute("width", width.to_string());
        writer.write_attribute("height", height.to_string());
        writer.write_attribute("x", x.to_string());
        writer.write_attribute("y", y.to_string());
        write_general_infos_xml(writer);
        
        /* The animations */
        for(int i = 0; i < keyframes.size; i++)
        {
            Ella.Keyframe kf = keyframes[i];

            write_x_y(writer, kf, fps);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "height", height, kf.height);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "width", width, kf.width);

            start_xml_element_transform(writer, kf, fps);
            write_xml_attribute_transform(writer, "rotate", rotation*180/Math.PI, kf.rotation*180/Math.PI);

            write_color(writer, kf, fps);

            change_keyframe(kf.time);
        }
        
        writer.end_element();
        change_keyframe(frame_tmp);
    }
}

public class Ella.Path : Ella.Item
{
    private Vala.ArrayList<Point> points = new Vala.ArrayList<Point>();

    public Point add_point(double x, double y)
    {
        var point = new Point(x, y);
        points.add(point);
        return point; 
    }

    public void select_point(Point index)
    {
        for(int i = 0; i < points.size; i++)
        {
            if(points[i] == index)
            {
                point_selected = i;
            }
        }
    }

    public void move_point(double event_x, double event_y)
    {
        event_x = event_x / width;
        event_y = event_y / height;
        points[point_selected].x = event_x;
        points[point_selected].y = event_y;
    }

    public void select_point_at(double event_x, double event_y)
    {
        event_x = (event_x - x) / width;
        event_y = (event_y - y) / height;
        foreach(Point point in points)
        {
            if(RectIsIn(event_x, event_y, point.rect))
            {
                select_point(point);
                return;
            }
        }
        select_point(add_point(event_x, event_y));
    }
    
    public void remove_point()
    {
        points.remove(points[point_selected]);
        point_selected = 0;
    }

    public override void draw(Cairo.Context cr, double x_root, double y_root)
    {
        cr.rotate(rotation);
        cr.translate(x - x_root, y - y_root);
        cr.scale(width, height);
        cr.set_line_width(line_width);
        foreach(Point point in points)
        {
            cr.line_to(point.x, point.y);
        }
        cr.close_path();
        cr.set_source_rgba(red, green, blue, alpha);
        cr.fill_preserve();
        cr.set_source_rgba(red_stroke, green_stroke, blue_stroke, alpha_stroke);
        cr.stroke();
        cr.scale(1/width, 1/height);
        cr.translate(-x + x_root, -y + y_root);
        cr.rotate (-rotation);
    }

    public override void draw_editing(Cairo.Context cr, double x_root, double y_root)
    {
        cr.rotate(rotation);
        cr.translate(x - x_root, y - y_root);
        foreach(Point item in points)
        {
            var rect = item.rect;
            cr.rectangle(rect.x * width, rect.y * height, rect.width, rect.height);
            cr.fill();
        }
        var rect = points[point_selected].rect;
        cr.rectangle(rect.x * width, rect.y * height, rect.width, rect.height);
        cr.set_source_rgb(1, 0, 0);
        cr.fill();
        cr.translate(-x + x_root, -y + y_root);
        cr.rotate (-rotation);
    }

    public override Rect Area()
    {
        Rect area = Rect();
        area.x = x;
        area.y = y;
        area.width = 0;
        area.height = 0;
        Point[] points = new Point[this.points.size];
        for(int i = 0; i < this.points.size; i++)
        {
            points[i] = new Point(this.points[i].x * width, this.points[i].y * height);
        }
        
        double max_value = get_smallest_value(points, 1);

        area.height -= max_value;
        area.y += max_value;

        max_value = get_largest_value(points, 1);
        area.height += max_value;

        max_value = get_smallest_value(points, 0);

        area.width -= max_value;
        area.x += max_value;

        max_value = get_largest_value(points, 0);
        area.width += max_value;
        return area;
    }
    
    public override void write_to_xml(Xml.TextWriter writer, int fps)
    {
        writer.start_element("path");
        int frame_tmp = frame_number;
        change_keyframe(0);
        write_general_infos_xml(writer);

        string path = "M %d,%d ".printf((int)points[0].x, (int)points[0].y);
        
        for(int i = 1; i < points.size; i++)
        {
            path = path + "L %d,%d ".printf((int)points[i].x, (int)points[i].y);
        }

        writer.write_attribute("d", path);


        /* The animations */
        for(int i = 0; i < keyframes.size; i++)
        {
            Ella.Keyframe kf = keyframes[i];

            start_xml_element_transform(writer, kf, fps);
            writer.write_attribute("type", "translate");
            writer.write_attribute("from", x.to_string().replace(",", ".") + "," + y.to_string().replace(",", "."));
            writer.write_attribute("to", kf.x.to_string().replace(",", ".") + "," + kf.y.to_string().replace(",", "."));
            writer.end_element();

            start_xml_element_transform(writer, kf, fps);
            write_xml_attribute_transform(writer, "rotate", rotation*180/Math.PI, kf.rotation*180/Math.PI);

            start_xml_element_transform(writer, kf, fps);
            writer.write_attribute("type", "scale");
            writer.write_attribute("from", width.to_string().replace(",", ".") + "," + height.to_string().replace(",", "."));
            writer.write_attribute("to", kf.width.to_string().replace(",", ".") + "," + kf.height.to_string().replace(",", "."));
            writer.end_element();

            write_color(writer, kf, fps);

            change_keyframe(kf.time);
        }

        writer.end_element();
        change_keyframe(frame_tmp);
    }
}

public class Ella.Image : Ella.Item
{
    public override void draw_editing(Cairo.Context cr, double x_root, double y_root)
    {
    }

    private Gdk.Pixbuf image_surface;

    string path;

    internal double _image_width;
    public double image_width
    { get {return _image_width;}
      set {_image_width = value; }
    }
    internal double _image_height;
    public double image_height
    { get {return _image_height;}
      set {_image_height = value; }
    }
    public void set_image(string path)
    {
        this.path = path;
        try
        {
            image_surface = new Gdk.Pixbuf.from_file(path);
        }
        catch(Error e)
        {
            print("Couldn't load this file: " + path + "\n");
        }
        image_width = image_surface.get_width();
        image_height = image_surface.get_height();
        
    }
    
    public override void draw(Cairo.Context cr, double x_root, double y_root)
    {
        cr.translate(x - x_root, y - y_root);
        cr.scale(width, height);
        double x_trans = image_width/2;
        double y_trans = image_height/2;
        cr.translate(x_trans, y_trans);
        cr.rotate(rotation);
        
        Gdk.cairo_set_source_pixbuf(cr, image_surface, -x_trans, -y_trans);
        cr.paint_with_alpha(alpha);
        
        cr.rotate (-rotation);

        cr.translate(-x_trans, -y_trans);
        cr.scale(1/width, 1/height);
        cr.translate(-x + x_root, -y + y_root);
    }

    public override Rect Area()
    {
        Rect area = Rect();
        area.x = x;
        area.y = y;
        area.width = 0;
        area.height = 0;
        
        Point[] points = {new Point(-image_width * width/2, image_height * height/2),
                            new Point(image_width * width/2, image_height * height/2),
                            new Point(-image_width * width/2, -image_height * height/2),
                            new Point(image_width * width/2, -image_height * height/2)};
        translate_points(ref points);

        double max_value = get_smallest_value(points, 1);

        area.height -= max_value;
        area.y += max_value;

        max_value = get_largest_value(points, 1);
        area.height += max_value;

        max_value = get_smallest_value(points, 0);

        area.width -= max_value;
        area.x += max_value;

        max_value = get_largest_value(points, 0);
        area.width += max_value;
        return area;
    }
    
    public override void write_to_xml(Xml.TextWriter writer, int fps)
    {
        int frame_tmp = frame_number;
        change_keyframe(0);
        writer.start_element("image");
        
        writer.write_attribute("width", image_width.to_string());
        writer.write_attribute("height", image_height.to_string());
        writer.write_attribute("xlink:href", path);
        writer.write_attribute("x", x.to_string());
        writer.write_attribute("y", y.to_string());
        write_general_infos_xml(writer);
        
        /* The animations */
        for(int i = 0; i < keyframes.size; i++)
        {
            Ella.Keyframe kf = keyframes[i];

            write_x_y(writer, kf, fps);

            start_xml_element_transform(writer, kf, fps);
            write_xml_attribute_transform(writer, "rotate", rotation*180/Math.PI, kf.rotation*180/Math.PI);

            start_xml_element_transform(writer, kf, fps);
            writer.write_attribute("type", "scale");
            writer.write_attribute("from", width.to_string().replace(",", ".") + "," + height.to_string().replace(",", "."));
            writer.write_attribute("to", kf.width.to_string().replace(",", ".") + "," + kf.height.to_string().replace(",", "."));
            writer.end_element();

            write_color(writer, kf, fps);

            change_keyframe(kf.time);
        }
        
        writer.end_element();
        change_keyframe(frame_tmp);
    }
}

public class Ella.Text : Ella.Item
{
    public string font_name = "Sans";

    public override void draw_editing(Cairo.Context cr, double x_root, double y_root)
    {
    }

    public override void draw(Cairo.Context cr, double x_root, double y_root)
    {
        /* Set text color */
        cr.set_source_rgba(red, green, blue, alpha);
        
        /**
         * Select font
         * TODO: we must support different fonts
         **/
        cr.select_font_face(font_name, Cairo.FontSlant.NORMAL, Cairo.FontWeight.NORMAL);
        cr.set_font_size(font_size);

        /* FIXME: ATM, this instrucition is useless since we don't draw any text */
        cr.set_line_width(line_width);

        var lines = text.split("\n");

        /* First, we compute the text size */

        m_text_width = 0.0;
        m_text_height = height/2 * lines.length * font_size;
        Cairo.TextExtents txt_extents = Cairo.TextExtents();
        for(int i = 0; i < lines.length; i++)
        {
            /* Get the size that the text uses */
            cr.text_extents(lines[i], out txt_extents);

            /* Check if a line is a longer than this one. If not, we must keep
             * the value. */
            m_text_width = Math.fmax(m_text_width, (txt_extents.x_advance));
        }

        m_text_width = m_text_width * width/2;

        cr.translate(x - x_root + m_text_width/2, y - y_root + m_text_height/2);
        cr.rotate(rotation);
        cr.scale(width/2, height/2);

        for(int i = 0; i < lines.length; i++)
        {
            cr.move_to((-m_text_width) / width,
                       (-m_text_height)/ height + i + 1);

            /* Draw the text */
            cr.show_text(lines[i]);
        }
        cr.scale(2/width, 2/height);
        cr.rotate(-rotation);
        cr.translate(-x + x_root - m_text_width/2, -y + y_root - m_text_height/2);
    }

    public override Rect Area()
    {
        Rect area = Rect();
        area.x = x;
        area.y = y;
        area.width = 0;
        area.height = 0;
        Point[] points = {new Point(-m_text_width/2, m_text_height/2),
                            new Point(m_text_width/2, m_text_height/2),
                            new Point(-m_text_width/2, -m_text_height/2),
                            new Point(m_text_width/2, -m_text_height/2)};
        translate_points(ref points);

        double max_value = get_smallest_value(points, 1);

        area.height -= max_value;
        area.y += max_value;

        max_value = get_largest_value(points, 1);
        area.height += max_value;

        max_value = get_smallest_value(points, 0);

        area.width -= max_value;
        area.x += max_value;

        max_value = get_largest_value(points, 0);
        area.width += max_value;
        return area;
    }
    
    public override void write_to_xml(Xml.TextWriter writer, int fps)
    {
        int frame_tmp = frame_number;
        change_keyframe(0);
        writer.start_element("text");
        
        writer.write_attribute("width", width.to_string());
        writer.write_attribute("height", height.to_string());
        writer.write_attribute("x", x.to_string());
        writer.write_attribute("y", y.to_string());
        write_general_infos_xml(writer);
        
        
        /*  the text itself */
        var lines = m_text.split("\n");
        for(int i = 0; i < lines.length; i++)
        {
            writer.write_element("tspan", lines[i]); 
            //writer.end_element();
        }
        /* The animations */
        for(int i = 0; i < keyframes.size; i++)
        {
            Ella.Keyframe kf = keyframes[i];

            write_x_y(writer, kf, fps);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "height", height, kf.height);

            start_xml_element(writer, kf, fps);
            write_xml_attribute(writer, "width", width, kf.width);

            start_xml_element_transform(writer, kf, fps);
            write_xml_attribute_transform(writer, "rotate", rotation*180/Math.PI, kf.rotation*180/Math.PI);

            write_color(writer, kf, fps);

            change_keyframe(kf.time);
        }
        
        writer.end_element();
        change_keyframe(frame_tmp);
    }
}
