/* Copyright (C) 2011 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */

using Gee;

namespace Ella
{
    public enum LayersType
    {
        DRAWING,
        SOUND
    }
}

public class Ella.KeyframeLayer : Object
{
    public KeyframeLayer(int time, KeyframeType type)
    {
        this.time = time;
        this.type = type;
    }
    public int time;
    public KeyframeType type;
}

public class Ella.Layer : GLib.Object
{
    public Ella.LayersType type;
    private bool _visible = true;
    private ArrayList<Ella.Item> item_list;
    public ArrayList<KeyframeLayer> keyframes_list;
    public bool visible
    {
        get { return _visible; }
        set { _visible = value; }
    }
    private string _id;
    public string id
    {
        get { return _id; }
        set { _id = value; }
    }
    private int _max_frames;
    public int max_frames
    {
        get { return _max_frames; }
        set { _max_frames = value; }
    }

    public int nb_item
    {
        get { return item_list.size; }
    }

    public void set_keyframe_type(int time, KeyframeType type)
    {
        foreach(var item in item_list)
        {
            foreach(Keyframe kf in item.keyframes)
            {
                if(kf.time == time)
                {
                    kf.type = type;
                    break;
                }
            }
        }
        foreach(var kf in keyframes_list)
        {
            if(kf.time == time)
            {
                kf.type = type;
                break;
            }
        }
    }

    public Layer()
    {
        item_list = new ArrayList<Ella.Item>();
        keyframes_list = new ArrayList<KeyframeLayer?>();
        keyframes_list.add(new KeyframeLayer(0, KeyframeType.NORMAL));
    }

    public void add_item(Ella.Item item)
    {
        item_list.add(item);
        foreach(Keyframe kf in item.keyframes)
        {
            bool found = false;
            foreach(var kf_l in keyframes_list)
            {
                if(kf_l.time == kf.time)
                {
                    found = true;
                    break;
                }
            }
            if(!found)
            {
                keyframes_list.add(new KeyframeLayer(kf.time, kf.type));
            }
            
        }
        foreach(var i in keyframes_list)
        {
            item.add_keyframe_linked(i.time, i.type);
        }
        
    }

    public Ella.Item get_item(int index)
    {
        return item_list[index];
    }

    public void draw_all_items(Cairo.Context cr, double x_root, double y_root)
    {
        if(!visible)
            return;
        for(int i = 0; i < item_list.size; i++)
        {
            item_list[i].draw(cr, x_root, y_root);
        }
    }

    public bool move_keyframe(int old_kf, int new_kf)
    {
        bool found = false;
        int index = -1;
        for(int i = 0; i < keyframes_list.size; i++) /* FIXME: foreach ? */
        {
            if(keyframes_list[i].time == new_kf)
            {
                found = false;
                break;
            }
            if(keyframes_list[i].time == old_kf)
            {
                index = i;
                found = true;
            }
        }
        if(!found)
        {
            return false;
        }
        keyframes_list[index].time = new_kf;
        for(int i = 0; i < item_list.size; i++) /* FIXME: foreach ? */
        {
            item_list[i].move_keyframe(old_kf, new_kf);
        }
        bump_maxkeyframe();
        return true;
    }

    public void select_here(ArrayList<Ella.Item> item_selected, double x, double y)
    {
        for(int i = item_list.size - 1; i > -1; i--)
        {
            Rect area = item_list[i].Area();
            if(area.x < x && x < area.x + area.width)
            {
                if(area.y < y && y < area.y + area.height)
                {
                    item_selected.add(item_list[i]);
                    break;
                }
            }
        }
    }

    public void select_all(ArrayList<Ella.Item> item_selected)
    {
        for(int i = 0; i < item_list.size; i++)
        {
          item_selected.add(item_list[i]);
        }
    }
    public void select_area(ArrayList<Ella.Item> item_selected, double x, double y, double x2, double y2)
    {
        for(int i = 0; i < item_list.size; i++)
        {
            Rect area = item_list[i].Area();
            if(x < area.x && x2 > area.x + area.width)
            {
                if(y < area.y && y2 > area.y + area.height)
                {
                    item_selected.add(item_list[i]);
                }
            }
        }
    }
    public double[] compute_size_request(double frame, double page_width, double page_height)
    {
        double x1 = -500;
        double x2 = page_width + 500;
        double y1 = -500;
        double y2 = page_height + 500;
        
        for(int i = 0; i < item_list.size; i++)
        {
            Rect area_item = item_list[i].Area();
            if(area_item.x < x1)
                x1 = area_item.x;
            if(area_item.y < y1)
                y1 = area_item.y;
            if(area_item.x + area_item.width + frame > x2)
                x2 = area_item.x + area_item.width + frame;
            if(area_item.y + area_item.height + frame > y2)
                y2 = area_item.y + area_item.height + frame;
        }
        return {x1, x2, y1, y2};
    }

    public void change_keyframe(int time)
    {
        for(int i = 0; i < item_list.size; i++)
        {
            Ella.Item item = item_list[i];
            item.change_keyframe(time);
        }
    }

    public void add_keyframe(int time)
    {
        bool found = false;
        for(int i = 0; i < keyframes_list.size; i++)
        {
            if(keyframes_list[i].time == time)
            {
                found = true;
                break;
            }
        }
        if(!found)
        {
            keyframes_list.add(new KeyframeLayer(time, KeyframeType.NORMAL));
        }
        for(int i = 0; i < item_list.size; i++)
        {
            Ella.Item item = item_list[i];
            item.add_keyframe(time);
        }
        bump_maxkeyframe();
    }
    
    private void bump_maxkeyframe()
    {
        max_frames = 0;
        for(int i = 0; i < keyframes_list.size; i++)
            max_frames = int.max(max_frames, keyframes_list[i].time);
    }

    public bool remove_item(Item item)
    {
        return item_list.remove(item);
    }

    public bool is_editable(int time)
    {
        for(int i = 0; i < keyframes_list.size; i++)
        {
            if(time == keyframes_list[i].time)
                return true;
        }
        return false;
    }

    public void write_to_xml(Xml.TextWriter writer, int fps)
    {
        string style = "";
        writer.start_element("g");
        writer.write_attribute("id", id);
        if(!visible)
        {
            style = "display:none;";
        }
        writer.write_attribute("style", style);
        for(int i = 0; i < item_list.size; i++)
        {
            item_list[i].write_to_xml(writer, fps);
        }
        writer.end_element();
    }

    public void item_up(int item_number)
    {
        if(item_number + 1 < item_list.size)
        {
            Ella.Item item_tmp = item_list[item_number];
            item_list[item_number] = item_list[item_number + 1];
            item_list[item_number + 1] = item_tmp;
        }
        else
        {
            print("Couldn't raise this item ;)\n");
        }
    }

    public bool item_up_instance(Ella.Item item)
    {
        if(item_list.index_of(item) != -1)
        {
            item_up(item_list.index_of(item));
            return true;
        }
        return false;
    }

    public void item_down(int item_number)
    {
        if(item_number - 1 >= 0)
        {
            Ella.Item item_tmp = item_list[item_number];
            item_list[item_number] = item_list[item_number - 1];
            item_list[item_number - 1] = item_tmp;
        }
        else
        {
            print("Couldn't low this item ;)\n");
        }
    }

    public bool item_down_instance(Ella.Item item)
    {
        if(item_list.index_of(item) != -1)
        {
            item_down(item_list.index_of(item));
            return true;
        }
        return false;
    }

}
