/*
 * ella-dirs.h
 * This file is part of ella
 *
 * Copyright (C) 2008 Ignacio Casal Quinteiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */
#ifndef H_ELLA_DIRS_20101130
#define H_ELLA_DIRS_20101130

#include <stdlib.h>

#include <glib/gstdio.h>

#include "config.h"
#include "ella-print.h"

/**
 * \defgroup dirs Ella Directories
 * \brief Functions to create/choose the data/user configuration directory.
 * @{
 **/
 
const gchar* ella_dirs_get_user_config_dir(void);
gint ella_dirs_check_create_user_config_dir(void);

/** 
 * \brief Check and create the configuration directory, it needs to be run at _every_
 * start of Ella. */
gint ella_check_config_dir(void);

/**
 * \brief Check where is the data directory, it can be (in the right order) :
 * ./data (only if ella_data_flag is in this directory, to avoid any problem
 * if a user has another data folder
 * ELLA_DATADIR (a define which is defined in config.h by CMake)
 * /usr/share/ella
 * If one directory is found, we just don't check the others. */
gint ella_check_data_dir(void);

/**
 * \brief Get the data dir got in ella_check_data_dir(void).
 **/
const gchar* ella_get_data_dir(void);

/**
 * \brief Get the config dir got in ella_check_data_dir(void). */
gchar* ella_get_config_dir(void);

/** @} */
#endif 
