/* Copyright (C) 2010 Lucas Baudin
*
* This file is part of Ella Project.
*
* Ella Project is free software : you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Ella Project is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY ; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
*/

#include "ella-files.h"

FILE* ella_files_open_ella_file(const gchar* path)
{
    return fopen(path, "w");
}

FILE* ella_files_open_ella_file_read(const gchar* path)
{
	return fopen(path, "r");
}

int ella_files_copy(const gchar* source, const gchar* destination)
{
    FILE* src  = fopen(source, "rb");
    int status = 0;
    
    if(src)
    {
        FILE* dest = fopen(destination, "wb");

        if(dest)
        {
            char buffer[BUFSIZ];
            gboolean end = FALSE;

            while(!end)
            {
                size_t nb_read = fread(buffer, 1, sizeof buffer, src);

                if(!ferror(src))
                {
                    fwrite(buffer, 1, nb_read, dest);

                    if(feof(src))
                    {
                        end = TRUE;
                    }
                }
                else
                {
                    end = TRUE;
                    status = 3;
                }
            }
            fclose(dest);
        }
        else
        {
            status = 2;
        }
        fclose(src);
    }
    else
    {
        status = 1;
    }
    
    return status;
}

gchar* ella_files_get_font_file(const gchar* font)
{
    PangoContext* Pct = NULL;
    PangoFont* Pft    = NULL;
    PangoFontDescription* font_desc = NULL;
    FcPattern* pattern = NULL;
    FcChar8* sToRealFontFile = NULL;
    if(font == NULL)
    {
        font = g_strdup_printf("%s","sans 12");
        g_warning("No font specified, we'll use sans 12");
    }
    /* first, we retrieve the current pango context */
    Pct = gdk_pango_context_get();

    /* 2nd, we get a PangoFontDescription for a gchar describing the font face
     * with size, eg 'sans 12' */
    font_desc = pango_font_description_from_string(font);

    /* we get a pointer on a pango font */
    Pft = pango_context_load_font(Pct, font_desc);

    /* We get from the pangofont description the pattern which will be tested by
     * fc-config */  
    g_object_get(Pft, "pattern", &pattern, NULL);
    if (pattern) 
    {
        FcPatternGetString(pattern, FC_FILE, 0, &sToRealFontFile);
    }
    /* We must redo opertions with a standard font in order to avoid problems.
     * FIXME: is it really necessary? Needs more testing. */
    else
    {
        g_warning("Couldn't find this font: %s, we'lle use sans 12", font);
        font = g_strdup_printf("%s","sans 12");
        font_desc = pango_font_description_from_string(font);
        Pft = pango_context_load_font(Pct, font_desc);  
        g_object_get(Pft, "pattern", &pattern, NULL);
        FcPatternGetString(pattern, FC_FILE, 0, &sToRealFontFile);
    }

    return (gchar*)sToRealFontFile;
}

gchar *g_substr(const gchar* string, gint start, gint  end)
{
  gsize len = (end - start + 1);
  gchar* output = g_malloc0(len + 1);
  return g_utf8_strncpy (output, &string[start], len);
}

gboolean ella_files_write_ella_file(EllaFilesData* file_data, const gchar* path)
{
    setlocale(LC_ALL, "C");
    xmlTextWriterPtr writer;
    gint xml_state;
    gchar* color_temp = NULL;
    /* Open the file */
    writer = xmlNewTextWriterFilename(path, 0);
    if(writer == NULL)
    {
        g_critical("Couldn't write the svg file: %s", path);
        return FALSE;
    }
    xmlTextWriterSetIndent(writer, 4);
    xmlTextWriterSetIndentString(writer, (xmlChar*)"\t");
    /* TODO: we should handle more errors */
    
    /* Start the document */
    /* TODO: maybe we could have multiple enconding settings */
    xml_state = xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
    
    /* create the root svg node */
    xml_state = xmlTextWriterStartElement(writer, BAD_CAST "svg");
    
    /* Generic attribute, most are unused in Ella, but they are used to keep
     * compatibility with inkscape. */
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:dc", BAD_CAST "http://purl.org/dc/elements/1.1/");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:rdf", BAD_CAST "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:svg", BAD_CAST "http://www.w3.org/2000/svg");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns", BAD_CAST "http://www.w3.org/2000/svg");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:sodipodi", BAD_CAST "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:inkscape", BAD_CAST "http://www.inkscape.org/namespaces/inkscape");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:ella", BAD_CAST "http://ella.tuxfamily.org");
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:xlink", BAD_CAST "http://www.w3.org/1999/xlink");

    /* height and width */
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "height", BAD_CAST g_strdup_printf("%f", file_data->height));
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "width", BAD_CAST g_strdup_printf("%f", file_data->width));

    /* fps */
    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "ella:fps", BAD_CAST g_strdup_printf("%d", file_data->fps));

    /* create the named view node, it will contains the page background */
    xml_state = xmlTextWriterStartElement(writer, BAD_CAST "sodipodi:namedview");



    color_temp = ella_color_to_string(&(file_data->background_color));

    xml_state = xmlTextWriterWriteAttribute(writer, BAD_CAST "pagecolor", BAD_CAST color_temp);

    xml_state = xmlTextWriterEndElement(writer);
    /* Layers and item */
    GSList* node_files_data = g_slist_nth(file_data->layers, 0);
    while(node_files_data)
    {
        ella_layer_write_to_xml(node_files_data->data, writer, file_data->fps);
        node_files_data = g_slist_next(node_files_data);
    }

    xml_state = xmlTextWriterEndDocument(writer);
    xmlFreeTextWriter(writer);

    setlocale(LC_ALL, "");
    return TRUE;
}

EllaFilesData* ella_files_parse_ella_file(const gchar* path_svg)
{
    xmlTextReader* svg_xml;
    GdkColor color;
    gboolean succes;
    gint ret;
    gchar* attribute_value;
    EllaFilesData* svg_informations = g_malloc(sizeof(EllaFilesData));
    EllaLayer* layer;
    EllaItem* item;
    
    setlocale(LC_ALL, "C");
    
    svg_xml = xmlReaderForFile(path_svg, NULL, 0);
    ret = xmlTextReaderRead(svg_xml);
    svg_informations->layers = NULL;
    if(svg_xml)
    {
        while(ret == 1)
        {
            gchar* node_name = (gchar*)xmlTextReaderConstName(svg_xml);
            /* If it is a start element */
            if(xmlTextReaderNodeType(svg_xml) == 1)
            {
                if(!g_strcmp0(node_name, "svg"))
                {
                    svg_informations->width = strtod((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"width"), NULL);
                    svg_informations->height = strtod((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"height"), NULL);
                    svg_informations->fps = (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"ella:fps"), NULL, 10);
                }
                else if(!g_strcmp0(node_name, "sodipodi:namedview"))
                {
                    succes = gdk_color_parse((gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"pagecolor"), &color);
                    if(succes)
                    {
                        svg_informations->background_color.alpha = 0;
                        svg_informations->background_color.red = color.red/(double)0xFFFF;
                        svg_informations->background_color.green = color.green/(double)0xFFFF;
                        svg_informations->background_color.blue = color.blue/(double)0xFFFF;
                    }
                    else
                    {
                        g_critical("Couldn't read background color.\n");
                    }
                }
                else if(!g_strcmp0(node_name, "g"))
                {
                    layer = ella_layer_new();
                    layer->type = ELLA_LAYERS_TYPE_DRAWING;
                    svg_informations->layers = g_slist_append(svg_informations->layers, layer);

                    /* name */
                    ella_layer_set_id(layer, (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"id"));

                    /* visibility */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml,(xmlChar*) "style");
                    if(attribute_value != NULL)
                        ella_files_parse_css_for_layer(layer, attribute_value);
                }
                else if(!g_strcmp0(node_name, "path"))
                {
                    item = ELLA_ITEM(ella_path_new());
                    ella_layer_add_item(layer, item);
                    succes = FALSE;
                    
                    /* General properties */
                    ella_item_set_width(item, 1);
                    ella_item_set_height(item, 1);

                    /* style */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"style");
                    if(attribute_value)
                        ella_files_parse_css_for_item(item, attribute_value);
                        
                    /* transformations */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"transform");
                    if(attribute_value)
                        ella_files_parse_transform_for_item(item, attribute_value);
                    
                    /* Points */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"d");
                    if(attribute_value)
                        ella_files_parse_path_for_item(item, attribute_value);
                }
                else if(!g_strcmp0(node_name, "rect"))
                {
                    item = ELLA_ITEM(ella_rectangle_new());
                    ella_layer_add_item(layer, item);
                    succes = FALSE;
                    
                    /* General properties */
                    ella_item_set_width(item,
                                        (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml,
                                                                              (xmlChar*)"width"), NULL, 10));
                    ella_item_set_height(item,
                                         (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml,
                                                                               (xmlChar*)"height"), NULL, 10));
                    ella_item_set_x(item,
                                    (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"x"), NULL, 10));
                    ella_item_set_y(item,
                                    (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"y"), NULL, 10));

                    /* style */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"style");
                    if(attribute_value)
                        ella_files_parse_css_for_item(item, attribute_value);
                        
                    /* transformations */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"transform");
                    if(attribute_value)
                        ella_files_parse_transform_for_item(item, attribute_value);
                }
                else if(!g_strcmp0(node_name, "image"))
                {
                    item = ELLA_ITEM(ella_image_new());
                    ella_layer_add_item(layer, item);
                    succes = FALSE;
                    
                    /* General properties */
                    ella_image_set_image(item,
                                        (gchar*)xmlTextReaderGetAttribute(svg_xml,
                                                                          (xmlChar*)"xlink:href"));

                    /* General properties */
                    /*ella_image_set_image_width(item,
                                        (double)strtol((char*)xmlTextReaderGetAttribute(svg_xml,
                                                                              (xmlChar*)"width"), NULL, 10));
                    ella_image_set_image_height(item,
                                         (double)strtol((char*)xmlTextReaderGetAttribute(svg_xml,
                                                                               (xmlChar*)"height"), NULL, 10));*/
                    ella_item_set_x(item,
                                    (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"x"), NULL, 10));
                    ella_item_set_y(item,
                                    (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"y"), NULL, 10));

                    /* style */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"style");
                    if(attribute_value)
                        ella_files_parse_css_for_item(item, attribute_value);
                        
                    /* transformations */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"transform");
                    if(attribute_value)
                        ella_files_parse_transform_for_item(item, attribute_value);
                }
                else if(!g_strcmp0(node_name, "ellipse"))
                {
                    item = ELLA_ITEM(ella_circle_new());
                    ella_layer_add_item(layer, item);
                    succes = FALSE;
                    
                    /* General properties */
                    gint width =  (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"rx"), NULL, 10)*2;
                    gint height = (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"ry"), NULL, 10)*2;
                    ella_item_set_width(item, width);
                    ella_item_set_height(item, height);
                    ella_item_set_x(item, (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"cx"), NULL, 10) - width/2);
                    ella_item_set_y(item, (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"cy"), NULL, 10) - height/2);

                    /* style */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"style");
                    if(attribute_value)
                        ella_files_parse_css_for_item(item, attribute_value);
                        
                    /* transformations */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"transform");
                    if(attribute_value)
                        ella_files_parse_transform_for_item(item, attribute_value);
                        
                }
                else if(!g_strcmp0(node_name, "text"))
                {
                    item = ELLA_ITEM(ella_text_new());
                    ella_layer_add_item(layer, item);
                    succes = FALSE;
                    
                    /* General properties */
                    ella_item_set_x(item, (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"x"), NULL, 10));
                    ella_item_set_y(item, (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"y"), NULL, 10));

                    /* style */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"style");
                    if(attribute_value)
                        ella_files_parse_css_for_item(item, attribute_value);
                        
                    /* transformations */
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"transform");
                    if(attribute_value)
                        ella_files_parse_transform_for_item(item, attribute_value);
                }
                else if(!g_strcmp0(node_name, "tspan"))
                {
                    /* FIXME: we should handle text coords */
                    ret = xmlTextReaderRead(svg_xml);
                    if(ella_item_get_text(item) != NULL)
                    {
                        ella_item_set_text(item, g_strdup_printf("%s\n%s", ella_item_get_text(item), xmlTextReaderConstValue(svg_xml)));
                    }
                    else
                    {
                        ella_item_set_text(item, (gchar*)xmlTextReaderConstValue(svg_xml));
                    }
                }
                else if(!g_strcmp0(node_name, "animate") ||
                        !g_strcmp0(node_name, "animateTransform") ||
                        !g_strcmp0(node_name, "animateColor"))
                {
                    attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"attributeName");
                    gint frame = strtod((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"end"), NULL) * svg_informations->fps;
                    gdouble value = (int)strtol((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to"), NULL, 10);
                    ella_layer_add_keyframe(layer, frame);
                    ella_item_change_keyframe(item, frame);
                    if(!g_strcmp0(attribute_value, "width"))
                    {
                        ella_item_set_width(item, value);
                    }
                    else if(!g_strcmp0(attribute_value, "height"))
                    {
                        ella_item_set_height(item, value);
                    }
                    else if(!g_strcmp0(attribute_value, "x"))
                    {
                        ella_item_set_x(item, value);
                    }
                    else if(!g_strcmp0(attribute_value, "y"))
                    {
                        ella_item_set_y(item, value);
                    }
                    if(!g_strcmp0(attribute_value, "rx"))
                    {
                        ella_item_set_width(item, value*2);
                    }
                    else if(!g_strcmp0(attribute_value, "ry"))
                    {
                        ella_item_set_height(item, value*2);
                    }
                    else if(!g_strcmp0(attribute_value, "cx"))
                    {
                        ella_item_set_x(item, value - ella_item_get_width(item)/2);
                    }
                    else if(!g_strcmp0(attribute_value, "cy"))
                    {
                        ella_item_set_y(item, value - ella_item_get_height(item)/2);
                    }
                    else if(!g_strcmp0(attribute_value, "fill"))
                    {
                        attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to");
                        ella_item_parse_color(item, attribute_value);
                    }
                    else if(!g_strcmp0(attribute_value, "stroke"))
                    {
                        attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to");
                        ella_item_parse_color_stroke(item, attribute_value);
                    }
                    else if(!g_strcmp0(attribute_value, "fill-opacity"))
                    {
                        ella_item_set_alpha(item,
                                            strtof((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to"), NULL));
                    }
                    else if(!g_strcmp0(attribute_value, "stroke-opacity"))
                    {
                        ella_item_set_alpha_stroke(item,
                                            strtof((char*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to"), NULL));
                    }
                    else if(!g_strcmp0(attribute_value, "transform"))
                    {
                        attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"type");
                        if(!g_strcmp0(attribute_value, "rotate"))
                        {
                            ella_item_set_rotation(item, value * M_PI / 180);
                        }
                        else if(!g_strcmp0(attribute_value, "scale"))
                        {
                            attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to");
                            ella_files_parse_scale_for_item(item, attribute_value);
                        }
                        else if(!g_strcmp0(attribute_value, "translate"))
                        {
                            attribute_value = (gchar*)xmlTextReaderGetAttribute(svg_xml, (xmlChar*)"to");
                            ella_files_parse_tr_for_item(item, attribute_value);
                        }
                    }
                }
            }
            ret = xmlTextReaderRead(svg_xml);
        }
    }

    else
    {
        g_error("Couldn't load this file: %s", path_svg);
    }
    setlocale(LC_ALL, "");
    return svg_informations;
}

