/* Copyright (C) 2010 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */

#ifndef H_ELLA_FILES_21122010
#define H_ELLA_FILES_21122010

#include <stdio.h>
#include <glib.h>
#include <pango/pangofc-font.h>
#include <gdk/gdk.h>
#include <locale.h>
#include <libintl.h>
#include "ellacore-common.h"
#include "ella-color.h"

#include <math.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>

/**
 * \brief Open a file to save, Ella's file format.
 **/
FILE* ella_files_open_ella_file(const gchar* path);

/**
 * \brief Open a file to read, Ella's file format.
 **/
FILE* ella_files_open_ella_file_read(const gchar* path);

/**
  * \brief Copy a file
  * \param source The path of the files
  * \param destination The path of the copied file
  * \return If copy is successful, it returns 0, otherwise it returns 1 (cannot
  *         open source file), 2 (cannot open destination file) or 3 (reading
  *         error).
  **/
int ella_files_copy(const gchar* source, const gchar* destination);

/**
 * \brief Find the 'real' file font for a font name and size
 * \param font A gchar like this pattern 'sans 12' 
 * \return A gchar with a path to the file
 **/
gchar* ella_files_get_font_file(const gchar* font);


typedef struct
{
    gdouble width;
    gdouble height;
    gint movie_frequency;
    EllaColor background_color;
    EllaColor stroke_color;
    EllaColor fill_color;
    gint version;
    gint fps;
    GSList* layers;
} EllaFilesData;

#define LONG_STRING 64000

EllaFilesData* ella_files_parse_ella_file(const gchar* path_descriptor);

gboolean ella_files_write_ella_file(EllaFilesData* file_data, const gchar* path);

#endif
