/* Copyright (C) 2010 Lucas Baudin
*
* This file is part of Ella Project.
*
* Ella Project is free software : you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Ella Project is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY ; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
*/
#include "ella-math.h"

gdouble editor_compute_angle(gdouble x_o,    gdouble y_o,
                             gdouble x_from, gdouble y_from,
                             gdouble x_to,   gdouble y_to)
{
    double size_from = 0.0;
    double size_to   = 0.0;
    double x_from_on_radius = 0.0; 
    double y_from_on_radius = 0.0;
    double x_to_on_radius = 0.0;
    double y_to_on_radius = 0.0;
    double angle_from = 0.0;
    double angle_to   = 0.0;

    if((x_o == x_from && y_o == y_from) || (x_o == x_to && y_o == y_to))
        return -1;

    x_from -= x_o;
    x_to   -= x_o;
    y_from -= y_o;
    y_to   -= y_o;

    /* First, let's move x_from to a radius of 1: */
    size_from = sqrt(pow(x_from, 2) + pow(y_from, 2));
    x_from_on_radius = x_from/size_from;
    y_from_on_radius = y_from/size_from;

    size_to = sqrt(pow(x_to, 2) + pow(y_to, 2));
    x_to_on_radius = x_to/size_to;
    y_to_on_radius = y_to/size_to;

    angle_from = asin(y_from_on_radius);
    angle_to = asin(y_to_on_radius);

    if(x_to < 0)
    {
        angle_to = M_PI - angle_to;
    }
    if(x_from < 0)
    {
        angle_from = M_PI - angle_from;
    }

    /*if(x_from < 0)
    {
        angle_from = angle_from + M_PI;
    }
    
    if(x_to < 0)
    {
        angle_to = angle_to + M_PI;
    }*/
    return angle_to - angle_from;
    
}

void ella_utils_sort_coords(gdouble* x_from, gdouble* y_from, gdouble* x_to, gdouble* y_to)
{
    if(*x_to < *x_from)
    {
        gdouble t = *x_to;
        *x_to      = *x_from;
        *x_from    = t;
    }
    if(*y_to < *y_from)
    {
        gdouble t = *y_to;
        *y_to      = *y_from;
        *y_from    = t;
    }
}
