/* Copyright (C) 2010 Lucas Baudin
*
* This file is part of Ella Project.
*
* Ella Project is free software : you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Ella Project is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY ; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
*/
#ifndef H_CONFIG_20101130
#define H_CONFIG_20101130

#include <glib.h>
#include <gtk/gtk.h>
#include <stdio.h>

#include "ella-dirs.h"

/** Add a new key to the user_config, with this function, you can only add a
 * string, see ella_add_key_* for the others type. This function (or one of the
 * others 'add_key' must be called at the start, to be sure we don't request a
 * key which does't exist.
 * So, at the start, we must add the keys :
 * ella_add_key("NEW_PROPERTY", "default_value"); */
void ella_add_key(const gchar* field, const gchar* value);

/** It is the same as ella_add_key, but it adds a double instead a string (very
 * useful for sizes, for instance). */
void ella_add_key_double(const gchar* field, double value);

/** It is the same as ella_add_key, but it adds a double instead a string (very
 * useful for sizes, for instance). */
void ella_add_key_int(const gchar* field, int value);

/** Load the user configuration file, it is called in main(), when Ella starts. */
void ella_load_config(void);

/** Save the user configuration, it is called in main(); when Ella stops. */
void ella_save_config(void);

/** Set the value of a (string) key. 'field' is the key name, and value is it
 * new value. */
void ella_config_set_value(const gchar* field, gchar* value);

/** Set the value of a (int) key. 'field' is the key name, and value is it
 * new value. */
void ella_config_set_value_int(const gchar* field, int value);

/** Set the value of a (double) key. 'field' is the key name, and value is it
 * new value. */
void ella_config_set_value_double(const gchar* field, double value);

/** Get the value of a (double) key. 'string' is the key name. */
double ella_config_get_value_double(const gchar* string);

/** Get the value of a (int) key. 'string' is the key name. */
int ella_config_get_value_int(const gchar* string);

/** Get the value of a (string) key. 'string' is the key name. */
gchar* ella_config_get_value(const gchar* string);

GdkColor* ella_config_get_gtk_color(GtkWidget* widget);

gdouble ella_config_get_gtk_color_r(GtkWidget* widget);
gdouble ella_config_get_gtk_color_g(GtkWidget* widget);
gdouble ella_config_get_gtk_color_b(GtkWidget* widget);

gdouble ella_config_get_gtk_color_bg_r(GtkWidget* widget);
gdouble ella_config_get_gtk_color_bg_g(GtkWidget* widget);
gdouble ella_config_get_gtk_color_bg_b(GtkWidget* widget);

gdouble ella_config_get_gtk_color_fg_r(GtkWidget* widget);
gdouble ella_config_get_gtk_color_fg_g(GtkWidget* widget);
gdouble ella_config_get_gtk_color_fg_b(GtkWidget* widget);

#endif
