/* Copyright (C) 2010 Sylvain Laperche
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */
#include "ella-files-tests.h"
static gboolean ella_file_fatal_handler(const gchar* log_domain,
                              GLogLevelFlags log_level,
                              const gchar* message,
                              gpointer user_data)
{
    return FALSE;
}

#define E_ROUND(obj) roundf(obj*100)/100

static void test_rectangle_read(void);
static void test_rectangle(void)
{
    EllaFilesData* file_data = NULL;
    EllaLayer* layer;
    EllaItem* item;

    /* Write svg file */
    file_data = g_malloc(sizeof(EllaFilesData));
    file_data->width = 800;
    file_data->height = 600;
    file_data->fps = 10;
    file_data->layers = NULL;

    layer = ella_layer_new();

    item = ELLA_ITEM(ella_rectangle_new());
    ella_layer_add_item(layer, item);

    ella_layer_add_keyframe(layer, 12);

    ella_item_change_keyframe(item, 0);
    ella_item_set_width(item, 100);
    ella_item_set_height(item, 200);
    ella_item_set_line_width(item, 2.5);
    ella_item_set_red(item, 0.6);
    ella_item_set_red_stroke(item, 1.0);
    ella_item_set_alpha(item, 0.4);
    ella_item_set_rotation(item, M_PI/2);

    ella_item_change_keyframe(item, 12);
    ella_item_set_width(item, 123);
    ella_item_set_height(item, 100);
    
    ella_item_set_red(item, 1);
    ella_item_set_green(item, 1);
    ella_item_set_blue(item, 1);
    ella_item_set_alpha(item, 0.80);
    ella_item_set_alpha_stroke(item, 0.55);

    ella_layer_add_keyframe(layer, 50);
    ella_item_change_keyframe(item, 50);
    ella_item_set_width(item, 100);
    ella_item_set_height(item, 200);
    ella_item_set_rotation(item, M_PI);
    ella_item_set_alpha(item, 1);

    item = ELLA_ITEM(ella_rectangle_new());
    ella_layer_add_item(layer, item);
    
    file_data->layers = g_slist_append(file_data->layers, layer);
    g_assert(ella_files_write_ella_file(file_data, "/tmp/test-ella-svg-rect.esvg") == TRUE);
    test_rectangle_read();
}
static void test_rectangle_read(void)
{
    EllaLayer* layer;
    EllaItem* item;
    EllaFilesData* file_read = ella_files_parse_ella_file("/tmp/test-ella-svg-rect.esvg");
    g_assert(file_read != NULL);

    g_assert_cmpfloat(file_read->fps, ==, 10);
    layer = (EllaLayer*)g_slist_nth_data(file_read->layers, 0);

    item = ella_layer_get_item(layer, 0);
    g_assert(item != NULL);

    ella_item_change_keyframe(item, 0);
    g_assert_cmpint(ella_item_get_width(item), ==, 100);
    g_assert_cmpint(ella_item_get_height(item), ==, 200);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, M_PI/2);
    g_assert_cmpfloat(ella_item_get_line_width(item), ==, 2.5);
    g_assert_cmpfloat(E_ROUND(ella_item_get_red(item)), ==, E_ROUND(0.6));
    g_assert_cmpfloat(E_ROUND(ella_item_get_alpha(item)), ==, E_ROUND(0.4));

    /* Width/height animation */
    ella_item_change_keyframe(item, 12);
    g_assert_cmpfloat(ella_item_get_width(item), ==, 123);
    g_assert_cmpint(ella_item_get_height(item), ==, 100);
    g_assert_cmpfloat(E_ROUND(ella_item_get_alpha(item)), ==, E_ROUND(0.80));
    g_assert_cmpfloat(E_ROUND(ella_item_get_alpha_stroke(item)), ==, E_ROUND(0.55));

    /* Width/height interpolation, and rotation */
    ella_item_change_keyframe(item, 50);
    g_assert_cmpint(ella_item_get_width(item), ==, 100);
    g_assert_cmpint(ella_item_get_height(item), ==, 200);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, M_PI);
    g_assert_cmpfloat(ella_item_get_red(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_green(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_blue(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_alpha(item), ==, 1);
}

void ella_files_tests(void)
{
    g_type_init();
    FILE* save  = NULL;
    FILE*  fp   = NULL;
    gchar* file = NULL;
    gchar* src  = NULL;
    gchar* src_content = NULL;
    gchar* dst_content = NULL;
    EllaFilesData* file_data = NULL;
    EllaLayer* layer;
    EllaItem* item;

    /* Tests for ella_files_open_ella_file(). */
    /* NULL pointer. */
    g_assert(ella_files_open_ella_file(NULL) == NULL);
    /* Empty string. */
    g_assert(ella_files_open_ella_file("") == NULL);
    /* Normal case. */
    save = ella_files_open_ella_file("foobar");
    g_assert(save != NULL);

    fclose(save);
    unlink("foobar");

    /* Tests for ella_files_open_ella_file_read(). */
    file = g_build_filename(ella_get_data_dir(), "test_for_ella-file", NULL);

    /* NULL pointer. */
    g_assert(ella_files_open_ella_file_read(NULL) == NULL);
    /* Empty string. */
    g_assert(ella_files_open_ella_file_read("") == NULL);
    /* Bad filename. */
    g_assert(ella_files_open_ella_file_read("foobar") == NULL);
    /* Normal case. */
    fp = ella_files_open_ella_file_read(file);
    g_assert(fp != NULL);

    fclose(fp);
    g_free(file);




    /* Tests for ella_files_copy(). */
    src = g_build_filename(ella_get_data_dir(), "src_for_ella-file", NULL);

    /* NULL pointer. */
    g_assert_cmpint(ella_files_copy(NULL, ""),  ==, 1);
    g_assert_cmpint(ella_files_copy(src, NULL), ==, 2);
    /* Empty string. */
    g_assert_cmpint(ella_files_copy("", "bar"), ==, 1);
    g_assert_cmpint(ella_files_copy(src, ""),   ==, 2);
    /* Bad filename. */
    g_assert_cmpint(ella_files_copy("foo", "bar"), ==, 1);
    /* Normal case. */
    g_assert_cmpint(ella_files_copy(src, "bar"), ==, 0);

    g_file_get_contents(src,   &src_content, NULL, NULL);
    g_file_get_contents("bar", &dst_content, NULL, NULL);
    g_assert_cmpstr(src_content, ==, dst_content);

    g_free(dst_content);
    g_free(src_content);
    unlink("bar");
    g_free(src);
    /* Tests for ella_files_get_font_file(). */
    g_test_log_set_fatal_handler(ella_file_fatal_handler, NULL);
    if (g_test_trap_fork(0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR))
    {
        /* NULL pointer. */
        g_assert(ella_files_get_font_file(NULL) != NULL);
        /* Empty string. */
        g_assert(ella_files_get_font_file("") != NULL);
        /* Bad filename. */
        g_assert(ella_files_get_font_file("hcdsvbgfmoj") != NULL);
        /* Normal case. */
        g_assert(ella_files_get_font_file("sans 12") != NULL);
        exit(0);
    }
    g_test_trap_assert_passed();

    test_rectangle();

    // CSS tests
    item = ELLA_ITEM(ella_rectangle_new());
    g_assert(ella_files_parse_css_for_item(item, "fill:#00ffff;") == TRUE);
    g_assert_cmpfloat(ella_item_get_red(item), ==, 0);
    g_assert_cmpfloat(ella_item_get_green(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_blue(item), ==, 1);
    g_assert(ella_files_parse_css_for_item(item, "fill:#ff7fff;") == TRUE);
    g_assert_cmpfloat(ella_item_get_red(item), ==, 1.0);
    g_assert_cmpfloat(ella_item_get_green(item), ==, ((double)0x7F)/(double)0xFF);
    g_assert_cmpfloat(ella_item_get_blue(item), ==, 1.0);
    g_assert(ella_files_parse_css_for_item(item, "fill:#00ffff;") == TRUE);
    g_assert_cmpfloat(ella_item_get_red(item), !=, 1);
    g_assert_cmpfloat(ella_item_get_green(item), !=, 0);
    g_assert_cmpfloat(ella_item_get_blue(item), !=, 0);
    g_assert(ella_files_parse_css_for_item(item, "stroke:#00ffff;") == TRUE);
    g_assert_cmpfloat(ella_item_get_red_stroke(item), !=, 1);
    g_assert_cmpfloat(ella_item_get_green_stroke(item), !=, 0);
    g_assert_cmpfloat(ella_item_get_blue_stroke(item), !=, 0);

    g_assert(ella_files_parse_transform_for_item(item, "rotate(90)") == TRUE);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, M_PI/2);

    file_data = ella_files_parse_ella_file(g_build_filename(ella_get_data_dir(),
                                                             "tests",
                                                             "test.svg",
                                                             NULL));
    g_assert(file_data != NULL);
    g_assert_cmpfloat(file_data->width, ==, 550);
    g_assert_cmpfloat(file_data->height, ==, 440);

    g_assert_cmpfloat(file_data->background_color.red, ==, 0);
    g_assert_cmpfloat(file_data->background_color.green, ==, 0);
    g_assert_cmpfloat(file_data->background_color.blue, ==, 1);
    g_assert_cmpfloat(file_data->background_color.alpha, ==, 0);
    g_assert_cmpfloat(file_data->fps, ==, 24);

    layer = (EllaLayer*)g_slist_nth_data(file_data->layers, 0);
    g_assert_cmpint(ella_layer_get_visible(layer), ==, TRUE);
    g_assert_cmpstr(ella_layer_get_id(layer), ==, "layer1");
    g_assert_cmpint(ella_layer_get_max_frames(layer), ==, 48);
#if 0
    g_assert_cmpint(ella_layer_get_nb_item(layer), ==, 2);
#endif
    item = ella_layer_get_item(layer, 0);
    g_assert(item != NULL);
    g_assert_cmpint(ella_item_get_width(item), ==, 200);
    g_assert_cmpint(ella_item_get_height(item), ==, 800);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, M_PI/2);
    g_assert_cmpfloat(ella_item_get_line_width(item), ==, 10);
    g_assert_cmpfloat(ella_item_get_red(item), ==, 1.0);
    g_assert_cmpfloat(ella_item_get_green(item), ==, 0.0);
    g_assert_cmpfloat(ella_item_get_blue(item), ==, 1.0);
    g_assert_cmpfloat(ella_item_get_alpha(item), ==, 0.5);
    g_assert_cmpfloat(ella_item_get_font_size(item), ==, 1);


    item = ella_layer_get_item(layer, 1);
    g_assert(item != NULL);
    g_assert_cmpfloat(ella_item_get_width(item), ==, 0.5);
    g_assert_cmpfloat(ella_item_get_height(item), ==, 0.5);
    g_assert_cmpfloat(ella_item_get_x(item), ==, 100);
    g_assert_cmpfloat(ella_item_get_y(item), ==, 200);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, 0);
    g_assert_cmpfloat(ella_item_get_red(item), ==, 0.0);
    g_assert_cmpfloat(ella_item_get_green(item), ==, 0.0);
    g_assert_cmpfloat(ella_item_get_blue(item), ==, 0.0);
    g_assert_cmpfloat(ella_item_get_alpha(item), ==, 1);
    g_assert_cmpstr(ella_item_get_text(item), ==, "text");
    g_assert_cmpfloat(ella_item_get_font_size(item), ==, 70);

    layer = (EllaLayer*)g_slist_nth_data(file_data->layers, 1);
    g_assert(layer != NULL);
    g_assert_cmpint(ella_layer_get_visible(layer), ==, FALSE);
    g_assert_cmpstr(ella_layer_get_id(layer), ==, "layer2");
#if 0
    g_assert_cmpint(ella_layer_get_max_frames(layer), ==, 10);
#endif

    /* There are only two layers in our descriptor.lst */
    layer = g_slist_nth_data(file_data->layers, 2);
    g_assert(layer == NULL);

    /* Colors */
    EllaColor color;
    color.red = 1;
    color.green = 0;
    color.blue = 1;
    g_assert_cmpstr(ella_color_to_string(&color), ==, "#ff00ff");
    color.red = 0.5;
    color.green = 0.5;
    color.blue = 0.5;
    g_assert_cmpstr(ella_color_to_string(&color), ==, "#7f7f7f");

    /* Write svg file */
    file_data = g_malloc(sizeof(EllaFilesData));
    file_data->width = 800;
    file_data->height = 600;
    file_data->fps = 24;
    file_data->background_color.red = 0.45;
    file_data->background_color.green = 0;
    file_data->background_color.blue = 1;
    /* FIXME: it shouldn't be 0 */
    file_data->background_color.alpha = 0;
    layer = ella_layer_new();

    item = ELLA_ITEM(ella_rectangle_new());
    ella_item_set_width(item, 100);
    ella_item_set_height(item, 200);
    ella_item_set_line_width(item, 2.5);
    ella_layer_add_item(layer, item);
    ella_item_set_red(item, 0.5);
    ella_item_set_red_stroke(item, 1.0);
    ella_item_set_rotation(item, M_PI/2);

    ella_item_add_keyframe(item, 25);
    ella_item_change_keyframe(item, 25);
    ella_item_set_width(item, 123);
    ella_item_set_height(item, 100);
    
    ella_item_set_red(item, 1);
    ella_item_set_green(item, 1);
    ella_item_set_blue(item, 1);
    ella_item_set_alpha(item, 1);

    ella_item_add_keyframe(item, 50);
    ella_item_change_keyframe(item, 50);
    ella_item_set_width(item, 100);
    ella_item_set_height(item, 200);
    ella_item_set_rotation(item, M_PI);

    item = ELLA_ITEM(ella_circle_new());
    ella_item_set_width(item, 150);
    ella_item_set_height(item, 250);
    ella_item_set_line_width(item, 3.5);
    ella_layer_add_item(layer, item);


    ella_item_set_x(item, 300);
    ella_item_set_y(item, 400);

    ella_item_add_keyframe(item, 25);
    ella_item_change_keyframe(item, 25);
    ella_item_set_x(item, 100);
    ella_item_set_y(item, 200);

    ella_layer_set_id(layer, "layer2");
    file_data->layers = NULL;
    file_data->layers = g_slist_append(file_data->layers, layer);

    item = ELLA_ITEM(ella_text_new());
    ella_item_set_width(item, 2);
    ella_item_set_height(item, 2);
    ella_item_set_text(item, "Yeah!");
    ella_layer_add_item(layer, item);

    item = ELLA_ITEM(ella_image_new());
    ella_image_set_image(item, g_build_filename(ella_get_data_dir(), "ui/icons/logo.png", NULL));
    ella_item_set_width(item, 2);
    ella_item_set_height(item, 3);
    g_assert_cmpfloat(ella_image_get_image_height(item), ==, 150);
    ella_layer_add_item(layer, item);

    item = ELLA_ITEM(ella_path_new());
    ella_layer_add_item(layer, item);
    ella_path_add_point(ELLA_PATH(item), 10, 10);
    ella_path_add_point(ELLA_PATH(item), 100, 100);
    ella_path_add_point(ELLA_PATH(item), 0, 35);

    layer = ella_layer_new();
    ella_layer_set_visible(layer, FALSE);
    ella_layer_set_id(layer, "layer3");

    file_data->layers = g_slist_append(file_data->layers, layer);
    g_assert(ella_files_write_ella_file(file_data, "/tmp/test-ella-svg.esvg") == TRUE);


    
    
    EllaFilesData* file_read = ella_files_parse_ella_file("/tmp/test-ella-svg.esvg");
    g_assert(file_read != NULL);
    g_assert_cmpfloat(file_read->width, ==, file_data->width);
    g_assert_cmpfloat(file_read->height, ==, file_data->height);

    g_assert_cmpfloat(E_ROUND(file_read->background_color.red), ==, E_ROUND(file_data->background_color.red));
    g_assert_cmpfloat(file_read->background_color.green, ==, file_data->background_color.green);
    g_assert_cmpfloat(file_read->background_color.blue, ==, file_data->background_color.blue);
    g_assert_cmpfloat(file_read->background_color.alpha, ==, file_data->background_color.alpha);
    g_assert_cmpfloat(file_read->fps, ==, file_data->fps);

    layer = (EllaLayer*)g_slist_nth_data(file_read->layers, 0);
    g_assert_cmpint(ella_layer_get_visible(layer), ==, TRUE);
    g_assert_cmpstr(ella_layer_get_id(layer), ==, "layer2");

    layer = (EllaLayer*)g_slist_nth_data(file_read->layers, 1);
    g_assert_cmpint(ella_layer_get_visible(layer), ==, FALSE);
    g_assert_cmpstr(ella_layer_get_id(layer), ==, "layer3");

    layer = (EllaLayer*)g_slist_nth_data(file_read->layers, 0);
    g_assert_cmpint(ella_layer_get_nb_item(layer), ==, 5);

    item = ella_layer_get_item(layer, 0);
    g_assert(item != NULL);
    ella_item_change_keyframe(item, 0);
    g_assert_cmpint(ella_item_get_width(item), ==, 100);
    g_assert_cmpint(ella_item_get_height(item), ==, 200);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, M_PI/2);
    g_assert_cmpfloat(ella_item_get_line_width(item), ==, 2.5);
    g_assert_cmpfloat(roundf(ella_item_get_red(item)*100)/100, ==, 0.5);
    g_assert_cmpfloat(ella_item_get_red(item), !=, 1.0);

    /* Width/height animation */
    ella_item_change_keyframe(item, 25);
    g_assert_cmpint(ella_item_get_width(item), ==, 123);
    g_assert_cmpint(ella_item_get_height(item), ==, 100);

    /* Width/height interpolation, and rotation */
    ella_item_change_keyframe(item, 50);
    g_assert_cmpint(ella_item_get_width(item), ==, 100);
    g_assert_cmpint(ella_item_get_height(item), ==, 200);
    g_assert_cmpfloat(ella_item_get_rotation(item), ==, M_PI);
    g_assert_cmpfloat(ella_item_get_red(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_green(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_blue(item), ==, 1);
    g_assert_cmpfloat(ella_item_get_alpha(item), ==, 1);

    /* the circle */
    item = ella_layer_get_item(layer, 1);
    g_assert(item != NULL);
    g_assert_cmpint(ella_item_get_width(item), ==, 150);
    g_assert_cmpint(ella_item_get_height(item), ==, 250);
    g_assert_cmpfloat(ella_item_get_line_width(item), ==, 3.5);
    
    /* x and y animations */
    ella_item_change_keyframe(item, 0);
    g_assert_cmpint(ella_item_get_x(item), ==, 300);
    g_assert_cmpint(ella_item_get_y(item), ==, 400);

    ella_item_change_keyframe(item, 25);
    g_assert_cmpint(ella_item_get_x(item), ==, 100);
    g_assert_cmpint(ella_item_get_y(item), ==, 200);

    /* the text */
    item = ella_layer_get_item(layer, 2);
    g_assert(item != NULL);
    g_assert_cmpint(ella_item_get_width(item), ==, 2);
    g_assert_cmpint(ella_item_get_height(item), ==, 2);
    g_assert_cmpstr(ella_item_get_text(item), ==, "Yeah!");

    /* the image */
    item = ella_layer_get_item(layer, 3);
    g_assert(item != NULL);
    g_assert_cmpint(ella_item_get_width(item), ==, 2);
    g_assert_cmpint(ella_item_get_height(item), ==, 3);
    g_assert_cmpint(ella_image_get_image_width(item), ==, 150);
    g_assert_cmpint(ella_image_get_image_height(item), ==, 150);
}
