/* Copyright (C) 2010 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */

#include "ella-math-tests.h"
#include "ella-dirs-tests.h"
#include "ella-files-tests.h"
#include "ella-print.h"
#include "ella-fonts.h"
#include <stdlib.h>

static void ella_print_tests(void)
{
    if (g_test_trap_fork(0, 0))
    {
        ella_debug("something");
        exit(0);
    }
    g_test_trap_assert_passed();
#ifdef ELLA_DEBUG
    g_test_trap_assert_stdout("*something*");
#else
    g_test_trap_assert_stdout("");
#endif
}

static void ella_undo_tests(void)
{
/*    EllaUndoManager* manager = ella_undo_manager_new();
    EllaUndoEvent* event = ella_undo_event_new();
    EllaUndoEvent* event2 = ella_undo_event_new();
    g_assert(manager != NULL);

    ella_undo_manager_register(manager, event);
    ella_undo_manager_register(manager, event2);
    ella_undo_manager_back(manager);

    g_assert(ella_undo_manager_get_current_event(manager) == event2);
    ella_undo_manager_back(manager);
    g_assert(ella_undo_manager_get_current_event(manager) == event);
    ella_undo_manager_back(manager);
    g_assert(ella_undo_manager_get_current_event(manager) == event);
    ella_undo_manager_forward(manager);
    g_assert(ella_undo_manager_get_current_event(manager) == event2);
*/}

static void ella_layers_tests(void)
{
    EllaLayer* layer = ella_layer_new();
    EllaItem* item = ELLA_ITEM(ella_rectangle_new());
    ella_layer_add_item(layer, item);
    EllaItem* item_ = ella_layer_get_item(layer, 0);
    g_assert(item == item_);
    EllaItem* item2 = ELLA_ITEM(ella_rectangle_new());
    ella_layer_add_item(layer, item2);
    item_ = ella_layer_get_item(layer, 0);
    g_assert(item_ != item2);
    item_ = ella_layer_get_item(layer, 1);
    g_assert(item_ == item2);
    ella_layer_item_up(layer, 0);
    item_ = ella_layer_get_item(layer, 0);
    g_assert(item_ == item2);
    item_ = ella_layer_get_item(layer, 1);
    g_assert(item_ == item);

    g_assert(ella_layer_item_up_instance(layer, item2) == TRUE);
    item_ = ella_layer_get_item(layer, 1);
    g_assert(item_ == item2);
}

static void ella_items_tests(void)
{
    gboolean succes;
    return;
    EllaItem* item = ELLA_ITEM(ella_rectangle_new());
    g_assert(item != NULL);
    ella_item_set_width(item, 100.0);
    g_assert_cmpfloat(ella_item_get_width(item), ==, 100.0);
    ella_item_add_keyframe(item, 5);
    succes = ella_item_change_keyframe(item, 5);
    g_assert_cmpint(succes, ==, TRUE);
    ella_item_set_width(item, 200.0);
    g_assert_cmpfloat(ella_item_get_width(item), ==, 200.0);
    succes = ella_item_change_keyframe(item, 2);
    g_assert_cmpint(succes, ==, FALSE);
    g_assert_cmpfloat(ella_item_get_period(item), ==, 5.0);
    g_assert_cmpfloat(ella_item_get_width(item), ==, 140.0);
    succes = ella_item_change_keyframe(item, 3);
    g_assert_cmpfloat(ella_item_get_width(item), ==, 160.0);
}

int main(int argc, char* argv[])
{
    g_test_init(&argc, &argv, NULL);
    gdk_init(&argc, &argv);
    g_test_add_func("/Ella Core/Ella Math", ella_maths_tests_compute_angle);
    g_test_add_func("/Ella Core/Ella Dirs", ella_dirs_tests);
    g_test_add_func("/Ella Core/Ella Files", ella_files_tests);
    g_test_add_func("/Ella Core/Ella Print", ella_print_tests);
    g_test_add_func("/Ella Core/Ella Items", ella_items_tests);
    g_test_add_func("/Ella Core/Ella Layers", ella_layers_tests);
    g_test_add_func("/Ella Core/Ella Undo", ella_undo_tests);

    return g_test_run();
}
