/* Copyright (C) 2010 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */
 
#include "ella-color_selection.h"

enum
{
    VALUE_CHANGED,
    LAST_SIGNAL
};

G_DEFINE_TYPE(EllaColorSelection, ella_color_selection, GTK_TYPE_HBOX);

static guint ella_color_selection_signals[LAST_SIGNAL] = { 0 };
static void ella_color_selection_class_init          (EllaColorSelectionClass* klass);
static void ella_color_selection_init                (EllaColorSelection*      ecs);
static void ella_color_selection_update_color_value  (EllaColorSelection*      ecs);

static void ella_color_selection_class_init (EllaColorSelectionClass *klass)
{
    GtkWidgetClass* widget_class;
    widget_class = GTK_WIDGET_CLASS(klass);

    /* Add the value-changed signal */
    ella_color_selection_signals[VALUE_CHANGED] = g_signal_new(
        "value-changed",
        G_OBJECT_CLASS_TYPE(widget_class),
        G_SIGNAL_RUN_FIRST,
        G_STRUCT_OFFSET(EllaColorSelectionClass, value_changed),
        NULL, NULL,
        g_cclosure_marshal_VOID__VOID,
        G_TYPE_NONE, 0);
}

static void ella_color_selection_value_changed(GtkWidget* widget, EllaColorSelection* ecs)
{
    if(ELLA_IS_COLOR_SELECTION_CHANNEL(widget))
    {
        EllaColorSelectionChannel* channel = ELLA_COLOR_SELECTION_CHANNEL(widget);
        gdouble red, green, blue;

        red   = ella_color_selection_channel_get_value_r(ecs->red);
        green = ella_color_selection_channel_get_value_g(ecs->green);
        blue  = ella_color_selection_channel_get_value_b(ecs->blue);

        switch(channel->channel)
        {
            case ELLA_COLOR_CHANNEL_RED:
                red = ella_color_selection_channel_get_value(channel);
                break;
            case ELLA_COLOR_CHANNEL_GREEN:
                green = ella_color_selection_channel_get_value(channel);
                break;
            case ELLA_COLOR_CHANNEL_BLUE:
                blue = ella_color_selection_channel_get_value(channel);
                break;
        }
        ella_color_selection_set_value_rgb(ecs, red, green, blue);
    }
    else
    {
        gdouble red   = gtk_spin_button_get_value_as_int(ecs->spin_red)  /255.0;
        gdouble green = gtk_spin_button_get_value_as_int(ecs->spin_green)/255.0;
        gdouble blue  = gtk_spin_button_get_value_as_int(ecs->spin_blue) /255.0;

        ella_color_selection_set_value_rgb(ecs, red, green, blue);
    }

    ella_color_selection_update_color_value(ecs);

    /* Emit the signal to say that the widget value changed. */
    g_signal_emit(ecs,
                  ella_color_selection_signals[VALUE_CHANGED],
                  0,
                  NULL);
}
static void ella_color_selection_value_alpha_changed(GtkWidget* widget, EllaColorSelection* ecs)
{
    /* Emit the signal to say that the widget value changed. */
    g_signal_emit(ecs,
                  ella_color_selection_signals[VALUE_CHANGED],
                  0,
                  NULL); 
}
static void ella_color_selection_init(EllaColorSelection *ecs)
{
    GtkWidget* vbox = gtk_vbox_new(FALSE, 2);

    GtkWidget* hbox_r = gtk_hbox_new(FALSE, 1);
    GtkWidget* hbox_g = gtk_hbox_new(FALSE, 1);
    GtkWidget* hbox_b = gtk_hbox_new(FALSE, 1);

    ecs->spin_red = GTK_SPIN_BUTTON(gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(255,0,255,1,1,0)), 1, 0));
    ecs->spin_green = GTK_SPIN_BUTTON(gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(255,0,255,1,1,0)), 1, 0));
    ecs->spin_blue = GTK_SPIN_BUTTON(gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(255,0,255,1,1,0)), 1, 0));

    ecs->red = ELLA_COLOR_SELECTION_CHANNEL(ella_color_selection_channel_new());
    ecs->blue = ELLA_COLOR_SELECTION_CHANNEL(ella_color_selection_channel_new());
    ecs->green = ELLA_COLOR_SELECTION_CHANNEL(ella_color_selection_channel_new());

    ella_color_selection_channel_set_channel(ecs->red, ELLA_COLOR_CHANNEL_RED);
    ella_color_selection_channel_set_channel(ecs->blue, ELLA_COLOR_CHANNEL_BLUE);
    ella_color_selection_channel_set_channel(ecs->green, ELLA_COLOR_CHANNEL_GREEN);

    gtk_box_pack_start(GTK_BOX(hbox_r),
                       GTK_WIDGET(ecs->red),
                       TRUE,
                       TRUE,
                       0);
    gtk_box_pack_start(GTK_BOX(hbox_g),
                       GTK_WIDGET(ecs->green),
                       TRUE,
                       TRUE,
                       0);
    gtk_box_pack_start(GTK_BOX(hbox_b),
                       GTK_WIDGET(ecs->blue),
                       TRUE,
                       TRUE,
                       0);

    gtk_box_pack_start(GTK_BOX(hbox_r),
                       GTK_WIDGET(ecs->spin_red),
                       FALSE,
                       FALSE,
                       0);
    gtk_box_pack_start(GTK_BOX(hbox_g),
                       GTK_WIDGET(ecs->spin_green),
                       FALSE,
                       FALSE,
                       0);
    gtk_box_pack_start(GTK_BOX(hbox_b),
                       GTK_WIDGET(ecs->spin_blue),
                       FALSE,
                       FALSE,
                       0);

    gtk_box_pack_start(GTK_BOX(vbox),
                       hbox_r,
                       TRUE,
                       TRUE,
                       0);
    gtk_box_pack_start(GTK_BOX(vbox),
                       hbox_g,
                       TRUE,
                       TRUE,
                       0);
    gtk_box_pack_start(GTK_BOX(vbox),
                       hbox_b,
                       TRUE,
                       TRUE,
                       0);

    /* Let's connect the signals of the gradient :) */
    g_signal_connect(ecs->red,
                     "value-changed",
                     G_CALLBACK(ella_color_selection_value_changed),
                     ecs);
    g_signal_connect(ecs->blue,
                     "value-changed",
                     G_CALLBACK(ella_color_selection_value_changed),
                     ecs);
    g_signal_connect(ecs->green,
                     "value-changed",
                     G_CALLBACK(ella_color_selection_value_changed),
                     ecs);

    /* Now the SpinButton signals. */
    g_signal_connect(ecs->spin_red,
                     "value-changed",
                     G_CALLBACK(ella_color_selection_value_changed),
                     ecs);
    g_signal_connect(ecs->spin_blue,
                     "value-changed",
                     G_CALLBACK(ella_color_selection_value_changed),
                     ecs);
    g_signal_connect(ecs->spin_green,
                     "value-changed",
                     G_CALLBACK(ella_color_selection_value_changed),
                     ecs);

    gtk_box_pack_start(GTK_BOX(ecs),
                       vbox,
                       TRUE,
                       TRUE,
                       0);

    ecs->alpha_adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(100,0,100,1,1,0));

    g_signal_connect(ecs->alpha_adjustment, "value-changed", G_CALLBACK(ella_color_selection_value_alpha_changed), ecs);

    GtkWidget* vscale = gtk_vscale_new(ecs->alpha_adjustment);
    gtk_scale_set_draw_value(GTK_SCALE(vscale), FALSE);

    gtk_widget_set_tooltip_text(vscale,
        _("Move the cursor to change the opacity level of the selected object(s)"));

    gtk_box_pack_start(GTK_BOX(ecs), vscale, FALSE, FALSE, 0);

    /* The gradient tooltips. */
    gtk_widget_set_tooltip_text(GTK_WIDGET(ecs->red),
            _("Red channel, click on the gradient to choose the red value"));
    gtk_widget_set_tooltip_text(GTK_WIDGET(ecs->green),
            _("Green channel, click on the gradient to choose the green value"));
    gtk_widget_set_tooltip_text(GTK_WIDGET(ecs->blue),
            _("Blue channel, click on the gradient to choose the blue value"));

    ecs->color_value = g_malloc(sizeof *(ecs->color_value));
}

GtkWidget* ella_color_selection_new(void)
{
    return GTK_WIDGET(g_object_new(ella_color_selection_get_type(), NULL));
}

GdkColor* ella_color_selection_get_value(EllaColorSelection* ecs)
{
    return ecs->color_value;
}

gdouble ella_color_selection_get_value_alpha(EllaColorSelection* ecs)
{
    return gtk_adjustment_get_value(ecs->alpha_adjustment);
}

static void ella_color_selection_update_color_value(EllaColorSelection* ecs)
{
    /* Cairo value are 0 -> 1.0, but gdk ones are 0 -> 0xFFFF. */
    ecs->color_value->red = ella_color_selection_channel_get_value_r(ecs->red)*0xFFFF;
    ecs->color_value->green = ella_color_selection_channel_get_value_g(ecs->green)*0xFFFF;
    ecs->color_value->blue = ella_color_selection_channel_get_value_b(ecs->blue)*0xFFFF;
}

/**
 * \brief Set the RGB value of the EllaColorSelection. The opacity level is not
 * changed, it is the same as the previous color. 
 * \param ecs A EllaColorSelection
 * \param red red value of the new color
 * \param green green value of the new color
 * \param blue blue value of the new color
 **/
void ella_color_selection_set_value_rgb(EllaColorSelection* ecs,
                                        gdouble red,
                                        gdouble green,
                                        gdouble blue)
{
    ella_color_selection_channel_set_value_r(ecs->red, red);
    ella_color_selection_channel_set_value_r(ecs->green, red);
    ella_color_selection_channel_set_value_r(ecs->blue, red);

    
    ella_color_selection_channel_set_value_g(ecs->red, green);
    ella_color_selection_channel_set_value_g(ecs->green, green);
    ella_color_selection_channel_set_value_g(ecs->blue, green);

    ella_color_selection_channel_set_value_b(ecs->red, blue);
    ella_color_selection_channel_set_value_b(ecs->green, blue);
    ella_color_selection_channel_set_value_b(ecs->blue, blue);

    gtk_spin_button_set_value(ecs->spin_red, red * 255);
    gtk_spin_button_set_value(ecs->spin_green, green * 255);
    gtk_spin_button_set_value(ecs->spin_blue, blue * 255);

    ella_color_selection_update_color_value(ecs);
}

/**
 * \brief Set the RGBA value of the EllaColorSelection.
 * \param ecs A EllaColorSelection
 * \param red red value of the new color
 * \param green green value of the new color
 * \param blue blue value of the new color
 * \param alpha alpha value if the new color
 **/
void ella_color_selection_set_value_rgba(EllaColorSelection* ecs,
                                         gdouble red,
                                         gdouble green,
                                         gdouble blue,
                                         gdouble alpha)
{
    ella_color_selection_set_value_rgb(ecs, red, green, blue);
    gtk_adjustment_set_value(ecs->alpha_adjustment, alpha * 100);
}
