/* Copyright (C) 2011 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */

int layer_height = 25;

/**
 * The timeline of Ella.
 *
 */
public class Ella.Timeline : Gtk.HBox
{
    TimelineLayer layers;
    TimelineFrame frames;
    TimelineRuler ruler;
    public signal void layer_changed();
    public signal void keyframe_added();
    Gtk.HScrollbar scrollbar;
    Gtk.DrawingArea hide_layer;

    public Timeline()
    {
        homogeneous = false;
        layers = new TimelineLayer();
        ruler = new TimelineRuler();
        height_request = 85;

        var scrolled_layers = new Gtk.Layout();
        var scrolled_frames = new Gtk.Layout();
        var scrolled_ruler = new Gtk.Layout();

        frames = new TimelineFrame(scrolled_frames, scrolled_ruler, scrolled_layers);
        scrolled_frames.put(frames, 0, 0);
        
        var vbox_layers = new Gtk.VBox(false, 0);
        
        hide_layer = new Gtk.DrawingArea();
        hide_layer.width_request = layers.width_request;
        hide_layer.height_request = layer_height*3/2;
        scrolled_layers.put(layers, 0, 0);
        scrolled_ruler.put(ruler, 0, 0);
        vbox_layers.pack_start(hide_layer, false, false);
        vbox_layers.pack_start(scrolled_layers, true, true);
        

        scrolled_layers.width_request = layers.width_request;

        scrolled_ruler.height_request = ruler.height_request;

        /* Connect asjutment */
        scrolled_ruler.hadjustment = scrolled_frames.get_hadjustment();
        scrolled_layers.vadjustment = scrolled_frames.get_vadjustment();
        scrollbar = new Gtk.HScrollbar(scrolled_frames.get_hadjustment());

        //vbox.pack_start(scrolled_layers, true, true);


        var vbox = new Gtk.VBox(false, 0);
        vbox.pack_start(scrolled_ruler, false, false);
        vbox.pack_start(scrolled_frames, true, true);
        vbox.pack_start(scrollbar, false, false);
    
        //scrolled.put(vbox, 0, 0);

        this.pack_start(vbox_layers, false);
        this.pack_start(vbox);
        this.pack_start(new Gtk.VScrollbar(scrolled_frames.get_vadjustment()), false);

        layers.layer_changed.connect(on_layer_changed);
        frames.keyframe_added.connect(on_keyframe_added);
        ruler.position_changed.connect(on_position_changed);
    }

    /**
     * Add a layer to the timeline layer list.
     **/
    public void add_layer(Ella.Layer layer)
    {
        frames.add_layer(layer);
        frames.compute_size_request();
        layers.add_layer(layer);
        layers.compute_size_request();
    }

    /**
     * Remove all layers from the timeline. The layer list is cleared.
     **/
    public void clear_all_layers()
    {
        frames.layers.clear();
        layers.layers.clear();
        layers.layer_selected = 0;
        ruler.position = 0;
    }

    private void on_layer_changed()
    {
        layer_changed();
    }

    private void on_keyframe_added()
    {
        ruler.width_request = frames.width_request;
        ruler.queue_draw();
        keyframe_added();
    }

    private void on_position_changed(int position)
    {
        frames.position = position;
        frames.queue_draw();
    }

    /**
     * Return the selected layer (the layer the user selected using the left
     * part of the timeline).
     **/
    public Ella.Layer get_selected_layer()
    {
        return layers.layers[layers.layer_selected];
    }

    /**
     * Get the TimelineLayer of the timeline. It is NOT the layers list.
     **/
    public TimelineLayer get_layers()
    {
        return layers;
    }
    public TimelineRuler get_ruler()
    {
        return ruler;
    }

    public void set_position(int time)
    {
        ruler.position = time;
        ruler.queue_draw();
        frames.position = time;
        frames.queue_draw();
    }

    public void layer_up(Ella.Layer layer)
    {
        int index = layers.layers.index_of(layer);
       
        if(index <= 0)
            return;

        layers.layer_selected -= 1;
        layers.layers[index] = layers.layers[index - 1];
        layers.layers[index - 1] = layer;
        layers.queue_draw();
        
        index = frames.layers.index_of(layer);
        
        frames.layers[index] = frames.layers[index - 1];
        frames.layers[index - 1] = layer;
        frames.queue_draw();
    }

    public void layer_down(Ella.Layer layer)
    {
        int index = layers.layers.index_of(layer);
       
        if(index >= layers.layers.size - 1)
            return;

        layers.layer_selected += 1;
        layers.layers[index] = layers.layers[index + 1];
        layers.layers[index + 1] = layer;
        layers.queue_draw();
        
        index = frames.layers.index_of(layer);
        
        frames.layers[index] = frames.layers[index + 1];
        frames.layers[index + 1] = layer;
        frames.queue_draw();
    }

    public void layer_remove(Ella.Layer layer)
    {
        if(layers.layers[layers.layer_selected] == layer)
            layers.layer_selected = 0;
        frames.layers.remove(layer);
        layers.layers.remove(layer);
        layers.queue_draw();
        frames.queue_draw();
    }

    public int image_per_secs
    {
        set { ruler.image_per_seconds = value; ruler.queue_draw(); }
    }
}

public class Ella.TimelineLayer : Gtk.DrawingArea
{

    public signal void layer_changed();
    public signal void layer_hidden(Ella.Layer layer);

    internal Gee.ArrayList<Layer> layers;
    internal int layer_selected = 0;
    double bg_r;
    double bg_g;
    double bg_b;
    double bg_sel_r;
    double bg_sel_g;
    double bg_sel_b;
    string gtk_font_name;
    Gtk.CheckButton check_on;

    public TimelineLayer()
    {
        add_events(Gdk.EventMask.BUTTON_PRESS_MASK
                  | Gdk.EventMask.BUTTON_RELEASE_MASK
                  | Gdk.EventMask.POINTER_MOTION_MASK
                  /*| Gdk.EventMask.KEY_PRESS_MASK
                  | Gdk.EventMask.KEY_RELEASE_MASK*/);
        width_request = 150;
        layers = new Gee.ArrayList<Layer>();
        bg_sel_r = Config.get_gtk_color_r(this);
        bg_sel_g = Config.get_gtk_color_g(this);
        bg_sel_b = Config.get_gtk_color_b(this);
        bg_r = Config.get_gtk_color_bg_r(this);
        bg_g = Config.get_gtk_color_bg_g(this);
        bg_b = Config.get_gtk_color_bg_b(this);
    
        var gtk_settings = Gtk.Settings.get_for_screen (get_screen ());
        gtk_settings.get ("gtk-font-name", out gtk_font_name);
        var font = Pango.FontDescription.from_string (gtk_font_name);
        gtk_font_name = font.get_family();
        
        check_on = new Gtk.CheckButton();
        compute_size_request();
        queue_draw();
    }

    internal void compute_size_request()
    {
        height_request = layer_height * layers.size;
    }
    public override bool button_press_event(Gdk.EventButton event)
    {
        int layer_to_select = (int) event.y / layer_height;
        if(layer_to_select < layers.size)
        {
            if(event.x > get_allocated_width() - layer_height)
            {
                layers[layer_to_select].visible = layers[layer_to_select].visible ? false : true;
                layer_hidden(layers[layer_to_select]);
            }
            else
            {
                layer_selected = layer_to_select;
                layer_changed();
            }
        }
        queue_draw();
        return true;
    }

    public override bool draw(Cairo.Context cr)
    {
        cr.select_font_face(gtk_font_name, Cairo.FontSlant.NORMAL, Cairo.FontWeight.NORMAL);
        cr.set_source_rgb(bg_r, bg_g, bg_b);
        cr.paint();
        draw_selection(cr);
        cr.set_font_size(layer_height/1.7);
        const double check_padding = 5;
        for(int i = 0; i < layers.size; i++)
        {
            var layer = layers[i];
            cr.set_source_rgb(0,0,0);
            cr.move_to(5, layer_height*(i + 1) - layer_height/4);
            cr.show_text(layer.id);
            Gtk.StyleContext style = check_on.get_style_context();
            style.set_state(Gtk.StateFlags.NORMAL);
            style.add_class("check");
            if(layer.visible)
            {
                style.set_state(Gtk.StateFlags.ACTIVE);
            }
            Gtk.render_check(style, cr,
                             get_allocated_width() - layer_height + check_padding, /* x */
                             layer_height*i + check_padding, /* y */
                             layer_height - check_padding*2, /* width */
                             layer_height - check_padding*2); /* height */
        }
        return true;
    }

    public void draw_selection(Cairo.Context cr)
    {
        if(layers.size > layer_selected)
        {
            cr.rectangle(0, layer_height*(layer_selected), get_allocated_width(), layer_height);
            cr.set_source_rgb(bg_sel_r, bg_sel_g, bg_sel_b);
            cr.fill();
        }
    }
    public void add_layer(Ella.Layer layer)
    {
        layers.add(layer);
        compute_size_request();
        queue_draw();
    }
}

public class Ella.TimelineFrame : Gtk.DrawingArea
{
    internal Gee.ArrayList<Layer> layers;
    int old_kf = -1;
    Layer kf_selected;
    Layer kf_hovered;
    double bg_sel_r;
    double bg_sel_g;
    double bg_sel_b;
    double bg_r;
    double bg_g;
    double bg_b;
    Gtk.Layout top;
    Gtk.Layout top2;
    Gtk.Layout top3;
    internal int position = 0;
    internal signal void keyframe_added();
    public TimelineFrame(Gtk.Layout top_, Gtk.Layout top2_, Gtk.Layout top3_)
    {
        add_events(Gdk.EventMask.BUTTON_PRESS_MASK
                  | Gdk.EventMask.BUTTON_RELEASE_MASK
                  | Gdk.EventMask.POINTER_MOTION_MASK
                  /*| Gdk.EventMask.KEY_PRESS_MASK
                  | Gdk.EventMask.KEY_RELEASE_MASK*/);
        width_request = 1000;
        layers = new Gee.ArrayList<Layer>();
        bg_sel_r = Config.get_gtk_color_r(this);
        bg_sel_g = Config.get_gtk_color_g(this);
        bg_sel_b = Config.get_gtk_color_b(this);
        bg_r = Config.get_gtk_color_bg_r(this);
        bg_g = Config.get_gtk_color_bg_g(this);
        bg_b = Config.get_gtk_color_bg_b(this);
        top = top_;
        top2 = top2_;
        top3 = top3_;
        compute_size_request();
    }

    public override bool scroll_event(Gdk.EventScroll event)
    {
        return true;
    }

    internal void compute_size_request()
    {
        height_request = layer_height * layers.size + 5;
        width_request = 2000;
        
        foreach(Layer layer in layers)
        {
            width_request = int.max(width_request, layer.max_frames * 5 + 5);
        }
        
        top.height = height_request;
        top.width = width_request;

        top2.width = width_request;

        top3.height = height_request + layer_height;
    }

    public override bool button_press_event(Gdk.EventButton event)
    {
        int layer_to_select = (int) event.y / layer_height;
        if(layer_to_select >= layers.size)
            return true;
        if(event.type == Gdk.EventType.2BUTTON_PRESS)
        {
            layers[layer_to_select].add_keyframe((int)event.x/5);
            keyframe_added();
        }
        if(event.type == Gdk.EventType.BUTTON_PRESS)
        {
            switch(event.button)
            {
            case 1:
                var layer = layers[layer_to_select];
                Gee.ArrayList<KeyframeLayer> kfs = layer.keyframes_list;
                for(int j = 0; j < kfs.size; j++) /* FIXME: foreach */
                {
                    if(kfs[j].time == (int) event.x / 5)
                    {
                        kf_selected = layer;
                        old_kf = kfs[j].time;
                        queue_draw();
                        return true;
                    }
                }
                break;
            case 3:
                var layer = layers[layer_to_select];
                layer.set_keyframe_type((int) event.x/5, KeyframeType.SMOOTH);
                break;
            }
        }
        queue_draw();
        return true;
    }

    public override bool button_release_event(Gdk.EventButton event)
    {
        kf_selected = null;
        queue_draw();
        return true;
    }

    public override bool motion_notify_event(Gdk.EventMotion event)
    {
        int layer_to_select = (int) event.y / layer_height;

        kf_hovered = null;
        if(kf_selected != null)
        {
            if(event.x < 0.0)
            {
                kf_selected.move_keyframe(old_kf, 0);
                old_kf = 0;
            }
            else
            {
                if(kf_selected.move_keyframe(old_kf, (int)event.x/5))
                    old_kf = (int)event.x/5;
            }
            compute_size_request();
        }
        else
        {
            if(layer_to_select < layers.size)
            {
                Gee.ArrayList<KeyframeLayer> kfs = layers[layer_to_select].keyframes_list;
                for(int j = 0; j < kfs.size; j++) /* FIXME: foreach ? */
                {
                    if(kfs[j].time == (int) event.x / 5)
                    {
                        kf_hovered = layers[layer_to_select];
                        old_kf = (int)event.x/5;
                        queue_draw();
                        return true;
                    }
                }
            }
        }
        queue_draw();
        return true;
    }

    public override bool draw(Cairo.Context cr)
    {
        cr.set_source_rgb(1, 1, 1);
        cr.paint();
        for(int i = 0; i < layers.size; i++)
        {
            cr.set_source_rgb(0.7, 0.7, 0.7);
            cr.move_to(0, layer_height*(i + 1));
            cr.line_to(get_allocated_width(), layer_height*(i+1));
            cr.set_line_width(0.5);
            cr.stroke();

            cr.set_source_rgb(0.85, 0.85, 0.85);
            cr.rectangle(0, layer_height*i, layers[i].max_frames*5, layer_height);
            cr.fill();
            Gee.ArrayList<KeyframeLayer> kfs = layers[i].keyframes_list;
            double old_fps_x = 0.0;
            foreach(var j in kfs)
            {
                switch(j.type)
                {
                case KeyframeType.NORMAL:
                    cr.rectangle(old_fps_x, layer_height*i + layer_height/2 - 2, j.time*5 - old_fps_x, 4);
                    cr.set_source_rgb(0,0,0);
                    cr.fill();
                    break;
                case KeyframeType.SMOOTH:
                    cr.move_to(old_fps_x, layer_height*(i+1));
                    double offset = (j.time*5 - old_fps_x)/3;
                    double x1 = j.time*5 - (j.time*5 - old_fps_x)/2 - offset;
                    double x2 = x1 + 2*offset;
                    double y1 = layer_height*i;
                    double y2 = y1;
                    cr.curve_to(x1, y1, x2, y2, j.time*5, layer_height*(i+1));
                    cr.set_line_width(2);
                    cr.set_source_rgb(0,0,0);
                    cr.stroke();
                    break;
                }
                cr.rectangle(j.time*5, layer_height*i, 5, layer_height);
                old_fps_x = j.time*5;
                if(j.time == old_kf && (layers[i] == kf_selected || layers[i] == kf_hovered) )
                {
                    cr.set_source_rgb(bg_sel_r, bg_sel_g, bg_sel_b);
                    cr.fill();
                    cr.set_source_rgb(0,0,0);
                }
                else
                {
                    cr.set_source_rgb(0,0,0);
                    cr.fill();
                }
            }
        }
        cr.set_source_rgb(bg_sel_r, bg_sel_g, bg_sel_b);
        cr.rectangle(position*5 + 1, 0, 3, get_allocated_height());
        cr.fill();
        return true;
    }
    public void add_layer(Ella.Layer layer)
    {
        layers.add(layer);
        compute_size_request();
        queue_draw();
    }
}
public class Ella.TimelineRuler : Gtk.DrawingArea
{
    public signal void position_changed(int position);
    internal int position = 0;
    bool pressed = false;
    double bg_r;
    double bg_g;
    double bg_b;
    double fg_r;
    double fg_g;
    double fg_b;
    double bg_sel_r;
    double bg_sel_g;
    double bg_sel_b;
    string gtk_font_name;
    internal int image_per_seconds = 24;
    public TimelineRuler()
    {
        add_events(Gdk.EventMask.BUTTON_PRESS_MASK
                  | Gdk.EventMask.BUTTON_RELEASE_MASK
                  | Gdk.EventMask.POINTER_MOTION_MASK
                  /*| Gdk.EventMask.KEY_PRESS_MASK
                  | Gdk.EventMask.KEY_RELEASE_MASK*/);
        height_request = layer_height*3/2;
        width_request = 2000;
        bg_r = Config.get_gtk_color_bg_r(this);
        bg_g = Config.get_gtk_color_bg_g(this);
        bg_b = Config.get_gtk_color_bg_b(this);
        fg_r = Config.get_gtk_color_fg_r(this);
        fg_g = Config.get_gtk_color_fg_g(this);
        fg_b = Config.get_gtk_color_fg_b(this);
        bg_sel_r = Config.get_gtk_color_r(this);
        bg_sel_g = Config.get_gtk_color_g(this);
        bg_sel_b = Config.get_gtk_color_b(this);
        
        var gtk_settings = Gtk.Settings.get_for_screen (get_screen ());
        gtk_settings.get ("gtk-font-name", out gtk_font_name);
        var font = Pango.FontDescription.from_string (gtk_font_name);
        gtk_font_name = font.get_family ();
    }

    public override bool button_press_event(Gdk.EventButton event)
    {
        position = (int) event.x / 5;
        position_changed(position);
        pressed = true;
        queue_draw();
        return true;
    }

    public override bool motion_notify_event(Gdk.EventMotion event)
    {
        if(event.x < 0.0)
        {
            position = (int) 0;
            position_changed(position);
            queue_draw();
            return true;
        }
        if(pressed)
        {
            position = (int) event.x / 5;
            position_changed(position);
            queue_draw();
        }
        return true;
    }

    public override bool button_release_event(Gdk.EventButton event)
    {
        pressed = false;
        queue_draw();
        return true;
    }

    public override bool draw(Cairo.Context cr)
    {
        cr.select_font_face(gtk_font_name, Cairo.FontSlant.NORMAL, Cairo.FontWeight.NORMAL);
        cr.set_source_rgb(bg_r,bg_g,bg_b);
        cr.paint();
        cr.set_font_size(11);
        cr.set_line_width(3);
        Cairo.TextExtents txt;
        cr.set_source_rgb(fg_r,fg_g,fg_b);
        for(int i = 0; i < get_allocated_width()/5; i+=5)
        {
            cr.text_extents(i.to_string(), out txt);
            cr.move_to(i*5 - txt.width/4 + 1, 22);
            cr.show_text((i).to_string());
            
            cr.rectangle(i* 5 + 1, layer_height*3/2-layer_height/2, 3, layer_height-layer_height/2);
            cr.fill();
        }
        
        cr.set_source_rgba(1, 0, 0, 0.5);
        for(int i = 0; i < get_allocated_width()/5; i+=image_per_seconds)
        {
            cr.text_extents(i.to_string(), out txt);
            cr.move_to(i*5 - txt.width/4 + 1, 10);
            cr.show_text((i/image_per_seconds).to_string() + "s");
            
            cr.rectangle(i* 5 + 1, layer_height*3/2-layer_height/2, 3, layer_height-layer_height/2);
            cr.fill();
        }
        
        cr.set_source_rgb(bg_sel_r, bg_sel_g, bg_sel_b);
        cr.rectangle(position * 5 + 1, layer_height-layer_height/2, 3, layer_height);
        cr.fill();
        return true;
    }
}
