/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "biblio.h"

/* icons */
#include "./icones_rcs/symbol.xpm"
#include "./icones_rcs/sound.xpm"
#include "./icones_rcs/dessin.xpm"
#include "./icones_rcs/image.xpm"
#include "./icones_rcs/flash24x24.xpm"
#include "./icones_rcs/button24x24.xpm"
/* variables globales */
/*GtkWidget *buttonBiblioRemove; obligatoirement externe ! et déclaré dans le .h */
GtkListStore *pBiblio;
GtkWidget *pListViewBiblio;/* permet d'afficher les objets */
gint BiblioItemsCount=0;/* compteur nombre objets utilisés */
gboolean f_biblio_already_load=FALSE;
/* Chargement des images non exportées */
GdkPixbuf *pIco_dessin;
GdkPixbuf *pIco_image;
GdkPixbuf *pIco_son;
GdkPixbuf *pIco_symbol;
GdkPixbuf *pIco_clip;
GdkPixbuf* pIco_button;
	
/***************************
 * drag n drop
 * ************************/
enum
    {
      BTARGET_STRING,
      BTARGET_URL,
	  BTARGET_TYPE_PIXMAP
    };

/* ci-dessous, 0 signifie aucune restriction
 * Cast to avoid warnings with -Wwrite-strings. */
static GtkTargetEntry Btargetentries[] =
    {
      {(char*)("STRING"),          0, BTARGET_STRING},
      {(char*)("text/plain"),      0, BTARGET_STRING},
      {(char*)("text/uri-list"),   0, BTARGET_URL},
	  {(char*)("image/x-xpixmap"), 0, BTARGET_TYPE_PIXMAP},
    };



/* forwards */
void add_tree_list_biblio(GtkListStore *pListStore,gint categorie,gchar *nom,gchar *path,gboolean actif);	

/*************************************
 * general error dialog
 * **********************************/
void biblio_dlg_misc_error(gchar *msg)
{
  GtkWidget *dialog;
  
  dialog = gtk_message_dialog_new(NULL,
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_MESSAGE_ERROR,
            GTK_BUTTONS_OK,
            "%s", msg);
  gtk_window_set_title(GTK_WINDOW(dialog), gettext("Error"));
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
}
/*************************************
 * function to set flag in order to
 * avoid double loading of the same
 * library file 
 * **********************************/
void biblio_set_library_load(gboolean flag)
{
	f_biblio_already_load=flag;
}
/**************************************
 * function to get the flag in order
 * to avoid double loading of the
 * same library
 * ***********************************/
gboolean  biblio_get_library_load(void)
{
	return f_biblio_already_load;
}
/*******************************
 fonction copie en binaire
 copie de source vers dest
*******************************/
int copier_fichier_biblio(char const * const source, char const * const destination)
{
    FILE* fSrc;
    FILE* fDest;
    char buffer[512];
    int NbLus;
    
    if ((fSrc = fopen(source, "rb")) == NULL)
      {
        return 1;
      }
    
    if ((fDest = fopen(destination, "wb")) == NULL)
      {
        fclose(fSrc);
        return 2;
      }    
    while ((NbLus = fread(buffer, 1, 512, fSrc)) != 0)
        fwrite(buffer, 1, NbLus, fDest);
    
    fclose(fDest);
    fclose(fSrc);
    
    return 0;
}
	
/**********************************
 * effacer tous les éléments de la
 * bibliothèque
 * *******************************/
void biblio_clear_items(GtkTreeView* pListView)
{
 GtkTreeModel* model = NULL;
 GtkTreePath* path;
 gchar* path1 = NULL;
 GtkTreeIter iter;
 gint count;
 
  if (BiblioItemsCount<=0)
       return;
  /*printf("BiblioItemsCount %d\n",BiblioItemsCount);*/
  model=gtk_tree_view_get_model(GTK_TREE_VIEW(pListView));
  /* astuce, car l'ordre de la pile est progressivement détruit ! */
  path1=g_strdup_printf ("%d:%d",0,0);
  path=gtk_tree_path_new_from_string(path1);
  for (count=0;count<BiblioItemsCount;count++)
     {    	  
	   gtk_tree_model_get_iter(model, &iter, path);	   
	   gtk_list_store_remove(GTK_LIST_STORE(model),&iter); 
     }/* next count */       	
  g_free(path1);
  gtk_tree_path_free(path);  
  BiblioItemsCount=0;   
} 	
/**********************************
 * sauvegarder la biblio
 * *******************************/
void biblio_save_biblio(gchar *path_to_file,GtkTreeView     *treeview)
{
    GtkTreeModel* model;
    GtkTreePath* path;
    gchar* path1;
    GtkTreeIter iter;
    gint count,atom,err;
    gchar *path_temp,*path_temp_archive,*command,*nom_item,*path_to_tmp_file,*memo;
	gchar buf[512];
	FILE* f;
    struct zip* archive;
    struct zip_source* zs;
  	
	command = g_strdup_printf("rm %s/.ella/*.*",getenv("HOME"));
	err = system(command);
	g_free(command);
	path_temp=g_strdup_printf("%s/.ella/archive.lst",getenv("HOME"));
    path_temp_archive=g_strdup_printf("%s/.ella/ella.zip",getenv("HOME"));
	/* passer en revue toute la liste */
	if (BiblioItemsCount>0)
	  {
		    model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));  
			/* créer le descripteur du contenu */
			f=fopen(path_temp,"w");
			for (count=0;count<BiblioItemsCount;count++)
			  {
				 path1=g_strdup_printf ("%d:%d",count,0);
			     path=gtk_tree_path_new_from_string(path1); 
				 gtk_tree_model_get_iter(model, &iter, path);
			     gtk_tree_model_get (model, &iter, BIBLIO_ATOM, &atom, -1);  
				 gtk_tree_model_get (model, &iter, BIBLIO_NAME_COLUMN, &nom_item, -1); 
				 gtk_tree_model_get (model, &iter, BIBLIO_MEMO_COLUMN, &memo, -1); 
				 switch (atom)
				   {
					   case BIBLIO_ATOM_BITMAP:
					     {
							 fprintf(f,"[BIBBTM]%s\n",g_strescape(nom_item,NULL));
							 break;
						 }
					   case BIBLIO_ATOM_SVG:
					     {
							 fprintf(f,"[BIBVCT]%s\n",g_strescape(nom_item,NULL));							 
							 break;
						 }
					   case BIBLIO_ATOM_WAV:
					     {
							 fprintf(f,"[BIBSND]%s\n",g_strescape(nom_item,NULL));							 
							 break;
						 }
					   case BIBLIO_ATOM_SYMBOL:
					     {
							 fprintf(f,"[BIBSYM]%s\n",g_strescape(nom_item,NULL));							 
							 break;
						 } 
					   case BIBLIO_ATOM_CLIP:
					     {
							fprintf(f,"[BIBCLI]%s\n",g_strescape(nom_item,NULL));	
							break;
						 }	 
					   case BIBLIO_ATOM_BUTTON:
					     {
							 /* TODO */
							printf("buuton \n");
							fprintf(f,"[BIBBTN]%s\n",g_strescape(nom_item,NULL));
							break;
						 }	 
					   default:printf("atom unknown ! \n");	 
				   }/*end switch*/
				if (strlen(memo)>0)
				  {
				    fprintf(f,"[REMARK]%s\n",g_strescape(memo,NULL));   
				  }
				g_free(nom_item);   
			    g_free(path1);
				if (memo!=NULL)
				  {
				     g_free(memo);
				  }
                gtk_tree_path_free(path);   		
			  }/* next */
			fclose(f);
	    /* créer un ZIP et y ajouter les fichiers */
		   err=0;
	       archive=zip_open(path_temp_archive,ZIP_CREATE,&err);/* avec ce mode, le fichier est écrasé ! */	
		   if(err != 0 || !archive)
             {
               zip_error_to_str(buf, sizeof buf, err, errno);
               fprintf(stderr,"«%s» %d in function «%s» ,err=%d\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,err,buf);
               exit(EXIT_FAILURE);
             }
		   zs = zip_source_file(archive,path_temp,0,0); /* lit par défaut tout le fichier */
           if (zs==NULL)
            {
              printf("erreur source \n");
              return;
            } 
           /* ajouter à l'archive le fichier texte de description */ 
           err = zip_add(archive,"archive.lst",zs);	  
		   for (count=0;count<BiblioItemsCount;count++)
			  {
				 path1=g_strdup_printf ("%d:%d",count,0);
			     path=gtk_tree_path_new_from_string(path1); 
				 gtk_tree_model_get_iter(model, &iter, path);	
				 /* ajouter éléments à l'archive */
				 gtk_tree_model_get (model, &iter, BIBLIO_PATH_NAME_COLUMN, &path_to_tmp_file, -1); 
				 gtk_tree_model_get (model, &iter, BIBLIO_NAME_COLUMN, &nom_item, -1); 
				 zs = zip_source_file(archive,path_to_tmp_file,0,0); /* lit par défaut tout le fichier */
				 if (zs==NULL)
                   {
                     printf("erreur source \n");
				     return;
			       }
				 err = zip_add(archive,g_path_get_basename(path_to_tmp_file),zs);					 
				 g_free(path1);
                 gtk_tree_path_free(path); 
				 g_free(path_to_tmp_file);
				 g_free(nom_item);   
			 }/* next */
		   /* utilisation terminée, fermeture de l'archive */
           zip_close(archive); 	 
		   /* transfert du fichier .zip vers sa cible */
		   copier_fichier_biblio(path_temp_archive,path_to_file);
	  }/* endif */
	g_free(path_to_file);	
	g_free(path_temp);
    g_free(path_temp_archive);	
}
/**********************************
 * dialogue de sauvegarde d'une
 * archive
 * *******************************/

GtkWidget*
create_dialogSaveArchiveBiblio (void)
{
  GtkWidget *dialogSaveArchiveBiblio;
  GtkWidget *dialog_vbox11;
  GtkWidget *dialog_action_area11;
  GtkWidget *button10;
  GtkWidget *button11;
  GtkFileFilter *filter;
		
  filter = gtk_file_filter_new ();
  dialogSaveArchiveBiblio = gtk_file_chooser_dialog_new (gettext("Save library as ..."), 
      NULL, GTK_FILE_CHOOSER_ACTION_SAVE,NULL,NULL, NULL);
  gtk_file_filter_add_pattern (filter, "*.bibella");
  gtk_file_filter_set_name (filter,gettext("libraries"));  		
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialogSaveArchiveBiblio),filter);	  
  gtk_container_set_border_width (GTK_CONTAINER (dialogSaveArchiveBiblio), 5);
  gtk_window_set_position (GTK_WINDOW (dialogSaveArchiveBiblio), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialogSaveArchiveBiblio), TRUE);
  gtk_window_set_icon_name (GTK_WINDOW (dialogSaveArchiveBiblio), "gtk-save");
  gtk_window_set_type_hint (GTK_WINDOW (dialogSaveArchiveBiblio), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox11 = gtk_dialog_get_content_area (dialogSaveArchiveBiblio);
  gtk_widget_show (dialog_vbox11);

  dialog_action_area11 = gtk_dialog_get_action_area (dialogSaveArchiveBiblio);
  gtk_widget_show (dialog_action_area11);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area11), GTK_BUTTONBOX_END);

  button10 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button10);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialogSaveArchiveBiblio), button10, GTK_RESPONSE_CANCEL);
  gtk_widget_set_can_default (button10, TRUE);

  button11 = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (button11);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialogSaveArchiveBiblio), button11, GTK_RESPONSE_OK);
  gtk_widget_set_can_default (button11, TRUE);
  gtk_widget_grab_default (button11);
  return dialogSaveArchiveBiblio;
}

/**********************************
 * dialogue d'ouverture d'une 
 * archive
 * *******************************/
GtkWidget*
create_dialogOpenArchiveBiblio (void)
{
  GtkWidget *dialogOpenArchiveBiblio;
  GtkWidget *dialog_vbox10;
  GtkWidget *dialog_action_area10;
  GtkWidget *button8;
  GtkWidget *button9;
  GtkFileFilter *filter;
		
  filter = gtk_file_filter_new ();
  dialogOpenArchiveBiblio = gtk_file_chooser_dialog_new (gettext("Import a library content"), 
                      NULL, GTK_FILE_CHOOSER_ACTION_OPEN,NULL,NULL, NULL);
	
  gtk_file_filter_add_pattern (filter, "*.bibella");
  gtk_file_filter_set_name (filter,gettext("libraries"));  		
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialogOpenArchiveBiblio),filter);
    			  
  gtk_container_set_border_width (GTK_CONTAINER (dialogOpenArchiveBiblio), 5);
  gtk_window_set_position (GTK_WINDOW (dialogOpenArchiveBiblio), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialogOpenArchiveBiblio), TRUE);
  gtk_window_set_icon_name (GTK_WINDOW (dialogOpenArchiveBiblio), "gtk-open");
  gtk_window_set_type_hint (GTK_WINDOW (dialogOpenArchiveBiblio), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox10 = gtk_dialog_get_content_area (dialogOpenArchiveBiblio);
  gtk_widget_show (dialog_vbox10);

  dialog_action_area10 = gtk_dialog_get_action_area (dialogOpenArchiveBiblio);
  gtk_widget_show (dialog_action_area10);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area10), GTK_BUTTONBOX_END);

  button8 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button8);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialogOpenArchiveBiblio), button8, GTK_RESPONSE_CANCEL);
  gtk_widget_set_can_default (button8, TRUE);

  button9 = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (button9);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialogOpenArchiveBiblio), button9, GTK_RESPONSE_OK);
  gtk_widget_set_can_default (button9, TRUE);

  gtk_widget_grab_default (button9);
  return dialogOpenArchiveBiblio;
} 	
/*********************************
 * callbacks de la bibliothèque
 * ******************************/
/********************************
 * archiver une bibliothèque
 * *****************************/
void
on_buttonBiblioArchive_clicked         (GtkButton       *button,
                                        GtkTreeView     *treeview)
{
	GtkWidget *dlg;
    gint reponse;
	gchar *path_to_file;
	
	dlg=create_dialogSaveArchiveBiblio ();
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dlg), gettext("noname.bibella"));	
    gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dlg),TRUE);/* contrôle overwrite */	
	reponse=gtk_dialog_run(GTK_DIALOG(dlg));
	if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
	/* quel fichier ? */
	path_to_file= gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
	gtk_widget_destroy(dlg);	
	/* archiver */
    biblio_save_biblio(path_to_file,treeview);		
}
/*****************************************
 * returns a pointer on treview GtkWidget
 * of library
 * ****************************************/
GtkTreeView* biblio_get_treview(void)
{
	return GTK_TREE_VIEW(pListViewBiblio);
}
/***********************
 * open an archive file
 * ************************/
gint biblio_open_archive( gchar* path_to_file, GtkTreeView     *treeview)
{
	gchar* p;
    gchar *path_temp,*path_to_tmp_file,*instr,*param;
    struct zip *archive;
    struct zip_file *fpz=NULL;
    struct zip_stat sbzip;
	char buf[1024];
	FILE* fichier;
	gboolean existe;
    GtkTreeModel* model;
    GtkTreePath* path;
    gchar* path1=NULL;
    GtkTreeIter iter;
    gint err,nto,i;	
	
 /* effacer le contenu actuel */
	biblio_clear_items(treeview);
	/* ouvrir l'archive */
	err=0;
    archive=zip_open(path_to_file,ZIP_CHECKCONS,&err);
    if(err != 0 || !archive)
        {
          zip_error_to_str(buf, sizeof buf, err, errno);
          fprintf(stderr,"«%s» %d in function «%s» ,err=%d\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,err,buf);
          exit(EXIT_FAILURE);
        }    	
	/* déterminer le nombre de fichiers contenus */
    nto = zip_get_num_files(archive);
	for (i=0;i<nto;i++)
      {
		 zip_stat_index(archive, i, 0, &sbzip);       
         /*pour savoir la taille du fichier et ainsi pouvoir le lire en entier*/
         if(zip_stat(archive, sbzip.name, 0, &sbzip) == -1)
           {
              strcpy(buf,zip_strerror(archive));
              fprintf(stderr,"«%s» %d in function «%s»\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,buf);
              zip_fclose(fpz);
              zip_close(archive);
              exit(EXIT_FAILURE);
           }/* if erreur */
		 fpz=zip_fopen(archive,sbzip.name,ZIP_FL_UNCHANGED);  
		 if(!fpz)
            {
              strcpy(buf,zip_strerror(archive));
			  fprintf(stderr,"«%s» %d in function «%s»\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,buf);
			  zip_close(archive);
			  exit(EXIT_FAILURE);
			}/* erreur */
		 p=g_malloc(sbzip.size*sizeof(gchar)+1);
         assert(p != NULL);
         /*lit le fichier*/
         if(zip_fread(fpz,p,sbzip.size) != sbzip.size)
           {
             fprintf(stderr,"«%s» %d in function «%s»\n«read error ...»\n",__FILE__,__LINE__,__FUNCTION__);
             g_free(p);
             zip_fclose(fpz);
             zip_close(archive);
             exit(EXIT_FAILURE);
           }/* erreur */
		 /* écriture en mode binaire */
		 path_to_tmp_file=g_strdup_printf("%s/%s",(gchar *)g_get_tmp_dir(),sbzip.name);
         fichier = fopen(path_to_tmp_file,"w");
         fwrite(p,sizeof(gchar),sbzip.size,fichier);
         fclose(fichier);
         zip_fclose(fpz);
         free(p);
		 g_free(path_to_tmp_file);   	
	  }/* next i */
    /* lecture terminée, fermeture de l'archive */
    zip_close(archive);  
	/* maintenant, il faut décoder le fichier de description */	  
	path_temp=g_strdup_printf("%s/%s",(gchar *)g_get_tmp_dir(),"archive.lst");
	existe=g_file_test(path_temp,G_FILE_TEST_EXISTS);
	if (!existe)
	   {
		   printf("fichier de description absent \n");
		   g_free(path_to_file);
		   g_free(path_temp);
		   return -1;
	   }
	/* lire le fichier de description */  
	fichier=fopen(path_temp,"r");
	if (fichier==NULL)
	     printf("problème !\n");
	/*printf("path temp:%s\n",path_temp);*/
	i=0;
	while (fgets(buf,1000,fichier)!=NULL)
	    {
			instr=g_strndup (buf,8);
			if (strlen(buf)>8)
      	           param=(gchar *)g_substr (buf, 8, strlen(buf)-2); 
			/*printf("instruction:%s<>opérande:%s\n",instr,g_strcompress(param)); */
			path_to_tmp_file=g_strdup_printf("%s/%s",(gchar *)g_get_tmp_dir(),g_strescape(param,NULL));
			/*printf("fichier dans tmp:%s\n",path_to_tmp_file); */
			if (g_strcmp0(instr,"[BIBBTM]")==0)
			   {
				  add_tree_list_biblio(pBiblio,1,param,path_to_tmp_file,FALSE);
				  i++; 
			   }
			if (g_strcmp0(instr,"[BIBVCT]")==0)
			   {
				  add_tree_list_biblio(pBiblio,2,param,path_to_tmp_file,FALSE);
				  i++;
			   }			   
			if (g_strcmp0(instr,"[BIBSND]")==0)
			   {
				  add_tree_list_biblio(pBiblio,3,param,path_to_tmp_file,FALSE); 
				  i++;
			   }
			if (g_strcmp0(instr,"[BIBSYM]")==0)
			   {
				  add_tree_list_biblio(pBiblio,4,param,path_to_tmp_file,FALSE); 
				  i++;
			   }   
			if (g_strcmp0(instr,"[BIBCLI]")==0)
			   {
				  add_tree_list_biblio(pBiblio,5,param,path_to_tmp_file,FALSE); 
				  i++;
			   }   
			if (g_strcmp0(instr,"[BIBBTN]")==0)
			   {
				  add_tree_list_biblio(pBiblio,6,param,path_to_tmp_file,FALSE); 
				  i++;
			   }   
			if (g_strcmp0(instr,"[REMARK]")==0)
			   {
				   model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));  
                   path1=g_strdup_printf ("%d:%d",i-1,0);/* astuce, car l'ordre de la pile est progressivement détruit ! */
                   path=gtk_tree_path_new_from_string(path1);
				   gtk_tree_model_get_iter(model, &iter, path);	  
				   gtk_list_store_set(GTK_LIST_STORE(model),&iter,BIBLIO_MEMO_COLUMN,param, -1); 	                				   
                   g_free(path1);
                   gtk_tree_path_free(path);  				   
			   }			   
			g_free(path_to_tmp_file);   			   
			g_free(instr);	
		}/* wend */
	fclose(fichier);	
	/* mettre à jour l'onglet bibliothèque */
	g_free(path_to_file);	
	return 0;	
}
/*******************************
 * callback to open a library archive
 * ************************************/
void
on_buttonBiblioOpenArchive_clicked     (GtkButton       *button,
                                        GtkTreeView     *treeview)
{
    gint reponse, retour;
	GtkWidget *dlg;
	gchar *path_to_file;
 	
	dlg=create_dialogOpenArchiveBiblio();
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));	
	reponse=gtk_dialog_run(GTK_DIALOG(dlg));
	if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
	/* quel fichier ? */
	path_to_file= gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
	gtk_widget_destroy(dlg);
   	retour=biblio_open_archive(path_to_file, treeview);
}

/****************************************
 * callback suite à l'édition d'un
 * commentaire
 * *************************************/
void on_biblio_comment_edited(GtkCellRendererToggle *cell, gchar *path,gchar *new_text,
 GtkTreeView     *treeview) 
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreePath *path_tree;
	GtkTreeIter iter;
	gchar *commentaire;
	
	/* récupérer la ligne concernée */
	selection = gtk_tree_view_get_selection ( treeview );
	gtk_tree_selection_set_mode ( selection, GTK_SELECTION_SINGLE );
	if ( gtk_tree_selection_get_selected ( selection, &model, &iter ) )
	{
		/* lis le n° du calque  */
		gtk_tree_model_get (model, &iter,-1);/* &iter avec ampersand obligatoire */		
		/* lis le chemin */
		path_tree = gtk_tree_model_get_path(model,&iter);
		commentaire= gtk_tree_path_to_string (path_tree);		
		/* modifier le bouton à l'écran */
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, BIBLIO_MEMO_COLUMN, new_text,-1);
		g_free(commentaire);
		gtk_tree_path_free(path_tree);		
	}		
}
/**********************************
 * retirer un élément de la liste
 * *******************************/
void on_buttonBiblioRemove_clicked(GtkButton       *button,
                                        GtkTreeView     *treeview)
{
 GtkTreeSelection *selection;
 GtkTreeModel *model;
 GtkTreeIter iter;
 
 selection = gtk_tree_view_get_selection ( treeview );
 gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE );  
 /* FIXME ! ther isn't any check, and perhaps we can ERASE a USED ressource ! */
 if ( gtk_tree_selection_get_selected (selection, &model, &iter ) )
	{	
		gtk_tree_model_get (model, &iter, -1);		
		gtk_list_store_remove(GTK_LIST_STORE(model),&iter);
		BiblioItemsCount--;
	}
}

/**********************************
 * création dynamique du modèle
 * *******************************/
GtkListStore *init_store_biblio(void)
{
    GtkListStore *pListStore;   
    GType *types;
    
    /* création dynamique du modèle */
    types = g_new0 (GType, BIBLIO_N_COLUMN);
    types[0] = GDK_TYPE_PIXBUF;/* image de l'objet de la biblio */
    types[1] = G_TYPE_STRING;/* nom de l'objet sous forme courte */
    types[2] = G_TYPE_STRING;/* chemin d'accès au fichier réel */
	types[3] = G_TYPE_BOOLEAN;
    types[4] = G_TYPE_STRING;/* commentaire */  
	types[5] = G_TYPE_STRING;/* indicateur visible ou non */  
	types[6] = G_TYPE_INT; /* atom du type d'objet */
	types[7] = G_TYPE_STRING; /*short name for buttons */
    pListStore = gtk_list_store_newv (8, types);
    g_free (types);
 return pListStore;			
}
/*************************************
 ajouter un élément à la bibliothèque
*************************************/
void add_tree_list_biblio(GtkListStore *pListStore,gint categorie,gchar *nom,gchar *path,gboolean actif)
{
   gchar *sLayerName, *sPathName, *sShortName;
   GtkTreeIter pIter;  
  
    /*init content */   
    sLayerName = NULL;
	sPathName = NULL;
	sShortName = NULL;	
    /* Insert elements */
   	sPathName = g_strdup_printf("%s",path);
    /* build new line of the list store */
    gtk_list_store_append(pListStore, &pIter);
    /* store datas */
    if(categorie==6)
	  {
		  sShortName = g_strdup_printf("%s", nom);
		  sLayerName = g_strdup_printf("button_%s",nom);
	  }
	else
	  {
		  sLayerName = g_strdup_printf("%s", nom);
      }   
    gtk_list_store_set(pListStore, &pIter,
            BIBLIO_NAME_COLUMN, sLayerName,
            BIBLIO_PATH_NAME_COLUMN, sPathName,  
			BIBLIO_MEMO_COLUMN,"", 
			BIBLIO_VISIBLE_COLUMN,FALSE, 
			BIBLIO_SHORT_NAME,
			sShortName,     
            -1);   
    /* choix de l'image à insérer */
    switch(categorie)
      {
         case 1:/* image bitmap */
            {
		       gtk_list_store_set(pListStore, &pIter,BIBLIO_TYPE_COLUMN,pIco_image, -1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_ATOM,BIBLIO_ATOM_BITMAP,-1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_VISIBLE_COLUMN,actif,-1);
               break;
            }
         case 2:/* dessin vectoriel */
            {
               gtk_list_store_set(pListStore, &pIter,BIBLIO_TYPE_COLUMN,pIco_dessin, -1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_ATOM,BIBLIO_ATOM_SVG,-1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_VISIBLE_COLUMN,actif,-1);  
               break;
            }
         case 3:/* son */
            {
               gtk_list_store_set(pListStore, &pIter,BIBLIO_TYPE_COLUMN,pIco_son, -1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_ATOM,BIBLIO_ATOM_WAV,-1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_VISIBLE_COLUMN,actif,-1);  
               break;
            }
		case 4:/* symbol */
            {
               gtk_list_store_set(pListStore, &pIter,BIBLIO_TYPE_COLUMN,pIco_symbol, -1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_ATOM,BIBLIO_ATOM_SYMBOL,-1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_VISIBLE_COLUMN,actif,-1);  
               break;
            }	
	    case 5:/* clip */
            {
               gtk_list_store_set(pListStore, &pIter,BIBLIO_TYPE_COLUMN,pIco_clip, -1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_ATOM,BIBLIO_ATOM_CLIP,-1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_VISIBLE_COLUMN,actif,-1);  
               break;
            }	
		case 6:/* button */
		    {
				/*TODO */
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_TYPE_COLUMN,pIco_button, -1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_ATOM,BIBLIO_ATOM_BUTTON,-1);
			   gtk_list_store_set(pListStore, &pIter,BIBLIO_VISIBLE_COLUMN,actif,-1);  
               break;
            }					
      }		  
   BiblioItemsCount++;  
   if(sLayerName)
      g_free(sLayerName);   
   if(sPathName)	  
      g_free(sPathName);   
   if(sShortName)
      g_free(sShortName);	  
}


/*********************************
 * créer la listree de type 
 * ListStore, donc sans
 * "enfants" (pas dépliables)
 * "ancetre" est un pointeur
 * sur l'item root de l'éditeur
 * ******************************/
GtkWidget *init_tree_list_biblio(GtkListStore *pListStore)
{
    GtkWidget *pListViewBib;
    GtkTreeViewColumn *pColumn;
    GtkCellRenderer *pCellRenderer;    

    /* vue des titres&flags des calques */
    pListViewBib = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pListStore));
    /* interdire l'affichage des en-têtes */
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(pListViewBib), TRUE);   
    /* Creation de la 1ère colonne : image */
    pCellRenderer = gtk_cell_renderer_pixbuf_new ();  
    pColumn = gtk_tree_view_column_new_with_attributes(gettext("Type"),
        				pCellRenderer,
        				"pixbuf", BIBLIO_TYPE_COLUMN,
        				NULL);
    /* Ajout de la colonne <image> à la vue */
    gtk_tree_view_append_column(GTK_TREE_VIEW(pListViewBib), pColumn);    
    /* Creation de la 2e colonne : nom court de l'objet */    
    pCellRenderer = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(pCellRenderer),"font","sans 8",NULL);  /* forcer la police */     
    pColumn = gtk_tree_view_column_new_with_attributes(gettext("Name"),
        pCellRenderer,
        "text", BIBLIO_NAME_COLUMN,
        NULL);                           
    gtk_tree_view_append_column(GTK_TREE_VIEW(pListViewBib), pColumn);        
    /* Ajout de la colonne <visible> à la vue */
	pCellRenderer = gtk_cell_renderer_toggle_new();
    pColumn = gtk_tree_view_column_new_with_attributes(gettext("Used"),
                                                       pCellRenderer,
        												"active", BIBLIO_VISIBLE_COLUMN, 
        												NULL);
    g_object_set(pCellRenderer,"activatable" ,TRUE,NULL); /* pour gérer la bascule, mais ne suffit pas, car nécessaire de l'imposer ensuite avec une call back */                           
    /*g_signal_connect(pCellRenderer, "toggled", G_CALLBACK(on_layer_visible_toggled),pListViewLayers); */
    gtk_tree_view_append_column(GTK_TREE_VIEW(pListViewBib), pColumn);  
    /* colonne commentaire BIBLIO_MEMO_COLUMN*/
	pCellRenderer = gtk_cell_renderer_text_new();
    /* le rendre éditable  */
    g_object_set(G_OBJECT(pCellRenderer), "editable", TRUE,  NULL);  
    g_object_set(G_OBJECT(pCellRenderer), "width-chars",16, NULL);    
    g_object_set(G_OBJECT(pCellRenderer),"font","sans 8",NULL);  /* forcer la police */     
    pColumn = gtk_tree_view_column_new_with_attributes(gettext("Comments"),
        			pCellRenderer,
        			"text", BIBLIO_MEMO_COLUMN,
        			NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(pListViewBib), pColumn); 
	/* ajouter la callback */
    g_signal_connect(pCellRenderer, "edited",
	           G_CALLBACK(on_biblio_comment_edited),pListViewBib);
	/* création de la colonne destinée au texte "invisible", le nom réel du fichier */
    pCellRenderer = gtk_cell_renderer_text_new();
    pColumn = gtk_tree_view_column_new_with_attributes(gettext("Localization"),
        			pCellRenderer,"text", BIBLIO_PATH_NAME_COLUMN,
           			NULL);
   /* gtk_tree_view_append_column(GTK_TREE_VIEW(pListViewBib), pColumn);  
	* initialiser le DnD - reste à gérer la destination */	
	gtk_drag_source_set(pListViewBib,GDK_BUTTON1_MASK,
	                    Btargetentries, 4,/* nbre valeurs déclarées */
                      GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_LINK);	
	BiblioItemsCount=0;				  				   
    return pListViewBib;
}
/*************************
 * création icones de la
 * bibliothèque
 * *********************/
void init_ico_biblio(void)
{
	pIco_dessin= gdk_pixbuf_new_from_xpm_data((const char **)dessin_xpm);
    pIco_image=gdk_pixbuf_new_from_xpm_data((const char **)image_xpm);
    pIco_son=gdk_pixbuf_new_from_xpm_data((const char **)sound_xpm);
	pIco_symbol=gdk_pixbuf_new_from_xpm_data((const char **)symbol_xpm);
	pIco_clip=gdk_pixbuf_new_from_xpm_data((const char **)flash24x24_xpm);
	pIco_button=gdk_pixbuf_new_from_xpm_data((const char **)button24x24_xpm);
}

/****************************
 * initialisation de la
 * bibliothèque
 * ************************/
 GtkListStore *biblio_init(GtkWidget *win,GtkWidget *butRemove,GtkWidget *buttonBiblioArchive,
                           GtkWidget *buttonBiblioOpenArchive)
{
  /* stockage */
  GtkListStore *pListStoreBiblio;/* magasin de rangement des objets de la biblio */
  /* permet d'afficher les objets */
  /* GtkWidget *pListViewBiblio; */
  
  
  /* passer les stores */
  init_ico_biblio();
  pListStoreBiblio=init_store_biblio();
  pListViewBiblio=init_tree_list_biblio(pListStoreBiblio);	
  /* ajoute à la fenêtre voulue l'arbre */
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (win), pListViewBiblio); 
  g_signal_connect ((gpointer) butRemove, "clicked",
                    G_CALLBACK (on_buttonBiblioRemove_clicked),
                    G_OBJECT(pListViewBiblio));
  g_signal_connect ((gpointer) buttonBiblioArchive, "clicked",
                    G_CALLBACK (on_buttonBiblioArchive_clicked),
                    G_OBJECT(pListViewBiblio));
  g_signal_connect ((gpointer) buttonBiblioOpenArchive, "clicked",
                    G_CALLBACK (on_buttonBiblioOpenArchive_clicked),
                    G_OBJECT(pListViewBiblio));					
  return pListStoreBiblio;
}
/**********************************************
 * extract a substring from a string
 * *******************************************/

gchar *clean_svg_datas (const gchar *s, guint start, guint end)
{
   gchar *new_s = NULL;

   if (s != NULL && start < end)
   {
/* (1)*/
      new_s = g_malloc (sizeof (*new_s) * (end - start + 2));
      if (new_s != NULL)
      {
         gint i;

/* (2) */
         for (i = start; i <= end; i++)
         {
/* (3) */
            new_s[i-start] = s[i];
         }
         new_s[i-start] = '\0';
      }
      else
      {
         fprintf (stderr, "Memoire insuffisante\n");
         exit(EXIT_FAILURE);
      }
   }
   return new_s;
}
/***********************************************
 * function to 'parse' a complex SVG file and
 * extract the first valid path ;
 * the path is returned by this function 
 * ********************************************/
gchar *biblio_parse_svg_file(gchar *nom_fich)
{
	FILE *fich;
	gboolean sortie=FALSE;
	gboolean path_found=FALSE;
	gchar *resultat=NULL;
	gchar buf[LONG_STRING+1];
	gchar *chaine=NULL;
	gchar *debut,*fin;
	
	
	/* we read the entire SVG file, until EOF or found a complete <path */
	fich=fopen(nom_fich,"r");
	 while ((feof(fich)==0)&&(sortie==FALSE))
      {
      	fgets(buf,LONG_STRING,fich);	
		/* there is an <path instruction inside ? */
		if (path_found==FALSE)/* because we want to avoid a double test */
		  {
			  resultat=g_strrstr((gchar *)buf,"<path");
			  if (resultat!=NULL)
			    path_found=TRUE;
		  }
		/* there is a data path structure inside ? */
		else
		  {
              /* space recquired to avoid confusing with id=path"*/
			  resultat=g_strrstr((gchar *)buf," d=\"");
			  if (resultat!=NULL)
			    {
			     /* we must clean the data field */
				 debut=g_strrstr((gchar *)buf,"=\"");
				 if (debut!=NULL)
				    {
				     fin=g_strrstr((gchar *)debut+1,"\"");				
				    }
				 if ((debut!=NULL)&&(fin!=NULL))
				   { /* we only use the string between 'debut+2' and fin-1 */				   
				    chaine=g_strdup_printf("%s",clean_svg_datas((gchar *)buf,debut+2-buf,fin-1-buf));					
				   }
			     else
				   {
					 biblio_dlg_misc_error(
					    gettext("Sorry, can't do that !\nThe paths datas are corrupted\nor too complex."));				     
				   }
				 sortie=TRUE;
			    }
		  } 
	  }
	/* close input file */
	fclose(fich);  	
	return chaine;
}
void button_put_colors_alpha_idle(FILE* f, guint32 fcolor, guint32 scolor)
{
  gdouble valeur_alpha;
  
  /* conversion :   */
   fprintf(f,"[IDLFCR]%d\n",(fcolor & 0xFF000000)>>24);
   fprintf(f,"[IDLFCG]%d\n",(fcolor & 0x00FF0000)>>16);
   fprintf(f,"[IDLFCB]%d\n",(fcolor & 0x0000FF00)>>8);
   valeur_alpha=(fcolor & 0x000000000000FF)/255.0*100;   
   fprintf(f,"[IDLFAL]%d\n",(gint)valeur_alpha);
   fprintf(f,"[IDLSCR]%d\n",(scolor & 0xFF000000)>>24);
   fprintf(f,"[IDLSCG]%d\n",(scolor & 0x00FF0000)>>16);
   fprintf(f,"[IDLSCB]%d\n",(scolor & 0x0000FF00)>>8);
   valeur_alpha=(scolor & 0x000000000000FF)/255.0*100;    
   fprintf(f,"[IDLSAL]%d\n",(gint)valeur_alpha);	
} 

void button_put_colors_alpha_hover(FILE* f, guint32 fcolor, guint32 scolor)
{
  gdouble valeur_alpha;
  
  /* conversion :   */
   fprintf(f,"[HOVFCR]%d\n",(fcolor & 0xFF000000)>>24);
   fprintf(f,"[HOVFCG]%d\n",(fcolor & 0x00FF0000)>>16);
   fprintf(f,"[HOVFCB]%d\n",(fcolor & 0x0000FF00)>>8);
   valeur_alpha=(fcolor & 0x000000000000FF)/255.0*100;   
   fprintf(f,"[HOVFAL]%d\n",(gint)valeur_alpha);
   fprintf(f,"[HOVSCR]%d\n",(scolor & 0xFF000000)>>24);
   fprintf(f,"[HOVSCG]%d\n",(scolor & 0x00FF0000)>>16);
   fprintf(f,"[HOVSCB]%d\n",(scolor & 0x0000FF00)>>8);
   valeur_alpha=(scolor & 0x000000000000FF)/255.0*100;    
   fprintf(f,"[HOVSAL]%d\n",(gint)valeur_alpha);	
} 

void button_put_colors_alpha_area(FILE* f, guint32 fcolor, guint32 scolor)
{
  gdouble valeur_alpha;
  
  /* conversion :   */
   fprintf(f,"[AREFCR]%d\n",(fcolor & 0xFF000000)>>24);
   fprintf(f,"[AREFCG]%d\n",(fcolor & 0x00FF0000)>>16);
   fprintf(f,"[AREFCB]%d\n",(fcolor & 0x0000FF00)>>8);
   valeur_alpha=(fcolor & 0x000000000000FF)/255.0*100;   
   fprintf(f,"[AREFAL]%d\n",(gint)valeur_alpha);
   fprintf(f,"[ARESCR]%d\n",(scolor & 0xFF000000)>>24);
   fprintf(f,"[ARESCG]%d\n",(scolor & 0x00FF0000)>>16);
   fprintf(f,"[ARESCB]%d\n",(scolor & 0x0000FF00)>>8);
   valeur_alpha=(scolor & 0x000000000000FF)/255.0*100;    
   fprintf(f,"[ARESAL]%d\n",(gint)valeur_alpha);	
} 

void button_put_colors_alpha_pressed(FILE* f, guint32 fcolor, guint32 scolor)
{
  gdouble valeur_alpha;
  
  /* conversion :   */
   fprintf(f,"[PREFCR]%d\n",(fcolor & 0xFF000000)>>24);
   fprintf(f,"[PREFCG]%d\n",(fcolor & 0x00FF0000)>>16);
   fprintf(f,"[PREFCB]%d\n",(fcolor & 0x0000FF00)>>8);
   valeur_alpha=(fcolor & 0x000000000000FF)/255.0*100;   
   fprintf(f,"[PREFAL]%d\n",(gint)valeur_alpha);
   fprintf(f,"[PRESCR]%d\n",(scolor & 0xFF000000)>>24);
   fprintf(f,"[PRESCG]%d\n",(scolor & 0x00FF0000)>>16);
   fprintf(f,"[PRESCB]%d\n",(scolor & 0x0000FF00)>>8);
   valeur_alpha=(scolor & 0x000000000000FF)/255.0*100;    
   fprintf(f,"[PRESAL]%d\n",(gint)valeur_alpha);	
} 

void button_init_box(FILE* f ,
                        gdouble x, gdouble y, 
                        gdouble width, gdouble height,
                        gdouble l_w, guint32 f_color, guint32 s_color, gdouble alpha
                     )
{
  fprintf(f, "[BTNTYP]%d\n",BIBLIO_BUTTON_BOX);	
  fprintf(f, "[IDLXPS]%f\n",x);
  fprintf(f, "[IDLYPS]%f\n",y); 
  fprintf(f, "[IDLWID]%f\n",width);
  fprintf(f, "[IDLHEI]%f\n",height);  
  fprintf(f, "[IDLLIW]%f\n",l_w);
  button_put_colors_alpha_idle(f, f_color, s_color);
  fprintf(f, "[HOVXPS]%f\n",x);
  fprintf(f, "[HOVYPS]%f\n",y); 
  fprintf(f, "[HOVWID]%f\n",width);
  fprintf(f, "[HOVHEI]%f\n",height);  
  fprintf(f, "[HOVLIW]%f\n",l_w);  
  button_put_colors_alpha_hover(f, f_color, s_color);
  fprintf(f, "[AREXPS]%f\n",x);
  fprintf(f, "[AREYPS]%f\n",y); 
  fprintf(f, "[AREWID]%f\n",width);
  fprintf(f, "[AREHEI]%f\n",height);  
  fprintf(f, "[ARELIW]%f\n",l_w);  
  button_put_colors_alpha_area(f, f_color, s_color);  
  fprintf(f, "[PREXPS]%f\n",x);
  fprintf(f, "[PREYPS]%f\n",y); 
  fprintf(f, "[PREWID]%f\n",width);
  fprintf(f, "[PREHEI]%f\n",height);  
  fprintf(f, "[PRELIW]%f\n",l_w);  
  button_put_colors_alpha_pressed(f, f_color, s_color);  
}					 

void button_init_ellipse(FILE* f ,
                        gdouble x, gdouble y, 
                        gdouble rx, gdouble ry,
                        gdouble l_w, guint32 f_color, guint32 s_color, gdouble alpha
                     )
{
  fprintf(f, "[BTNTYP]%d\n",BIBLIO_BUTTON_ELLIPSE);	
  fprintf(f, "[IDLXPS]%f\n",x);
  fprintf(f, "[IDLYPS]%f\n",y); 
  fprintf(f, "[IDLWID]%f\n",rx);
  fprintf(f, "[IDLHEI]%f\n",ry);  
  fprintf(f, "[IDLLIW]%f\n",l_w);
  button_put_colors_alpha_idle(f, f_color, s_color);
  fprintf(f, "[HOVXPS]%f\n",x);
  fprintf(f, "[HOVYPS]%f\n",y); 
  fprintf(f, "[HOVWID]%f\n",rx);
  fprintf(f, "[HOVHEI]%f\n",ry);  
  fprintf(f, "[HOVLIW]%f\n",l_w);  
  button_put_colors_alpha_hover(f, f_color, s_color);
  fprintf(f, "[AREXPS]%f\n",x);
  fprintf(f, "[AREYPS]%f\n",y); 
  fprintf(f, "[AREWID]%f\n",rx);
  fprintf(f, "[AREHEI]%f\n",ry);  
  fprintf(f, "[ARELIW]%f\n",l_w);  
  button_put_colors_alpha_area(f, f_color, s_color);  
  fprintf(f, "[PREXPS]%f\n",x);
  fprintf(f, "[PREYPS]%f\n",y); 
  fprintf(f, "[PREWID]%f\n",rx);
  fprintf(f, "[PREHEI]%f\n",ry);  
  fprintf(f, "[PRELIW]%f\n",l_w);  
  button_put_colors_alpha_pressed(f, f_color, s_color);  
}	
/***************************************
 * initialize a button : create a
 * new descriptor fill in /tmp
 * directory
 * ************************************/
gint biblio_init_button(gchar* name,gchar* original_title, gdouble x, gdouble y, gdouble rx, gdouble ry, 
                        gdouble width, gdouble height,
                        gdouble l_w, guint32 f_color, guint32 s_color, gdouble alpha,
						gchar* path_poly,
                        gchar* path_to_bitmap )
{
   gchar* path_to_tmp_file=NULL;
   gint erreur=0;
   FILE* f;
   
   path_to_tmp_file = g_strdup_printf("%s/%s.ellabutton",(gchar *)g_get_tmp_dir(),name);
   
   /* open description file */
   f = fopen(path_to_tmp_file,"w");
   fprintf(f, "[BUTTON]%s\n",name);
   switch(original_title[0])
     {
		case 'R':
		 {
             button_init_box(f, x, y, width, height, l_w, f_color, s_color, alpha);
			 break;
		 }
		case 'C':
		 {
             button_init_ellipse(f, x, y, rx, ry, l_w, f_color, s_color, alpha);
			 break;
		 } 
		default:printf("Non managed button type !\n");  
	 }/*end swicth*/

   /* close description file */
   fclose(f);
   add_tree_list_biblio(pBiblio,6,name,path_to_tmp_file,TRUE); 
   if(path_to_tmp_file)
       g_free(path_to_tmp_file);
   path_to_tmp_file = NULL;	 
   BiblioItemsCount++;  	
   return erreur;
}						
