/* Copyright (C) 2010 Lucas Baudin
*
* This file is part of Ella Project.
*
* Ella Project is free software : you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Ella Project is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY ; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
*/

#include "callbacks.h"
#include "export.h"
#include "interface_editor.h"
#include "callbacks/editor.h"
/* noms de fichiers */
static gchar* current_file_path = NULL;/* nom (temporaire) complet du fichier en cours d'utilisation */
gchar chemin_complet[512]; /* stocke de manière durable le chemin complet jusqu'au fichier */
gchar *chemin_export;
gboolean fModified=FALSE;
gboolean fFileNamed=FALSE;
/* autoarchivage biblio */
static gboolean auto_archive_biblio=FALSE;

void on_recent_chooser_activated(GtkWidget* recentchooser, GtkWidget* window)
{
    g_return_if_fail(GTK_IS_WINDOW(window));
    g_return_if_fail(GTK_IS_RECENT_CHOOSER(recentchooser));
    GFile* file = g_file_new_for_uri(gtk_recent_chooser_get_current_uri(GTK_RECENT_CHOOSER(recentchooser)));
    g_return_if_fail(G_IS_FILE(file));
    current_file_path = g_file_get_path(file);
    open_file(window, current_file_path);
}
/************************************
 * open an Ella File : parameter,
 * a correct path to the file
 * *********************************/
gint open_file(GtkWidget *window1, gchar *chemin)
{
    GSList* node_files_data;
    /*
     * Removed all layers, later, this will be wrapped in a function to clear
     * all the properties of our canvas
     */
    ella_canvas_clear_all_layers(canvas);
    ella_timeline_clear_all_layers(timeline);
    EllaFilesData* file_data = ella_files_parse_ella_file(chemin);
    layers_list = NULL;
    node_files_data = g_slist_nth(file_data->layers, 0);
    while(node_files_data)
    {
        layers_list = g_slist_append(layers_list, node_files_data->data);
        ella_canvas_add_layer(canvas, node_files_data->data);
        ella_timeline_add_layer(timeline, node_files_data->data);
        node_files_data = g_slist_next(node_files_data);
        
    }
    ella_canvas_set_page_width(canvas, file_data->width);
    ella_canvas_set_page_height(canvas, file_data->height);
    ella_canvas_set_page_color_red(canvas, file_data->background_color.red);
    ella_canvas_set_page_color_green(canvas, file_data->background_color.green);
    ella_canvas_set_page_color_blue(canvas, file_data->background_color.blue);
    timeline_set_movie_freq(file_data->fps);
    /* Let's register it */
    GtkRecentManager* recent = gtk_recent_manager_get_default();
    gchar* uri = g_file_get_uri(g_file_parse_name(chemin));
    if(!gtk_recent_manager_add_item(recent, uri))
    {
        g_critical("Couldn't register this files in the recently used files: %s", chemin);
    }
	return 0;  	
}		
/***********************************
 * special callbacks for 
 * recent files menus
 * *******************************/
void
on_recent1_activate                    (GtkMenuItem     *menuitem,
                                       GtkWidget *window1)
{
    gboolean existe;
    gint erreur;
    existe = g_file_test(ella_config_get_value("RECENT_FILE_0"), G_FILE_TEST_IS_REGULAR);

    if (!existe)
    {
        dlg_file_error();
    }

    else
    {
        cursor_busy(window1);
        erreur=open_file(window1, ella_config_get_value("RECENT_FILE_0"));
        cursor_normal(window1);
    }
}


void
on_recent2_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
    gboolean existe;
    gint erreur;
    existe = g_file_test(ella_config_get_value("RECENT_FILE_1"), G_FILE_TEST_IS_REGULAR);

    if (!existe)
    {
        dlg_file_error();
    }

    else
    {
        cursor_busy(window1);
        erreur=open_file(window1, ella_config_get_value("RECENT_FILE_1"));
        cursor_normal(window1);
    }
}


void
on_recent3_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
    gboolean existe;
    gint erreur;
    existe = g_file_test(ella_config_get_value("RECENT_FILE_2"), G_FILE_TEST_IS_REGULAR);

    if (!existe)
    {
        dlg_file_error();
    }

    else
    {
        cursor_busy(window1);
        erreur=open_file(window1, ella_config_get_value("RECENT_FILE_2"));
        cursor_normal(window1);
    }
}


void
on_recent4_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
    gboolean existe;
    gint erreur;
    existe = g_file_test(ella_config_get_value("RECENT_FILE_3"), G_FILE_TEST_IS_REGULAR);

    if (!existe)
    {
        dlg_file_error();
    }

    else
    {
        cursor_busy(window1);
        erreur=open_file(window1, ella_config_get_value("RECENT_FILE_3"));
        cursor_normal(window1);
    }
}


void
on_recent5_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
    gboolean existe;
    gint erreur;
    existe = g_file_test(ella_config_get_value("RECENT_FILE_4"), G_FILE_TEST_IS_REGULAR);

    if (!existe)
    {
        dlg_file_error();
    }

    else
    {
        cursor_busy(window1);
        erreur=open_file(window1, ella_config_get_value("RECENT_FILE_4"));
        cursor_normal(window1);
    }
}


void
on_recent6_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
    gboolean existe;
    gint erreur;
    existe = g_file_test(ella_config_get_value("RECENT_FILE_5"), G_FILE_TEST_IS_REGULAR);

    if (!existe)
    {
        dlg_file_error();
    }

    else
    {
        cursor_busy(window1);
        erreur=open_file(window1, ella_config_get_value("RECENT_FILE_5"));
        cursor_normal(window1);
    }
}
/*********************************
 * fonctions servant à différents
 * appels, soit menus,
 * soit boutons
 * bug résolu le 2/1/2010 :
 * pour une raison inconnue, il
 * a fallu ajouter une pseudo ligne
 * en fin de fichier ([FILEND])
 * *************************************/
gint ouvrir_fichier_ella(GtkWidget *window1)
{
    GtkWidget *dialog_ouvrir; 
    guint reponse;

	/* dlg choix du fichier */
    dialog_ouvrir=create_DialogOpenFile(gettext( "Open ..."), "*.esvg",NULL,NULL);

    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog_ouvrir),
                                        ella_config_get_value("PREFERED_FOLDER"));

    reponse = gtk_dialog_run(GTK_DIALOG(dialog_ouvrir));
    /* récupérer le nom et le tester	*/
    if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dialog_ouvrir);
           return 0;
      }
    /* donc réponse OK */
    current_file_path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_ouvrir));
    gtk_widget_destroy(dialog_ouvrir);
	/* save the prefered folder */
	
    ella_config_set_value("PREFERED_FOLDER", g_path_get_dirname(current_file_path));
	cursor_busy(window1);
	reponse = open_file(window1,current_file_path);
	cursor_normal(window1);
	return 0;
}

void ella_get_curent_files_data(EllaFilesData* data)
{
    data->height = ella_canvas_get_page_height(canvas);
    data->width = ella_canvas_get_page_width(canvas);
    data->layers = layers_list;
    data->fps = timeline_get_movie_freq();
    data->background_color.red = ella_canvas_get_page_color_red(canvas);
    data->background_color.green = ella_canvas_get_page_color_green(canvas);
    data->background_color.blue = ella_canvas_get_page_color_blue(canvas);
}

/*************************************
 * enregistrer et enregistrer/sous
 * dans la même fonction
 * **********************************/
void enregistrer(gchar* path)
{
    EllaFilesData data;
    ella_get_curent_files_data(&data);
    g_debug("Write file: %s\n", path);
    ella_files_write_ella_file(&data, path);
}
/*************************************************** 
 * enregistrer_sous pouvant être appelé de partout 
 * ************************************************/
gboolean enregistrer_ella(GtkWidget *window1)
{
    gint reponse;
    GtkWidget *dialog_save;
    gchar *extension;

    /* dialogue sauvegarde */
    dialog_save = create_file_save_dialog ();   
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog_save),
                                        ella_config_get_value("PREFERED_FOLDER"));
    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog_save),
                                      gettext("Untitled.esvg"));
    /* Filters */
    gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog_save), TRUE);/* contrôle overwrite */
    reponse = gtk_dialog_run(GTK_DIALOG(dialog_save)); 
    /* traitement de la réponse */
    if (reponse==GTK_RESPONSE_OK) 
    {
        /* attention, faire un test pour ajouter l'extension ! */	
        current_file_path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_save));
        /* update recent files list */
        gint i;
        for(i = 5; i > 0; i--)
        {
            gchar* key_name = g_strdup_printf("RECENT_FILE_%d", i - 1);
            gchar* key_name_next = g_strdup_printf("RECENT_FILE_%d", i);
            ella_config_set_value(key_name_next, ella_config_get_value(key_name));
        }
        ella_config_set_value("RECENT_FILE_0", current_file_path);
        /* save the prefered folder */
        ella_config_set_value("PREFERED_FOLDER", g_path_get_dirname(current_file_path));
        extension = g_strrstr(current_file_path, ".esvg");
        if (extension == NULL)
        {
            current_file_path = g_strdup_printf("%s.esvg",current_file_path);
        }
        gtk_widget_destroy(dialog_save);
        cursor_busy(window1);
        enregistrer(current_file_path);
        cursor_normal(window1);
        gtk_window_set_title(GTK_WINDOW (window1),
                             g_strdup_printf("%s - Ella", g_path_get_basename(current_file_path)));
        fModified=FALSE;
        fFileNamed=TRUE;
        return TRUE;
    }/* if OK */
    else
    {
        gtk_widget_destroy(dialog_save);
        return FALSE;
    }
}
/**********************************
 * nouveau projet ?
 * *******************************/
void nouveau(GtkWidget *window1)
{
  gboolean retour;
  GtkWidget* dlg;
  gint reponse;
  GdkColor pPgColor;
  
  GtkBuilder* builder = gtk_builder_new ();
 /* fichier modifié, mais pas enregistré ? */
 if (((fModified)&&(!fFileNamed)))
   {
	  /* dialogue */	 
      gtk_builder_add_from_file (builder, g_build_filename((const gchar *) ella_get_data_dir(), "ui/dlg_ask_before_new.ui", NULL), NULL);
	  dlg = GTK_WIDGET (gtk_builder_get_object (builder, "dialogAskBeforeNew"));
	  /* dlg=create_dialogAskBeforeNew (); */
	  reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
	  
	  if (reponse==GTK_RESPONSE_OK)
		  {
			gtk_widget_destroy(dlg);
	        retour=enregistrer_ella(window1); 
	        if (!retour)
	        return;
		  }
     if (reponse==GTK_RESPONSE_CANCEL)
	    {
			gtk_widget_destroy(dlg);
			return;
		}    
     if (reponse==GTK_RESPONSE_REJECT)
	    {
			gtk_widget_destroy(dlg);
		}             
   }

 editor_reset_item_counter();
 /* clear background */
 editor_set_page_margins(editor_get_default_margins());
 editor_set_page_width(editor_get_default_page_width());
 editor_set_page_height(editor_get_default_page_height());

 pPgColor=editor_get_default_page_color();
 editor_set_page_color(pPgColor.red,pPgColor.green, pPgColor.blue);
 fModified=FALSE;
 fFileNamed=FALSE;	
 
 gtk_window_set_title (GTK_WINDOW (window1), gettext( "Ella-noname.ella"));
}
void
 on_nouveau1_activate                   (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
 {
   nouveau(window1); 
 }


void on_ouvrir1_activate(GtkWidget  *window1, GtkWidget* menu_item)
{
    gint erreur;
    erreur = ouvrir_fichier_ella(window1);
}


void on_enregistrer1_activate(GtkMenuItem* menuitem, GtkWidget* window1)
{
    gboolean retour;

    if(current_file_path)
    {
        cursor_busy(window1);

        enregistrer(current_file_path);
        cursor_normal(window1);
    }
    else
    {
        retour = enregistrer_ella(window1);
    }
}


void on_enregistrer_sous1_activate(GtkMenuItem* menuitem, GtkWidget* window1)
{
    enregistrer_ella(window1);
}

/********************************************
 *  importer image dans la bibliothèque 
 * *****************************************/
void
on_une_image_bitmap1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget* dlg;
  gchar* path_to_file_to_import=NULL;
  gchar* path_to_temp_dir=NULL;
  gchar* short_name=NULL;
  gchar* path_target=NULL;
  GSList* list_selected_files,*node=NULL;
  
  dlg=dlg_create_dialogImporter (gettext( "Import a bitmap image"),1);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));  
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /*  OK response */
   list_selected_files=gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   /* now, we proceed until the end of selected files list is reached */
   if (list_selected_files)
     {
       node=g_slist_nth(list_selected_files,0);
       while(node)
        {
         /* transférer fichier vers répertoire temporaire */
	     path_to_file_to_import=g_strdup_printf("%s",(gchar *)(node->data));
	     short_name=(gchar *)g_basename((gchar *)(node->data));
         path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
         ella_files_copy(path_to_file_to_import,path_target);
         /* reste à mettre à jour la future list_store ! */
 
         add_tree_list_biblio(pBiblio,1,short_name,path_target,FALSE);  
         ella_canvas_import_png(canvas, path_to_file_to_import, 0, 0);
	     node=g_slist_next(node);
        }/* wend */
      g_slist_free(list_selected_files);	
     }/*endif list of files */
}
/********************************/
/**** importer une image svg ****/
/********************************/
void
on_une_image_vectorielle1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget* dlg;
  gchar* path_to_file_to_import;
  gchar* path_to_temp_dir;
  gchar* short_name;
  gchar* path_target;
  GSList* list_selected_files,*node=NULL;  
  
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext("Import a SVG file" ),2);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
   list_selected_files=gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   if (list_selected_files)
     {
       node=g_slist_nth(list_selected_files,0);
       while(node)
        {
         /* transférer fichier vers répertoire temporaire */
	     path_to_file_to_import=g_strdup_printf("%s",(gchar *)(node->data));
         short_name=(gchar *)g_basename((gchar *)(node->data));
         path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
         ella_files_copy(path_to_file_to_import,path_target);
         /* reste à mettre à jour la future list_store ! */
         add_tree_list_biblio(pBiblio,2,short_name,path_target,FALSE);      
         g_free(path_target);
	     g_free(path_to_file_to_import);
	     path_to_file_to_import=NULL;
         path_target=NULL;
         g_free((gchar *)(node->data));
	     node=g_slist_next(node);
        }/* wend */
      g_slist_free(list_selected_files);	
     }/*endif list of files */    
}
/***********************************
 * import a 'symbol', in fact a
 * SVG file which will be 
 * recoded
 * ********************************/
void
on_import_symbol_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  GSList* list_selected_files,*node=NULL;  
    
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext("Import a Symbol..." ),4);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
    list_selected_files=gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   if (list_selected_files)
     {
       node=g_slist_nth(list_selected_files,0);
       while(node)
        {
         /* transférer fichier vers répertoire temporaire */
	     path_to_file_to_import=g_strdup_printf("%s",(gchar *)(node->data));
         short_name=(gchar *)g_basename((gchar *)(node->data));
         path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
         ella_files_copy(path_to_file_to_import,path_target);
         /* reste à mettre à jour la future list_store ! */
         add_tree_list_biblio(pBiblio,4,short_name,path_target,FALSE);      
         g_free(path_target);
	     g_free(path_to_file_to_import);
	     path_to_file_to_import=NULL;
         path_target=NULL;
         g_free((gchar *)(node->data));
	     node=g_slist_next(node);
        }/* wend */
      g_slist_free(list_selected_files);	
     }/*endif list of files */
}					
/******************************
 * import a flash clip
 * ***************************/
 void on_import_clip_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  GSList* list_selected_files,*node=NULL;  
    
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext( "Import a clip"),5);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
   list_selected_files=gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   if (list_selected_files)
     {
       node=g_slist_nth(list_selected_files,0);
       while(node)
        {
         /* transférer fichier vers répertoire temporaire */
	     path_to_file_to_import=g_strdup_printf("%s",(gchar *)(node->data));
         short_name=(gchar *)g_basename((gchar *)(node->data));
         path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
         ella_files_copy(path_to_file_to_import,path_target);
         /* reste à mettre à jour la future list_store ! */
         add_tree_list_biblio(pBiblio,5,short_name,path_target,FALSE);      
         g_free(path_target);
	     g_free(path_to_file_to_import);
	     path_to_file_to_import=NULL;
         path_target=NULL;
         g_free((gchar *)(node->data));
	     node=g_slist_next(node);
        }/* wend */
      g_slist_free(list_selected_files);	
     }/*endif list of files */	
 }														
/*******************************
 * import a sound or music
 * ****************************/
void on_importer_son_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  GSList* list_selected_files,*node=NULL;  
    
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext( "Import a sound or a music"),3);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
     list_selected_files=gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   if (list_selected_files)
     {
       node=g_slist_nth(list_selected_files,0);
       while(node)
        {
         /* transférer fichier vers répertoire temporaire */
	     path_to_file_to_import=g_strdup_printf("%s",(gchar *)(node->data));
         short_name=(gchar *)g_basename((gchar *)(node->data));
         path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
         ella_files_copy(path_to_file_to_import,path_target);
         /* reste à mettre à jour la future list_store ! */
         add_tree_list_biblio(pBiblio,3,short_name,path_target,FALSE);      
         g_free(path_target);
	     g_free(path_to_file_to_import);
	     path_to_file_to_import=NULL;
         path_target=NULL;
         g_free((gchar *)(node->data));
	     node=g_slist_next(node);
        }/* wend */
      g_slist_free(list_selected_files);	
     }/*endif list of files */
}	
void on_annuler1_activate(GtkWidget* widget, gpointer user_data)
{
    g_debug("Cancel");	
}								
/********************************
 * options par défaut
 * ****************************/
void
on_options1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *dlg;

    dlg = create_dialogPrefs ((gint)editor_get_default_page_width(),
                         (gint)editor_get_default_page_height(),
                         (gint)editor_get_default_margins(),
                         timeline_get_default_movie_freq(),
                         512,
                         editor_get_default_page_color(),
                         auto_archive_biblio,
                         0);
    gtk_dialog_run(GTK_DIALOG(dlg)); 
    gtk_widget_destroy(GTK_WIDGET(dlg));
}
/****************************************
 * quitter Ella
****************************************/
void
on_quitter1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         *user_data)
{
 gint x,y,w,h;
 gint *xp,*yp,*wp,*hp;
 GtkWidget *window;
 
 
 window=user_data[0];
 /* get the current window geometry */
 gtk_window_get_position(GTK_WINDOW(window),&x,&y);
 gtk_window_get_size(GTK_WINDOW(window),&w,&h);
 /* store the values */
 xp=user_data[1];
 yp=user_data[2]; 
 wp=user_data[3];
 hp=user_data[4];
 *xp=x;
 *yp=y;
 *wp=w;
 *hp=h;
 if (dialog_ask_before_quit()==GTK_RESPONSE_OK)	
     gtk_main_quit();
}
/**********************************
 * clipboard
 * *******************************/
/* gestion de l'activabilité des sous-menus */
void on_edition_activate(GtkMenuItem     *menuitem,
                                        gpointer         *user_data)
{
}
/* copier */
void
on_copier1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}

/* coller */
void
on_coller1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}

/* couper */
void
on_couper1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}

/******************************
 * dupliquer la sélection ;
 * consiste à copier/coller à 
 * la même place
 * ***************************/
void
on_dupliquer1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}

/*** monter un objet au sommet de la pile ***/
void
on_monter_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   editor_raise(pSelectedItemsList);
}

/*** descendre un objet en bas de la pile ***/
void
on_descendre_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_lower(pSelectedItemsList);
}
/******************************
 * edit a button menu
 * ***************************/
void on_edit_as_button_activate                  (GtkMenuItem* menuitem, gpointer         *user_data)
{
	printf("edit a button ! \n");
}
/******************************
 * add script to button menu
 * ***************************/
void on_add_script_to_button_activate                  (GtkMenuItem* menuitem, gpointer         *user_data)
{
	printf("script for button ! \n");
}
/* rotations arbitraires */

void
on_rotation1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}
void
on_90sens_horaire1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   /* TODO: connect this item to our ellacanvas */
}
void
on_90_anti_horaire1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   /* TODO: connect this item to our ellacanvas */
}
void
on_180_sens_horaire1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   /* TODO: connect this item to our ellacanvas */
}	
void
on_270_sens_horaire1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   /* TODO: connect this item to our ellacanvas */
}	   											
/* sélectionner tous les objets */
void
on_tout_selectionner1_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}
/* annuler la sélection dans l'éditeur */
void
on_deselectionner_tout1_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}
/* suppression rapide d'un objet */
void
on_supprimer1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}
			
/***** taille du canevas *****/

void
on_taille1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *dlg;
    gint reponse;
    dlg = create_dialog_page_size ((gint)editor_get_page_width(),
                                 (gint)editor_get_page_height(),
                                 (gint)editor_get_page_margins());
    reponse = gtk_dialog_run(GTK_DIALOG(dlg));
    gtk_widget_destroy(dlg);
}

/**** choix de la couleur de page ****/
void
on_couleur_de_la_page1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    editor_set_page_color_menu();
}

/*************************************
 * change the movie footage
 * **********************************/
void
on_format_animation1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dlg;
  gint reponse;
 
  dlg=create_dialogFootage (timeline_get_movie_freq());
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  gtk_widget_destroy(dlg);   
  /* if OK, returns reponse == 1 */
}

/*******************************
 * dialogue 'à propos'         *
 * ****************************/

void
on_a_propos1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{ 
    GtkWidget *dlg;
    dlg = create_dialog_about();
    gtk_dialog_run(GTK_DIALOG(dlg)); 
    gtk_widget_destroy(GTK_WIDGET(dlg));
}

/**
 * Open the help page.
 * */
void on_help_activate(GtkMenuItem* menuitem, gpointer user_data)
{
    gboolean success;
    gchar* url;

    url = g_strconcat("file://", ella_get_data_dir(), "/ressources/help/help.html", NULL);

    success = gtk_show_uri(NULL, url, gdk_event_get_time(NULL), NULL);

    /* If we can't load local page, e.g. Ella isn't properly installed. */
    if(!success)
    {
        g_warning("Couldn't load local help page: %s\n Let's try to load online page.", url);
        success = gtk_show_uri(NULL, "http://ella.tuxfamily.org/wiki/Help", gdk_event_get_time(NULL), NULL);
        if(!success)
        {
            g_warning("Couldn't load online help page either.");
        }
    }
}
/**
 * Open the bugs page.
 **/
void on_bugs_activate(GtkMenuItem* menuitem, gpointer user_data)
{
    gtk_show_uri(NULL, "https://bugs.launchpad.net/ella", gdk_event_get_time(NULL), NULL);
}

void
on_utilisation_de_la_souris1_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}
/**********************************
 * demande quitter via case de 
 * fermeture
 * ********************************/
gboolean
on_window_ella_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         *user_data)
{
gint x,y,w,h;
 gint *xp,*yp,*wp,*hp;
 GtkWidget *window;
 
 
 window=user_data[0];
 /* get the current window geometry */
 gtk_window_get_position(GTK_WINDOW(window),&x,&y);
 gtk_window_get_size(GTK_WINDOW(window),&w,&h);

 /* store the values */
 xp=user_data[1];
 yp=user_data[2]; 
 wp=user_data[3];
 hp=user_data[4];
 *xp=x;
 *yp=y;
 *wp=w;
 *hp=h;	
	
  if (dialog_ask_before_quit()==GTK_RESPONSE_OK)	
     gtk_main_quit();
  return TRUE;
}

/*******************************
 * appels depuis les boutons de
 * l'IHM
 * ****************************/
void
on_buttonNew_clicked                   (GtkButton       *button,
                                       GtkWidget *window1)
{
   nouveau(window1); 
}
/********************************
 * sauvegarde rapide par clic
 * sur le bouton
 * *****************************/

void
on_buttonSave_clicked                  (GtkButton       *button,
                                        GtkWidget *window1)
{
gboolean retour;
 
    /* tester si le fichier a un nom */
    if(current_file_path)
    {
    enregistrer(current_file_path);
    }
    else
    {
    retour=enregistrer_ella(window1);
    }
}
/*******************************
 * clipboard
 * ****************************/
/* copier */
void
on_buttonCopy_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
}

/* coller */
void
on_buttonPaste_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
}

/* couper */
void
on_buttonCut_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
}

/**************************
 * renvoyer l'état du flag
 * sur l'autoarchivage de
 * la bibliothèque
 * ***********************/
gboolean get_autoarchive_biblio(void)
{
	return auto_archive_biblio;
}
/**************************
 * stocker l'état du flag
 * sur l'autoarchivage de
 * la bibliothèque
 * ***********************/
void set_autoarchive_biblio(gboolean etat)
{
	auto_archive_biblio=etat;
    ella_config_set_value_int("AUTO_ARCHIVE", auto_archive_biblio);
}
/*******************************
 * align selected item(s)
 * ****************************/
void
on_align_items1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dlg;
  gint reponse, h_align,v_align,z_align;

  dlg=create_dialogAlignement();
  reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
  gtk_widget_destroy(dlg);
  if (reponse!=0)/* 0=cancel */
    {
       h_align=dlg_get_horizontal_align();
	   v_align=dlg_get_vertical_align();
	   z_align=dlg_get_zone_align();
    }   
}										
/*********************************************
 * set a new guide layer, i.e. a path object
 * on a layer used to guide an animation
 * *****************************************/
void
on_new_guide_layer1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	printf("menu new guide layer !\n");
}

/********************************
 * callback pour le Dnd, en 
 * réception
 * *****************************/
void
view_onDragDataReceivedTimeline(GtkWidget *wgt, GdkDragContext *context, gint x, gint y,
                        GtkSelectionData *seldata, guint info, guint time,
                        gpointer userdata)
{
    /* TODO: Deprecated */
}
/********************************
 * callback pour le Dnd, en 
 * réception
 * *****************************/
void
view_onDragDataReceived(GtkWidget *wgt, GdkDragContext *context, gint x, gint y,
                        GtkSelectionData *seldata, guint info, guint time,
                        gpointer userdata)
{
    /* TODO: Deprecated */
}


void on_up_layer_pressed(GtkWidget* widget, gpointer user_data)
{
    EllaLayer* layer = ella_timeline_get_selected_layer(timeline); 
    GSList* layers = g_slist_nth(layers_list, 0);
    int i = g_slist_index(layers, layer);
    ella_canvas_layer_up(canvas, layer);
    ella_timeline_layer_up(timeline, layer);
    if(i > 0)
    {
        g_slist_nth(layers, i)->data = g_slist_nth(layers, i - 1)->data;
        g_slist_nth(layers, i - 1)->data = layer;
    }
}

void on_down_layer_pressed(GtkWidget* widget, gpointer user_data)
{
    EllaLayer* layer = ella_timeline_get_selected_layer(timeline); 
    GSList* layers = g_slist_nth(layers_list, 0);
    int i = g_slist_index(layers, layer);
    ella_canvas_layer_down(canvas, layer);
    ella_timeline_layer_down(timeline, layer);
    if(i + 1 < g_slist_length(layers))
    {
        g_slist_nth(layers, i)->data = g_slist_nth(layers, i + 1)->data;
        g_slist_nth(layers, i + 1)->data = layer;
    }
}

void on_remove_layer_pressed(GtkWidget* widget, GtkWidget* button)
{
    if(g_slist_length(layers_list) < 2)
        return;
    EllaLayer* layer = ella_timeline_get_selected_layer(timeline); 
    ella_canvas_layer_remove(canvas, layer);
    ella_timeline_layer_remove(timeline, layer);
    layers_list = g_slist_remove(layers_list, layer);
    if(g_slist_length(layers_list) < 2)
    {
        gtk_widget_set_sensitive(widget, FALSE);
    }
}

void on_speed_changed(GtkSpinButton* widget, GtkWidget* button)
{
    timeline_set_movie_freq(gtk_spin_button_get_value(widget));
}

void on_export_activate(GtkWidget* widget, gpointer data)
{
    ella_export_show_dialog(current_file_path);
}

/* GtkActions for the objects */
void on_action_object_up(GtkWidget* widget, GObject* data)
{
    ella_canvas_up_selected_item(canvas);
}

void on_action_object_down(GtkWidget* widget, GObject* data)
{
    ella_canvas_down_selected_item(canvas);
}

void on_action_object_remove(GtkWidget* widget, GObject* data)
{
    printf("Remove\n");
}

gboolean on_editor_button_press_event(GtkWidget* widget, GdkEventScroll* event)
{
    g_return_if_fail(GTK_IS_SCROLLED_WINDOW(widget));
    if(event->state & GDK_CONTROL_MASK)
    {
        if(event->direction == GDK_SCROLL_UP)
        {
            zoom_in(GTK_WIDGET(gtk_builder_get_object(ella_main_builder, "w_zoom_hscale")), NULL);
        }
        else if (event->direction == GDK_SCROLL_DOWN)
        {
            zoom_out(GTK_WIDGET(gtk_builder_get_object(ella_main_builder, "w_zoom_hscale")), NULL);
        }
        return TRUE;
    }
    return FALSE;
}

void on_action_back(GtkWidget* widget, gpointer data)
{
    ella_undo_manager_back(undo_manager);
    printf("here\n");
    gtk_widget_queue_draw(canvas);
}
void on_action_forward(GtkWidget* widget, gpointer data)
{
    ella_undo_manager_forward(undo_manager);
    printf("here\n");
    gtk_widget_queue_draw(canvas);
}
