/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialogs/dialogs_editor.h"
#include "biblio.h"


/***************************
 * variables locales
 * ************************/
GdkColor TmpSColor;
GdkColor TmpFColor;
GdkColor TmpPageColor;
gint TmpAlpha;

/******************************************
 * dialogue commun à toutes les fonctions
 * de couleurs ; on passe en entrée la
 * couleur courante sous forme
 * GdkColor
 * en sortie, chaine correspondant au choix,
 * chaine nulle si pas de clic sur OK
 * attention : alpha est une valeur de 
 * 0 à 1 !
 * ****************************************/
void redraw_preview(GtkWidget *widget)
{
	cairo_t *cr; 
	cr = gdk_cairo_create(gtk_widget_get_window (widget));

    cairo_set_source_rgb(cr,(gdouble)(TmpPageColor.red/65535.0),
                         (gdouble)(TmpPageColor.green/65535.0),
                         (gdouble)(TmpPageColor.blue/65535.0));
    cairo_rectangle(cr,0,0,64,64);
    cairo_fill(cr);
    cairo_set_source_rgba(cr, (gdouble)(TmpFColor.red/65535.0),
                          (gdouble)(TmpFColor.green/65535.0),
                          (gdouble)(TmpFColor.blue/65535.0),
                          (gdouble)(TmpAlpha/100.0));
    cairo_rectangle(cr,32,5,32,32);
    cairo_fill(cr);
    cairo_set_source_rgba(cr, (gdouble)(TmpSColor.red/65535.0),
                          (gdouble)(TmpSColor.green/65535.0),
                          (gdouble)(TmpSColor.blue/65535.0),
                          (gdouble)(TmpAlpha/100.0));
    cairo_rectangle(cr,32,5,32,32);
    cairo_stroke(cr);
    cairo_destroy(cr); 	
}

void on_DlgColorFColor_changed(GtkWidget *colorsel, GtkWidget* widget)
{
    gdouble couleur[4]; /* tableau temporaire pour les composantes */
   
        /* récupérer la couleur choisie  */
        gtk_color_selection_get_current_color(GTK_COLOR_SELECTION (colorsel),couleur);   
        TmpFColor.red=(guint16)(couleur[0]*65535);
        TmpFColor.green=(guint16)(couleur[1]*65535);          
        TmpFColor.blue=(guint16)(couleur[2]*65535);    
        /* redessiner l'aperçu */  
        redraw_preview(widget);     	             
}

void on_DlgColorSColor_changed(GtkWidget* colorsel, GtkWidget* widget)
{
	gdouble couleur[4]; /* tableau temporaire pour les composantes */
	
              gtk_color_selection_get_current_color(GTK_COLOR_SELECTION (colorsel),couleur);
              TmpSColor.red=(guint16)(couleur[0]*65535);
              TmpSColor.green=(guint16)(couleur[1]*65535);          
              TmpSColor.blue=(guint16)(couleur[2]*65535);    
			  redraw_preview(widget);         	             
   
}

void on_spinbuttonDlgColorAlpha_value_changed(GtkSpinButton* spinbutton,
                                              GtkWidget* widget)
{
	TmpAlpha = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));	
	/* changer la preview */
    redraw_preview(widget); 
}

/************ expose events pour les aperçus de couleurs */
gboolean on_drawingareaDlgColors_expose_event(GtkWidget* widget,
                                              GdkEventExpose* event,
                                              gpointer user_data)
{
 redraw_preview(widget); 
 return FALSE;
}

/************** dialogue couleur pour le fond de page ************/
gchar* dlg_couleur(const gchar* message)
{
	gchar* str = NULL;
	gdouble couleur[4]; /* tableau temporaire pour les composantes */
	GdkColor coul;
	GtkColorSelectionDialog* dialog_couleur;
    gint reponse;
    
    dialog_couleur = (GtkColorSelectionDialog*)gtk_color_selection_dialog_new(message);
    //gtk_widget_hide(dialog_couleur->help_button);
    reponse = gtk_dialog_run(GTK_DIALOG(dialog_couleur));
    if (reponse==GTK_RESPONSE_OK)
          {
              /* récupérer la couleur choisie  */
              gtk_color_selection_get_current_color(GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (dialog_couleur)),couleur);   
              coul.red=(guint16)(couleur[0]*65535);
              coul.green=(guint16)(couleur[1]*65535);          
              coul.blue=(guint16)(couleur[2]*65535);
              str=gdk_color_to_string(&coul);              	             
          }
    gtk_widget_destroy(GTK_WIDGET(dialog_couleur));
	return str;
}

/*****************************
 * choix de la fonte ;
 * en sortie, donne un pointeur
 * sur une structure de type
 * PangoFontDescription
 * ****************************/
PangoFontDescription* dlg_fontes(gchar* message, gchar* current_font)
{
	GtkResponseType result;
	gchar* fontname;
    PangoFontDescription* font_desc=NULL;		
	 
    GtkWidget* dialog = gtk_font_selection_dialog_new(message);
	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dialog),
                                                         current_font);
    result = gtk_dialog_run(GTK_DIALOG(dialog));

    if (result == GTK_RESPONSE_OK || result == GTK_RESPONSE_APPLY)
     {      
        fontname = gtk_font_selection_dialog_get_font_name(
                            GTK_FONT_SELECTION_DIALOG(dialog));
        font_desc = pango_font_description_from_string(fontname);	  
        g_free(fontname);	 
     }
    gtk_widget_destroy(dialog);	
	return font_desc;
}
/**********************************
 * saisie d'un texte à travers une
 * boîte de dialogue
 * *******************************/

gchar *dlg_saisie_text2(gchar *message, const gchar *txt_actuel)
{
  GtkWidget* DlgAddText;
  GtkWidget* dialog_vbox1;
  GtkWidget* vbox1;
  GtkWidget* label1;
  GtkWidget* scrolledwindow1;
  GtkWidget* TextViewEditor;
  GtkWidget* dialog_action_area1;
  GtkWidget* cancelbutton1;
  GtkWidget* okbutton1;
  /*GtkTooltips* tooltips;*/
  GtkResponseType result;
  GtkTextBuffer *textbuffer;
  GtkTextIter iStart;
  GtkTextIter iEnd;
  gchar* sBuffer=NULL;

  /*tooltips = gtk_tooltips_new ();*/

  DlgAddText = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (DlgAddText), message);
  gtk_window_set_type_hint (GTK_WINDOW (DlgAddText), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = gtk_dialog_get_content_area (DlgAddText);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new (gettext("Text :"));
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_SHADOW_IN);

  TextViewEditor = gtk_text_view_new ();
  gtk_widget_show (TextViewEditor);
  textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (TextViewEditor));
  /* passer le texte */
  gtk_text_buffer_set_text(textbuffer,txt_actuel,-1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), TextViewEditor);
/*  gtk_tooltips_set_tip (tooltips, TextViewEditor,
    gettext( "Type here the text you want to\nadd on your drawing."), NULL);*/

  dialog_action_area1 = gtk_dialog_get_action_area (DlgAddText);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (DlgAddText), cancelbutton1, GTK_RESPONSE_CANCEL);
  gtk_widget_set_can_default (cancelbutton1, TRUE);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (DlgAddText), okbutton1, GTK_RESPONSE_OK);
  gtk_widget_set_can_default (okbutton1, TRUE);

  result = gtk_dialog_run(GTK_DIALOG(DlgAddText));
  if (result == GTK_RESPONSE_OK || result == GTK_RESPONSE_APPLY)
  {
    /* récupérer le texte tapé */
    /* On recupere l'origine du buffer */
    gtk_text_buffer_get_start_iter(textbuffer, &iStart);
    /* On recupere la fin du buffer */
    gtk_text_buffer_get_end_iter(textbuffer, &iEnd);
    /* On copie le contenu du buffer dans une variable */
    sBuffer = gtk_text_buffer_get_text(textbuffer, &iStart, &iEnd, TRUE);
	
  }/* endif Ok */
  gtk_widget_destroy(DlgAddText); 
  return sBuffer;
}
/**********************************
 * saisie d'un texte à travers une
 * boîte de dialogue
 *********************************/
gchar* dlg_saisie_text(gchar* message, const gchar* txt_actuel)
{
  GtkWidget* DlgAddText;
  GtkWidget* TextViewEditor;
  GtkResponseType result;
  GtkTextBuffer* textbuffer;
  GtkTextIter iStart;
  GtkTextIter iEnd;
  gchar* sBuffer = NULL;


  GtkBuilder *builder_dlg = gtk_builder_new();
   gtk_builder_add_from_file(builder_dlg, g_build_filename(ella_get_data_dir(), "ui/dlg_saisie_text.ui", NULL), NULL);
   DlgAddText=GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_dlg_saisie_text")); 
  gtk_window_set_title (GTK_WINDOW (DlgAddText), message);

  TextViewEditor = GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_TextViewEditor")); 
  textbuffer=GTK_TEXT_BUFFER (gtk_builder_get_object (builder_dlg, "w_textbuffer"));
  textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(TextViewEditor));
  /* passer le texte */
  gtk_text_buffer_set_text(GTK_TEXT_BUFFER(textbuffer),txt_actuel,-1);
  result = gtk_dialog_run(GTK_DIALOG(DlgAddText));
  if (result == 1 )
  {
    /* récupérer le texte tapé */
    /* On recupere l'origine du buffer */
    gtk_text_buffer_get_start_iter(textbuffer, &iStart);
    /* On recupere la fin du buffer */
    gtk_text_buffer_get_end_iter(textbuffer, &iEnd);
    /* On copie le contenu du buffer dans une variable */
    sBuffer = gtk_text_buffer_get_text(textbuffer, &iStart, &iEnd, TRUE);
    printf("texte tapé:%s\n",sBuffer);
  }/* endif Ok */
  gtk_widget_destroy(DlgAddText); 
  return sBuffer;
}

/*******************************
 * gestion simultanée des deux 
 * couleurs et du canal
 * alpha ;
 * on passe en entrée les choix
 * actuels de couleurs
 * ****************************/
GtkWidget* create_dialogColorsAlpha(GdkColor scolor, GdkColor fcolor,
                                    GdkColor pcolor, gint alpha, gint page)
{ 
  GtkWidget* dialogColors2;
  GObject* spinbuttonDlgColorAlpha_adj;
  GtkWidget* spinbuttonDlgColorAlpha;
  GtkWidget* drawingareaDlgColors;
  GtkWidget *colorselectionFillColor;
  GtkWidget *colorselectionStrokeColor;
  GtkWidget* notebook_color=NULL;
  
  GtkBuilder *builder_dlg = gtk_builder_new();
  gtk_builder_add_from_file(builder_dlg, g_build_filename(ella_get_data_dir(), "ui/dlgChooseColor.ui", NULL), NULL);
  dialogColors2=GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_dialogColors2"));
  /* passage des paramètres actuels */
  TmpSColor=scolor;
  TmpFColor=fcolor;
  TmpPageColor=pcolor;
  TmpAlpha=alpha;

  
  spinbuttonDlgColorAlpha_adj =G_OBJECT(gtk_builder_get_object (builder_dlg, "w_spinbuttonDlgColorAlphaAdj")); 
  gtk_adjustment_set_value (GTK_ADJUSTMENT(spinbuttonDlgColorAlpha_adj),TmpAlpha);
  spinbuttonDlgColorAlpha = GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_spinbuttonDlgColorAlpha")); 
  drawingareaDlgColors = GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_drawingareaDlgColors")); 
  colorselectionFillColor=GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_colorselectionFillColor")); 
  colorselectionStrokeColor=GTK_WIDGET(gtk_builder_get_object (builder_dlg, "w_colorselectionStrokeColor")); 
  notebook_color = GTK_WIDGET(gtk_builder_get_object (builder_dlg, "notebook1")); 
  gtk_notebook_set_current_page       (GTK_NOTEBOOK(notebook_color),page);
  g_signal_connect ((gpointer) colorselectionFillColor, "color-changed",
                            G_CALLBACK (on_DlgColorFColor_changed),
                            G_OBJECT (drawingareaDlgColors));
  g_signal_connect ((gpointer) colorselectionStrokeColor, "color-changed",
                            G_CALLBACK (on_DlgColorSColor_changed),
                            G_OBJECT (drawingareaDlgColors));
  g_signal_connect ((gpointer) spinbuttonDlgColorAlpha, "value_changed",
                            G_CALLBACK (on_spinbuttonDlgColorAlpha_value_changed),
                            G_OBJECT (drawingareaDlgColors));
  g_signal_connect ((gpointer) drawingareaDlgColors, "expose_event",
                            G_CALLBACK (on_drawingareaDlgColors_expose_event),
                            G_OBJECT (drawingareaDlgColors));
  return dialogColors2;
}

/* fonctions permettant de lire depuis un autre module les valeurs choisies */
gint dlg_editeur_get_alpha(void)
{
	return TmpAlpha;
}

GdkColor dlg_editeur_get_scolor(void)
{
	return TmpSColor;
}

GdkColor dlg_editeur_get_fcolor(void)
{
	return TmpFColor;
}

/*****************************
 * fonctions pour charger des
 * images
 * ***************************/
/********************************
 File Dialog item
 ********************************/
 GtkWidget* create_DialogImportImageFile(gchar *titre,
                                         const gchar *filtre1,
                                         const gchar *filtre2,
                                         const gchar *filtre3)
{
  GtkWidget* DialogOpenFile;
  GtkWidget* dialog_vbox2;
  GtkWidget* dialog_action_area2;
  GtkWidget* button1;
  GtkWidget* button2;
  GtkFileFilter* filter;
  
  filter = gtk_file_filter_new ();

  DialogOpenFile = gtk_file_chooser_dialog_new (gettext("Import an image ..."), NULL,
             GTK_FILE_CHOOSER_ACTION_OPEN,NULL,NULL, NULL);
 
  gtk_file_filter_add_pattern (filter, filtre1);
  gtk_file_filter_add_pattern (filter, filtre2);
  gtk_file_filter_add_pattern (filter, filtre3);
  gtk_file_filter_set_name (filter,gettext("images"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(DialogOpenFile),filter);

  gtk_container_set_border_width (GTK_CONTAINER (DialogOpenFile), 5);
  gtk_window_set_position (GTK_WINDOW (DialogOpenFile), GTK_WIN_POS_CENTER);
  gtk_window_set_type_hint (GTK_WINDOW (DialogOpenFile), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_icon_name (GTK_WINDOW (DialogOpenFile), "gtk-open");

  dialog_vbox2 = gtk_dialog_get_content_area (DialogOpenFile);
  gtk_widget_show (dialog_vbox2);

  dialog_action_area2 = gtk_dialog_get_action_area (DialogOpenFile);
  gtk_widget_show (dialog_action_area2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);

  button1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button1);
  gtk_dialog_add_action_widget (GTK_DIALOG (DialogOpenFile), button1, GTK_RESPONSE_CANCEL);
  gtk_widget_set_can_default (button1, TRUE);

  button2 = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (button2);
  gtk_dialog_add_action_widget (GTK_DIALOG (DialogOpenFile), button2, GTK_RESPONSE_OK);
  gtk_widget_set_can_default (button2, TRUE);

  g_signal_connect ((gpointer) DialogOpenFile, "destroy",
                    G_CALLBACK (gtk_widget_destroy),
                    NULL);
  g_signal_connect ((gpointer) button1, "clicked",
                    G_CALLBACK (gtk_true),
                    NULL);

  gtk_widget_grab_default (button2);
  return DialogOpenFile;
}

/****************************************************** 
 * fonction globale gérant les dialogues et erreurs,
 * retournant NULL si problème, sinon un pointeur sur
 * un pixbuf
 * *******************************************************/
GdkPixbuf* dlg_import_image(gint e_w, gint e_h)
{
	GdkPixbuf* pb = NULL;
	GtkWidget* dlg;
	gint reponse;
	gchar* path_to_file;
    gchar* path_to_temp_dir;
    gchar* short_name;
    gchar* path_target;	
	gint width, height;
	GdkPixbufFormat* fmt;
	GError* error = NULL;
	
	dlg=create_DialogImportImageFile (gettext("Import an image"),
            "*.jpg","*.png","*.jpeg");
	reponse=gtk_dialog_run(GTK_DIALOG(dlg));
	if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return NULL;
      }
	/* quel fichier ? */
	path_to_file= gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
	gtk_widget_destroy(dlg);
	/* vérifier contenu et validité du fichier */
	fmt=gdk_pixbuf_get_file_info(path_to_file,&width,&height);
	if (fmt==NULL)
	  {
	   printf("erreur lecture ! \n");
	   return NULL;
      }
	else
	  {
		  printf("width %d height %d \n",width, height);
	  }
	/* contraindre les dimensions !!! */
	if ((width>e_w)||(height>e_h))
	      {
			  width=e_w;
			  height=e_h;
		  }
	pb=gdk_pixbuf_new_from_file_at_scale(path_to_file,width,height,TRUE, &error);
	/* send file to library */
	path_to_temp_dir= (gchar *)g_get_tmp_dir();
	/* transférer fichier vers répertoire temporaire */
    short_name=(gchar *)g_basename(path_to_file);
    path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
    ella_files_copy(path_to_file,path_target);
	/* reste à mettre à jour la future list_store ! */
    add_tree_list_biblio(pBiblio,1,short_name,path_target,TRUE);
    /* g_free(path_to_file_to_import); ne pas faire free
    g_free(path_to_temp_dir);nepas faire free */
    g_free(path_target);
	/* si erreur, sortir NULL */	
	g_free(path_to_file);
	return pb;
}
