/*
copyright 2010 Lucas Baudin <xapantu@gmail.com>
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialogs/page_size.h"

GtkWidget* create_dialog_page_size(gint width, gint height, gint margins)
{
    
    GtkBuilder* builder = gtk_builder_new();
    gtk_builder_add_from_file(
                builder,
                g_build_filename(ella_get_data_dir(), "ui/page_size.ui", NULL),
                NULL);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (builder,
                                                                        "width")),
                               width);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (builder,
                                                                        "margins")),
                               margins);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (builder,
                                                                        "height")),
                               height);
    /* Let's connect the signals! */
    g_signal_connect ((gpointer) gtk_builder_get_object (builder,
                                                         "width"),
                                            "value-changed",
                                            G_CALLBACK (on_spinWidth_changed),
                                            NULL);

    g_signal_connect ((gpointer) gtk_builder_get_object (builder,
                                                         "height"),
                                            "value-changed",
                                            G_CALLBACK (on_spinHeight_changed),
                                            NULL);

    g_signal_connect ((gpointer) gtk_builder_get_object (builder,
                                                         "margins"),
                                            "value-changed",
                                            G_CALLBACK (on_spinMargins_changed),
                                            NULL);

    return GTK_WIDGET(gtk_builder_get_object(builder, "page_size"));
}

void
on_spinWidth_changed                  (GtkWidget        *spin,
                                        gpointer         user_data)
{
    gint value;

    value =(gint)gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

    /* Test if the value is useable. This shouldn't happen since the spin button
     * is set with a no 0 adjustement. But we can check, even if it is useless. */
    if ((value==0)||(value<0))
    {
        /* problem */
        printf("False value! \n");
        value=550;/* default value*/
        /* update value on the widget */
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gdouble) 550);
    }
    editor_set_page_width(value);
}
void
on_spinHeight_changed                  (GtkWidget        *spin,
                                        gpointer         user_data)
{
    gint value;

    value =(gint)gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

    /* Test if the value is useable. This shouldn't happen since the spin button
     * is set with a no 0 adjustement. But we can check, even if it is useless. */
    if ((value==0)||(value<0))
    {
        /* problem */
        printf("False value! \n");
        value=550;/* default value*/
        /* update value on the widget */
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gdouble) 550);
    }
    editor_set_page_height(value);
}
void
on_spinMargins_changed                  (GtkWidget        *spin,
                                        gpointer         user_data)
{
    gint value;

    value =(gint)gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

    /* Test if the value is useable. This shouldn't happen since the spin button
     * is set with a no 0 adjustement. But we can check, even if it is useless. */
    if ((value==0)||(value<0))
    {
        /* problem */
        printf("False value! \n");
        value=EDITOR_CANVAS_MARGINS;/* default value*/
        /* update value on the widget */
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gdouble) EDITOR_CANVAS_MARGINS);
    }
    editor_set_page_margins(value);
}
