/*
copyright 2010 Lucas Baudin <xapantu@gmail.com>
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "dialogs/preferences.h"

GtkWidget* create_dialogPrefs (gint w, gint h,gint marges, gint fps,
                               gint footage, GdkColor pcolor,gboolean f_archive,
                               gint zoom)
{
   
    
    GtkBuilder *builder = gtk_builder_new ();
    gtk_builder_add_from_file (builder, g_build_filename(ella_get_data_dir(), "ui/preferences.ui", NULL), NULL);

    /* We'll return this pointer at the end of the function. */
    GtkWidget      *dialog;

    /* Get the dialog from the glade file. */
    dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_settings"));

    /* The default page width. */
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (
                                                    builder,
                                                    "spinWidth")
                                                ),
                                                ella_config_get_value_double("PAGE_WIDTH"));

    /* The default page height. */
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (
                                                    builder,
                                                    "spinHeight")
                                                ),
                                                (gdouble) h);

    /* The default margins. */
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (
                                                    builder,
                                                    "spinMargin")
                                                ),
                                                (gdouble) marges);

    /* If we save the media into the file or not. */
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (
                                                builder,
                                                "checkbuttonArchiveBiblioAuto")
                                            ),
                                            f_archive);
    /* check button for background(s saving */
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (
                                                builder,
                                                "checkbutton_export_background")
                                            ),
                                             editor_get_background_export());
    /* FIXME : not used yet, footage is _always_ 512. */
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (
                                                    builder,
                                                    "spinbuttonFootage")
                                                ),
                                                footage);

    /* The default FPS. */
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (gtk_builder_get_object (
                                                    builder,
                                                    "spinbuttonDefaultFPS")
                                                ),
                                                fps);

    /* defaut zoom for timeline */
    gtk_combo_box_set_active (GTK_COMBO_BOX(gtk_builder_get_object (
                                                    builder,
                                                    "combobox")),2-zoom);
    /* the color selection */
    gtk_color_selection_set_current_color  (GTK_COLOR_SELECTION (gtk_builder_get_object (builder, "colorselection1")), &pcolor);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object (builder, "ask_before_quit")), ella_config_get_value_int("DONT_ASK_BEFORE_QUIT"));

    gtk_builder_connect_signals (builder, NULL);

    return dialog;
}


void
on_entryWidth_changed                  (GtkWidget        *spin,
                                        gpointer         user_data)
{
    gint value;

    value =(gint)gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

    /* test if the value is useable */
    if ((value==0)||(value<0))
    {
        /* problem */
        printf("False value! \n");
        value=550;/* default value*/
        /* update value on the widget */
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gdouble) 550);
    }
    editor_set_default_page_width(value);
}


void
on_entryHeight_changed                 (GtkWidget        *spin,
                                        gpointer         user_data)
{
    gint value;

    value =(gint)gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

    /* test if the value is useable */
    if ((value==0)||(value<0))
    {   /* problem */
        printf("False value! \n");
        value=480;/* default value*/
        /* update value on the widget */
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gdouble) 440);
    }
    editor_set_default_page_height(value);
}

void
on_entryMargins_changed                 (GtkWidget        *spin,
                                        gpointer         user_data)
{
    gint value;

    value =(gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin));
    /* test if the value is usable */
    if ((value==0)||(value<0))
    {
        /* problem */
        printf("False value! \n");
        value=80;/* default value*/
        /* update value on the widget */
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),  80);
    }
    editor_set_default_margins((gdouble)value);
}

void
on_spinbuttonFootage_changed (GtkWidget *spb,
                                        gpointer         user_data)
{
    /* FIXME : why do we change the timeline zoom for the footage?! */
    /* timeline_set_default_movie_zoom(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spb))); */
}
void
on_combobox1PrefsZoom_changed          (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    timeline_set_default_movie_zoom(gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
}
void
on_checkbuttonArchiveBiblioAuto_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    set_autoarchive_biblio(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(togglebutton)));	
}

void 
on_checkbutton_export_background_toggled(GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	editor_set_background_export(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(togglebutton)));
	if(editor_get_background_export())
	   ella_config_set_value_int("EXPORT_BACKGROUND", 1);
	     else
	       ella_config_set_value_int("EXPORT_BACKGROUND", 0);
}

void
on_spinbuttonDefaultFPS_changed        (GtkWidget *spb,
                                        gpointer         user_data)
{
    timeline_set_default_movie_freq(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spb)));
}

void
on_color_changed                 (GtkColorSelection        *colorselection,
                                        gpointer         user_data)
{
    GdkColor color;
    gtk_color_selection_get_current_color(colorselection, &color);
    editor_set_default_page_color(color);
}

void settings_on_ask_before_quit_value_changed(GtkToggleButton* check, gpointer user_data)
{
    ella_config_set_value_int("DONT_ASK_BEFORE_QUIT", gtk_toggle_button_get_active(check));
}
