/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "editor/editor.h"

/* codes des objets :
 * B = image bitmap
 * F = page background
 * K =  layer
 * C = ellipse/circle
 * L = line
 * R = rectangle/box
 * P = path (filled)
 * U = path (not filled)
 * D = button
 */

#include "./icones_rcs/editor_img.xpm"

/* liste (pile) stockant les noms d'onjets et les pixbufs associés.
GSList *pile_images=NULL; */
EllaUIPanel* panel = NULL;
 gpointer *widgets_styles = NULL;
 GList* EllaItemList = NULL;
 /* éditeur */
 GtkWidget *labelRotationAngle;
 GtkWidget *labelMousePos;
 GtkWidget *labelMouseTips;
 GtkToolItem *pEditorModeItem; /* obligatoire pour récupérer le pointeur pour forcer l'activation */
 EllaEditorMode EditorMode = EDITOR_MODE_SELECT; /* 0= mode sélection */
 gboolean fAutoPlacement=FALSE;/* flag for autoplacement of items */
 gboolean fJustSelected=FALSE;/* in order to avoid auto call to change color of a multislection */
 gdouble AutoPLacementScale=20.0;/* scale for the virtual scale used for autplacement of items */
 gboolean SelectedArea; /* indique si une sélection est en cours */
 gdouble xev,yev;/* obligé car je ne me tire pas des pointeurs dans certains cas */
 gdouble x1sel,y1sel,x2sel,y2sel,wsel,hsel; /* coordonnées utilisées de la sélection courante */
 gdouble x1move,y1move; /* coordonnées pour la translation des items */
 gdouble x1drw,y1drw,x2drw,y2drw,x_delta,y_delta; /* coordonnées utilisées pour les tracés */
 gboolean fdrawing = FALSE;/* flag pour indiquer s'il faut tracer ou pas */
 gboolean fDontRespondToClick = FALSE;
 gboolean fresizing = FALSE; /* flag indiquant si l'on doit suivre la souris */
 gboolean frotation = FALSE; /* flag indiquant si une rotation est en cours */
 gboolean fDoubleClick = FALSE; /* flag in order to cancel a release event after a double-click */
 gboolean fmoved=FALSE; /* flag indiquant si la souris a bougé */
 gboolean fExportBackground = FALSE; /* flag for exporting or not of move's background */
 GList *pSelectedItemsList; /* liste des items, mais VALIDE, sans le fond et l'origine */
 gboolean fClipboardFull=FALSE; /* flag indiquant si le presse-papiers contient un élément */
 gdouble editor_canvas_max_width = 550; /* largeur maximum du canvas de l'éditeur vectoriel */
 gdouble editor_canvas_max_height = 440; /* hauteur maximum du canvas de l'éditeur vectoriel */
 gdouble editor_canvas_max_margins = EDITOR_CANVAS_MARGINS;
 guint ItemCounter=0; /* compte le nombre d'objets et permet de les numéroter/classer */
 /* couleurs */
 GdkColor PenColor ={0, 0, 0, 0};/* noir par défaut */
 GdkColor FillColor ={0, 65000, 0, 0};/* rouge par défaut */;
 GdkColor PageColor={0, 65535, 65535, 65535}; /* fond de page blanc par défaut */
 gint Alpha = 100;/* canal alpha*/
 /* styles de traits */
 gdouble LineWidth = 1.0; /* 1 pixel par défaut */
 gdouble XLineRound = 1.0;  /* arrondi horizontal */
 gdouble YLineRound = 1.0; /* arrondi vertical */
 /* fontes */
 PangoFontDescription* FntDesc=NULL;
 /* valeurs par défaut */
 gdouble editor_default_width,  editor_default_height, editor_default_margins;
 GdkColor editor_defaut_page_color;
 /* flag indiquant si c'est un 1er appel aux entrées de l'onglet "propriétés" */
 gboolean  fPropertiesEntries=TRUE;/* pour éviter l'appel lors de l'initialisation des champs */
 gdouble xmem,ymem,wmem,hmem;/* mémoriser les valeurs pour annuler un changement */
 gint PropertiesChange=0;/* stocke le type de changement : 0=rien, 1=position, 2=dimensions */
 gint r_rempli_mem,v_rempli_mem,b_rempli_mem,r_cont_mem,v_cont_mem,b_cont_mem,a_rempli_mem,a_cont_mem;
 gdouble ep_ligne_mem;

 GtkWidget* pWinMemo;

 gboolean fFirstCall = TRUE;
 gboolean fFirstCallPropertiesStyles = TRUE;
 gint NbElementsSelectedItemsList;
 gint NbCallsToReject;
 /* pointers on color preview of notification area */
  GtkWidget *preview_Pp;
  GtkWidget *preview_Ap;
 /* vars for path objects */
 gdouble xmem_path,ymem_path;
 gint NbElemPath=0;
 GSList *ListElemPath=NULL;
 

/* utilities */
/***********************************
 * store flag for background
 * export for svg
 * ********************************/
void editor_set_background_export(gboolean flag)
{
	fExportBackground = flag;
}
/***********************************
 * get flag for backgroun's export
 * ********************************/
gboolean editor_get_background_export(void)
{
	return fExportBackground;
}

/******************************************
 * with a coordinate, computes a new
 * coordinate indexed on the autoplacement
 * scale
 * ****************************************/
gdouble get_autoplaced_coordinate(gdouble valeur, gdouble pas_grille)
{
	valeur=(gdouble) ((gint)valeur / (gint)pas_grille);/* division entière */
	valeur=(gdouble)valeur*pas_grille;/* arrondie au plus proche inférieur */
	return valeur;
}
/*************************************
 * stocke en local le pointeur sur
 * les widgets des styles
 * **********************************/
void editor_store_pointer_to_styles(gpointer *p)
{
	widgets_styles=p;
}

/**********************************
 * fonction ad_hoc pour obtenir
 * un pointeur sur la GSList de
 * la pile des images
 * *******************************/
GSList *editor_get_images_stack(void)
{
	return pile_images;
}

/**********************************
 * specific function to get a 
 * pointer on button stack
 * *******************************/
GSList* editor_get_buttons_stack(void)
{
	return StackButtons;
}
/**********************************
 * management of clips list
 * *******************************/
/*********************************
 * add a clip item on stack
 * of clips
 * ******************************/
GSList* add_pile_clips(GSList* list, gchar* titre, gchar* path_to_file)
{
 GSList* temp=NULL;
 stack_clips* temp_data=NULL;
 
  temp_data = g_malloc(sizeof(stack_clips));
  temp_data->clip_name = g_strdup(titre); 
  temp_data->clip_file = g_strdup(path_to_file); 
 
  temp = g_slist_append (list,temp_data);
  return temp;	
}

/**********************************
 * specific function to get a 
 * pointer on clips stack
 * *******************************/
GSList* editor_get_clips_stack(void)
{
	return StackClips;
}

/***********************************
 extraire une sous-chaîne
 attention : il faut libérer
 la mémoire APRES avec 
g_free (output);
***********************************/
gchar *g_substr ( const gchar* string, gint start,gint  end)
{
  gsize len = (end - start + 1);
  gchar *output = g_malloc0 (len + 1);
  return g_utf8_strncpy (output, &string[start], len);
}
/****************************************
 * convertir données couleurs
 * séparées RGB+A (en valeur flottante 0-1)
 * vers du 32 bits RGB ;
 * thanks to : Damon Chaplin (goocanvas)
 * ***************************************/
guint
convert_color_to_rgba (gdouble red, gdouble green, gdouble blue, gdouble alpha)
{
  guint red_byte, green_byte, blue_byte, alpha_byte;

  red_byte = red * 256;
  red_byte -= red_byte >> 8;

  green_byte = green * 256;
  green_byte -= green_byte >> 8;

  blue_byte = blue * 256;
  blue_byte -= blue_byte >> 8;

  alpha_byte = alpha * 256;
  alpha_byte -= alpha_byte >> 8;

  return (red_byte << 24) + (green_byte << 16) + (blue_byte << 8) + alpha_byte;
}
/************************************
 *there is a selection ? 
 * **********************************/
gboolean editor_get_selection_state(void)
{
	if (pSelectedItemsList!=NULL)
	   return TRUE;
	else
	   return FALSE;
}


/*************************************
 * ajouter un nouvel item, en tenant
 * compte de son NOM pour incrémenter
 * intelligemment le compteur d'objets
 * ***********************************/
void editor_add_new_item(gchar *title)
{
  gchar *compteur;
  gint count;
    
  compteur=g_substr(title,1,strlen(title)-1);
  count=(int)strtol(compteur, NULL, 10);
  if (count>ItemCounter)
    ItemCounter=count;		
}
void editor_inc_item_counter(void)
{
	ItemCounter++;
}
gint editor_get_item_counter(void)
{
	return ItemCounter;
}
/***************************
 * détruit liste des items,
 * en particulier des 
 * images
 * *************************/
void editor_reset_item_counter(void)
{
	ItemCounter =   0;
	pile_images=   NULL;
}
/*****/
void editor_set_page_width(gdouble w)
{
    ella_canvas_set_page_width(canvas, w);
    editor_canvas_max_width=w;
}	
void editor_set_page_height(gdouble h)
{
    ella_canvas_set_page_height(canvas, h);
    editor_canvas_max_height=h;
}	
void editor_set_page_margins(gdouble m)
{
  editor_canvas_max_margins = m;
}
void editor_set_default_page_width(gdouble w)
{
    editor_default_width=w;
    ella_config_set_value_double("PAGE_WIDTH", w);
}	

void editor_set_default_page_height(gdouble h)
{
    editor_default_height=h;
    ella_config_set_value_double("PAGE_HEIGHT", h);
}
void editor_set_default_margins(gdouble h)
{
    editor_default_margins=h;
    ella_config_set_value_double("PAGE_MARGINS", h);
}
gdouble editor_get_default_page_width(void)
{
  return editor_default_width;
}	
gdouble editor_get_default_page_height(void)
{
  return editor_default_height;
}
gdouble editor_get_default_margins(void)
{	
  return editor_default_margins;
}
/********************
 * changer couleur
 * de fond de la
 * scène
 * *****************/
void editor_set_page_color(gint r,gint v, gint b)
{
	PageColor.red = r;
	PageColor.green = v;
	PageColor.blue = b;
	/* TODO: change the canvas color. */
}
/* par défaut */
void editor_set_default_page_color(GdkColor color)
{
	editor_defaut_page_color = color;
    ella_config_set_value_int("PAGE_COLOR_RED", editor_defaut_page_color.red);
    ella_config_set_value_int("PAGE_COLOR_GREEN", editor_defaut_page_color.green);
    ella_config_set_value_int("PAGE_COLOR_BLUE", editor_defaut_page_color.blue);
}
/* par défaut */
void editor_set_default_page_color_rgb(gint r,gint v, gint b)
{
	editor_defaut_page_color.red=r;
	editor_defaut_page_color.green=v;
	editor_defaut_page_color.blue=b;
    ella_config_set_value_int("PAGE_COLOR_RED", editor_defaut_page_color.red);
    ella_config_set_value_int("PAGE_COLOR_GREEN", editor_defaut_page_color.green);
    ella_config_set_value_int("PAGE_COLOR_BLUE", editor_defaut_page_color.blue);
}
GdkColor editor_get_default_page_color(void )
{
	return editor_defaut_page_color;
}
gdouble editor_get_page_width(void)
{
 	return  editor_canvas_max_width;
}
gdouble editor_get_page_height(void)
{
 	return  editor_canvas_max_height;
}
gdouble editor_get_page_margins(void)
{
	return (gdouble) editor_canvas_max_margins;
}
GdkColor editor_get_page_color(void)
{
	return  PageColor;
}

/**********************************
 * redraw the color preview of
 * the current selected color
 * set, on the notification area
 * of the editor ;
 * input : 2 pointers on drawing
 *  areas
 * *******************************/
void editor_editeur_set_color_set_preview(GtkWidget *win1, GdkColor TmpPpColor,GtkWidget *win2, GdkColor TmpApColor)
{
  cairo_t *cr1,*cr2;
  /* foreground */
  /*GdkColor TmpPpColor;*/
  /* background */
  /*GdkColor TmpApColor;*/
  
  if (win1==NULL)
      return;
  if (win2==NULL)
      return;
	  
  cr1 = gdk_cairo_create(gtk_widget_get_window(win1));	
  cr2 = gdk_cairo_create(gtk_widget_get_window(win2));	
  cairo_set_source_rgb(cr1,(gdouble)(TmpPpColor.red/65535.0),
                         (gdouble)(TmpPpColor.green/65535.0),
						 (gdouble)(TmpPpColor.blue/65535.0));
  cairo_rectangle(cr1,0,0,12,20);
  cairo_fill(cr1);
  
  cairo_set_source_rgb(cr2,(gdouble)(TmpApColor.red/65535.0),
                         (gdouble)(TmpApColor.green/65535.0),
						 (gdouble)(TmpApColor.blue/65535.0));
  cairo_rectangle(cr2,0,0,12,20);
  cairo_fill(cr2);
   
  cairo_destroy(cr1);
  cairo_destroy(cr2);
}
/* store pointers for preview */
void editor_store_preview(GtkWidget *da1, GtkWidget *da2)
{
	preview_Pp=da1;
	preview_Ap=da2;
}								

/***********************************
 * choix de la couleur de fond de
 * page depuis le menu déroulant
 * ********************************/
void editor_set_page_color_menu(void)
{
    GdkColor color;
    GtkColorSelectionDialog* dialog_couleur;
    gint reponse;
    GtkWidget* color_selection;

    dialog_couleur = GTK_COLOR_SELECTION_DIALOG(gtk_color_selection_dialog_new(_("Scene Background")));

    reponse = gtk_dialog_run(GTK_DIALOG(dialog_couleur));

    if(reponse == GTK_RESPONSE_OK)
    {
        color_selection = gtk_color_selection_dialog_get_color_selection(dialog_couleur);

        gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(color_selection),
                                              &color);
        ella_canvas_set_page_color_red(canvas, ((double)color.red) / 0xFFFF);
        ella_canvas_set_page_color_green(canvas, ((double)color.green) / 0xFFFF);
        ella_canvas_set_page_color_blue(canvas, ((double)color.blue) / 0xFFFF);
    }
    gtk_widget_destroy(GTK_WIDGET(dialog_couleur));
}

/***************************
 * monter et descendre des
 * objets sélectionnés ;
 * attention : met au
 * sommet ou en bas, 
 * directement
 * *************************/
void editor_raise(GList *selection)
{
}
/*****************/
void editor_lower(GList *selection)
{
}

/************************
 * dupliquer les objets
 * de la sélection
 * *********************/
void editor_duplicate(void)
{
}

/**********************************
 * fonction pour calculer la
 * nouvelle dimension horizontale
 * après redimensionnement
 * *******************************/
gdouble resize_compute(gdouble before, gdouble after)
{
	gdouble t;
	t= after/before;/* >1 = plus grand ; <1 = plus petit */
	return t;
}

/*************************************************
 * obtain a SVG path sequence from internal path
 * description
 * **********************************************/
gchar *ella_path_to_svg_path(GSList *list, gboolean remplir)
{
   gchar *path=NULL;
   gchar *part;
   GSList *temp;
   gdouble x1,y1;

   if((list != NULL) && (g_slist_length(list) > 0))
   {
      /* we pass all elements to path string first element, a M instruction */
      temp=g_slist_nth(list, 0);
      path=g_strdup_printf("M %d,%d ", (gint)((ElemPath *)(temp->data))->x,
                                       (gint)((ElemPath *)(temp->data))->y);
      x1=((ElemPath *)(temp->data))->x;
      y1=((ElemPath *)(temp->data))->y;								
      /* other elements , L instruction */
      temp = g_slist_nth(list, 1);
      while(temp)
      {
         part=g_strdup_printf("L %d,%d ", (gint)((ElemPath *)(temp->data))->x,
                                          (gint)((ElemPath *)(temp->data))->y);
         /* concat a new string */
         path=g_strconcat (path,part , NULL); 
         g_free(part);
         part=NULL;
         temp = g_slist_next(temp);
      }  
      /*if (remplir)
        { we must add and element to close the path's outline */
         part=g_strdup_printf("L %d,%d ",(gint)x1,(gint)y1);
         path=g_strconcat (path,part , NULL); 
         g_free(part);		
		 part=NULL;   	
      /* }	 */ 		  
   }	
   return path;
}
EllaCanvas *canvas = NULL;
/**************************************
 * initialisation du canvas de
 * l'éditeur
 * entrée : scrolledwindow à laquelle
 * est accroché le canvas,
 * sortie : pointeur sur le nouveau
 * root du canvas ainsi créé.
 * locale sauf pour les constantes
 * ***********************************/
EllaCanvas *init_canvas_editor(GtkWidget* win, EllaUIPanel* panel_)
{
    canvas = ella_canvas_new();
    ella_canvas_set_tool(canvas, ELLA_TOOL_SELECT);

    /* Page size */
    ella_canvas_set_page_width(canvas, editor_get_default_page_width());
    ella_canvas_set_page_height(canvas, editor_get_default_page_height());

    /* Page color */
    ella_canvas_set_page_color_red(canvas,
                                         ((double)editor_get_default_page_color().red) / 0xFFFF);
    ella_canvas_set_page_color_green(canvas,
                                         ((double)editor_get_default_page_color().green) / 0xFFFF);
    ella_canvas_set_page_color_blue(canvas,
                                         ((double)editor_get_default_page_color().blue) / 0xFFFF);

    /* And add our canvas to the scrolled window */
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(win), GTK_WIDGET(canvas));

    return canvas;
}

EllaUIPanel* get_properties_panel(void)
{
    return panel;
}
