/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef H_EDITOR_20101130
#define H_EDITOR_20101130

/*translations */
#include <locale.h>
#include <libintl.h>

#include "editor/editor_callbacks.h"

#include "ella-user_config.h"

#include "widgets/properties_panel.h"
#include "ella-widgets.h"

/*
GooCanvasItem* rootEditorItems;
GooCanvasItem* rootGroup;
*/


EllaCanvas *init_canvas_editor(GtkWidget* win, EllaUIPanel* panel_);

void editor_reset_item_counter (void);
void editor_inc_item_counter   (void);
gint editor_get_item_counter   (void);
void editor_add_new_item(gchar* title);
gchar* g_substr(const gchar* string, gint start, gint end);
guint
convert_color_to_rgba (gdouble red, gdouble green, gdouble blue, gdouble alpha);
void editor_wipe_page  (void);
GSList* editor_get_images_stack(void);
void editor_lower(GList* selection);
void editor_raise(GList* selection);
void editor_duplicate (void);
void editor_react_properties_entries(GtkEditable* editable, gint champ);
void editor_react_properties_styles_entries(GtkEditable* editable,
											gpointer*    user_data);
void editor_react_styles_area_fillcolor_expose_event(GtkWidget*      widget,
                                                     GdkEventExpose* event,
                                                     gpointer*       user_data);
void editor_react_styles_area_strokecolor_expose_event(GtkWidget* widget,
                                                       GdkEventExpose* event,
                                                       gpointer* user_data);
gboolean
editor_on_area_fillcolor_key_press_event        (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data);
										
gboolean
editor_on_area_strokecolor_key_press_event        (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data);																							   
gboolean editor_get_selection_state    (void);
void editor_store_pointer_to_styles(gpointer* p);
void on_checkbuttonEditorAttractionMagnetic_toggled(GtkToggleButton* togglebutton,
                                                    gpointer user_data);
void editor_store_preview(GtkWidget* da1, GtkWidget* da2);
void editor_editeur_set_color_set_preview(GtkWidget* win1, GdkColor TmpPpColor,
                                          GtkWidget* win2, GdkColor TmpApColor);
gboolean editor_set_align(gint h, gint v, gint z);

void choix_couleurs(gint page);
void selection_raz(void);

gchar* ella_path_to_svg_path(GSList* list, gboolean remplir);

gdouble get_autoplaced_coordinate(gdouble valeur, gdouble pas_grille);

void set_mode_select(gboolean effacer);

extern gpointer*  widgets_styles;
extern GtkWidget* pWinMemo;

extern GtkWidget* labelRotationAngle;
extern GtkWidget* labelMousePos;
extern GtkWidget* labelMouseTips;
 
extern guint EditorMode; /* 0 = mode selection. */
/* Coordonnees utilisees pour les traces. */
extern gdouble x1drw;
extern gdouble y1drw;
extern gdouble x2drw;
extern gdouble y2drw;
extern gdouble x_delta;
extern gdouble y_delta;
/* Flag for autoplacement of items. */
extern gboolean fAutoPlacement;
/* In order to avoid auto call to change color of a multislection. */
extern gboolean fJustSelected;
/* Scale for the virtual scale used for autplacement of items. */
extern gdouble AutoPLacementScale;
/* Indique si une selection est en cours. */
extern gboolean SelectedArea;
/* Oblige car je ne me tire pas des pointeurs dans certains cas. */
extern gdouble xev;
extern gdouble yev;
/* Coordonnees utilisees de la selection courante. */
extern gdouble x1sel;
extern gdouble y1sel;
extern gdouble x2sel;
extern gdouble y2sel;
extern gdouble wsel;
extern gdouble hsel; 
/* Coordonnees pour la translation des items. */
extern gdouble x1move;
extern gdouble y1move; 
/* Flag pour indiquer s'il faut tracer ou pas. */
extern gboolean fdrawing;
extern gboolean fDontRespondToClick;
/* Flag indiquant si l'on doit suivre la souris. */
extern gboolean fresizing;
/* Flag indiquant si une rotation est en cours. */
extern gboolean frotation;
/* Flag in order to cancel a release event after a double-click. */
extern gboolean fDoubleClick;
/* Flag indiquant si la souris a bouge. */
extern gboolean fmoved;
/* Liste des items, mais VALIDE, sans le fond et l'origine */
extern GList *pSelectedItemsList;
/* Flag indiquant si le presse-papiers contient un element. */
extern gboolean fClipboardFull;
/* Largeur maximum du canvas de l'editeur vectoriel. */
extern gdouble editor_canvas_max_width;
/* Hauteur maximum du canvas de l'editeur vectoriel. */
extern gdouble editor_canvas_max_height; 

/* Colors. */
extern GdkColor PenColor;   /* Noir par defaut. */
extern GdkColor FillColor;  /* Rouge par defaut. */
extern GdkColor PageColor;  /* Fond de page blanc par defaut. */
extern gint Alpha;          /* Canal alpha. */
extern gdouble xmem_path;
extern gdouble ymem_path;
extern gint NbElemPath;
extern GSList* ListElemPath;


gdouble resize_compute(gdouble before, gdouble after);

GSList* editor_get_clips_stack(void);
GSList* add_pile_clips(GSList* list, gchar* titre, gchar* path_to_file);

GSList* editor_get_buttons_stack(void);				 

extern gboolean fFirstCall;
extern gboolean fFirstCallPropertiesStyles;
extern gint NbElementsSelectedItemsList;
extern gint NbCallsToReject;
/* Pointers on color preview of notification area. */
extern GtkWidget *preview_Pp;
extern GtkWidget *preview_Ap;



extern PangoFontDescription* FntDesc;
/* Flag indiquant si c'est un 1er appel aux entrees de l'onglet "proprietes". */
extern gboolean fPropertiesEntries; /* Pour eviter l'appel lors de l'initialisation des champs. */
/* Memoriser les valeurs pour annuler un changement. */
extern gdouble wmem;
extern gdouble hmem;
/* Stocke le type de changement : 0 = rien, 1 = position, 2 = dimensions. */
extern gint PropertiesChange; 


extern gdouble LineWidth;  /* 1 pixel par defaut. */
extern gdouble XLineRound; /* Arrondi horizontal. */
extern gdouble YLineRound; /* Arrondi vertical. */
 
/* Maybe they are useless here. */
extern EllaCanvas* canvas;
/* Obligatoire pour recuperer le pointeur pour forcer l'activation. */
extern GtkToolItem* pEditorModeItem; 
extern GList* EllaItemList;


void editor_set_default_page_width      (gdouble w);
void editor_set_default_page_height     (gdouble h);
void editor_set_default_page_color      (GdkColor color);

void editor_set_default_page_color_rgb  (gint r, gint v, gint b);
void editor_set_default_margins         (gdouble h);
gdouble editor_get_default_page_width   (void);
gdouble editor_get_default_page_height  (void);
gdouble editor_get_default_margins      (void);
GdkColor editor_get_default_page_color  (void);

void editor_set_page_color_menu         (void);

void editor_set_page_color              (gint r, gint v, gint b);
void editor_set_page_width              (gdouble w);
void editor_set_page_height             (gdouble h);
void editor_set_page_margins            (gdouble m);

gdouble editor_get_page_width           (void); 
gdouble editor_get_page_height          (void);
gdouble editor_get_page_margins         (void);

GdkColor editor_get_page_color          (void);

/* FIXME : this function shouldn't be used, it is a dirty hack I introduce to
 * port code faster, new code mustn't use it. */
EllaUIPanel* get_properties_panel(void);

GList* editor_get_all_items(void);

void editor_resize_selection(gdouble width, gdouble height);

void editor_set_background_export(gboolean flag);
gboolean editor_get_background_export(void);
void editor_on_frame_changed(GtkWidget* canvas, gint frame);

#endif
