/********************************/
/* symboles, constantes, types,*/
/* utilisés par Ella          */
/*****************************/
#ifndef H_ELLASYMS_20101130
#define H_ELLASYMS_20101130

#include <gtk/gtk.h>

#define LONG_STRING 64000
#define PI_PERSO 3.14159265358979323844
/* header des fonctions de la timeline */
#define MAX_FRAMES 1024
#define MAX_LAYERS 32
#define RULER_WIDTH 8
#define RULER_HEIGHT 24
/* constantes pour le contenu du descripteur des images */
#define ELLA_FRAME_NON_USED 0
#define ELLA_FRAME_USED_EMPTY 1
#define ELLA_FRAME_KEY_EMPTY 2
#define ELLA_FRAME_KEY_USED 4
#define ELLA_FRAME_MOVEMENT 8
#define ELLA_FRAME_ZOOM 16
#define ELLA_FRAME_ROTATION 32
#define ELLA_FRAME_LUMINANCE 64
#define ELLA_FRAME_ALPHA 128
#define ELLA_FRAME_KEY_USED_FOR_SOUND 256
#define ELLA_FRAME_MORPHING 512
#define ELLA_FRAME_PATH_MOTION 1024

#define MAX_TOOLS 7 /* nombre total d'outils sous forme de radio boutons */

#define EDITOR_MAX_WIDTH 1024 /* largeur maxi de la zone de dessin */
#define EDITOR_MAX_HEIGHT 1024 /* hauteur maxi de la zone de dessin */
#define EDITOR_CANVAS_MARGINS 80 /* marge pour faire le bord de la zone de dessin */
#define MAX_FOOTAGE 16384 /* nombre maximum d'images */
/* modes d'action de l'éditeur */

typedef enum
{
    EDITOR_MODE_SELECT,
    EDITOR_MODE_RESIZE,
    EDITOR_MODE_ROTATE,
    EDITOR_MODE_LINE,
    EDITOR_MODE_PATH,
    EDITOR_MODE_BOX,
    EDITOR_MODE_CIRCLE,
    EDITOR_MODE_TEXT,
    EDITOR_MODE_IMAGE,
    EDITOR_MODE_MOVE
} EllaEditorMode;
/* editor's tools */
#define EDITOR_TOOL_SELECT 2
/***************************
 * drag n drop
 * ************************/
enum
{
    TARGET_STRING,
    TARGET_URL,
    TARGET_TYPE_PIXMAP
};

/* champs des entrées de l'onglet propriétés */	
enum
{
    FIELD_X,
    FIELD_Y,
    FIELD_W,
    FIELD_H
};
/* codes des annulations de l'onglet propriétés */
enum
{
    PROP_CANCEL_NONE,
    PROP_CANCEL_POS,
    PROP_CANCEL_SIZE
};
/* layer type */	  
enum
{
    DRAWING_LAYER,
    SOUND_LAYER,
    GUIDE_LAYER
};	  	  

enum
{
    LAYER_NAME_COLUMN,
    LAYER_VISIBLE_COLUMN,
    LAYER_VISIBLE_TEXT,
    LAYER_ICON_TYPE,  
    LAYER_GROUP_DRAW, 
    LAYER_MOVEMENT_INDEX,
    LAYER_TYPE,
    N_COLUMN
};

/* structure contenant la description des effets */
typedef struct
{
    guint keyframe; /* # of frame, PC coordinates */
    gboolean empty;/* flag empty key-frame ? */
    gboolean effect_motion; /* flags */
    gboolean effect_rotation;
    gboolean effect_zooming;
    gboolean effect_alpha;
    gboolean effect_lumi;
    gboolean effect_morphing;
    gboolean sound;/* flag en cas de présence d'une piste son */
    gchar* sound_file;/* name of the sound file used at this keyframe */
    gint x_move; /* translation en pixels */
    gint y_move;
    gint zoom;/* de 0 à x % */
    gdouble rotation;/* en degrés */  
    gint alpha; /* 0-100 % */
    gint lumi; /* 0 à °° */  
    gint morphing;/* dummy for future */ 
	gchar* key_name;
	GList* items_list;/* if we want to use our ela-items in place of goocanvas items */
} effects_data_type;
	  
/* structure pour stocker dans une pile les images */
typedef struct
{
    GdkPixbuf* pixb;
    gchar* nom;
    gchar* complete_path;
} stack_images;

typedef struct 
{
    gchar* snd_file;
    gchar* item_snd_name;
    gboolean playing;
} stack_sound;

/* new type for path objects */
typedef struct
{
    gdouble x;
    gdouble y;
} ElemPath;

typedef struct
{
	gchar* clip_file;
	gchar* clip_name;
} stack_clips;

typedef struct
{
	gchar* button_name;/* name of the button in Ella system*/
	gchar* button_file;/* name of the hidden file in /tmp which describes the button */
	gint button_type;/* in order to know if it's a bitmap button with special files */
	gchar* button_file_idle;/* bitmap file for 'idle' state of a bitmpa button */
	gchar* button_file_hover;/* same for 'hover' state */
	gchar* button_file_area;/*same for 'sensible area' definition */
	gchar* button_file_pressed;/* same for 'pressedclicked' state */
} button_item;

GSList* pile_images;    
GSList* StackClips;
GSList* StackButtons;

/* accès aux éléments du notebook */
GtkWidget* scrolledwindowEffects;
GtkWidget* scrolledwindowTransitions;  
GtkWidget* scrolledwindowProperties;
/* transitions */
GtkWidget* checkbuttonInterpola;
GtkWidget* checkbuttonZoom;
GtkWidget* checkbuttonRotation;
GtkWidget* checkbuttonFading;
GtkWidget* checkbuttonAlphaT;
GtkWidget* checkbuttonMorphing;
GtkWidget* spinbuttonRotation;
GtkWidget* spinbuttonFading;
GtkWidget* spinbuttonZoom;
GtkWidget* spinbuttonAlphaT;
/* propriétés */
GtkSpinButton* entryXPosProperties;
GtkSpinButton* entryYPosProperties;
GtkSpinButton* entryWidthProperties;
GtkSpinButton* entryHeightProperties;
GtkWidget* checkbuttonConstrainProperties;  
/* builder objects */
GtkBuilder* builder_dlg;  

#define GOO_CANVAS_ANCHOR_CENTER GTK_ANCHOR_CENTER
#define GOO_CANVAS_ANCHOR_WEST GTK_ANCHOR_WEST

#endif
