/* Copyright (C) 2011 Lucas Baudin
 *
 * This file is part of Ella Project.
 *
 * Ella Project is free software : you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ella Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ella Project. If not, see <http ://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include "ella-user_config.h"
#include "ella-widgets.h"
#include "interface.h"
#include "callbacks.h"

#include "export.h"

static GtkBuilder* builder = NULL;

static void on_export_png_finished(EllaCanvas* canvas, gpointer data)
{
    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(builder, "png_loading")), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(builder, "export_png_button")), TRUE);
}

static void on_export_gif_finished(EllaCanvas* canvas, gpointer data)
{
    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(builder, "gif_loading")), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(builder, "export_gif_button")), TRUE);
}

static void on_export_avi_finished(EllaCanvas* canvas, gpointer data)
{
    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(builder, "avi_loading")), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(builder, "export_avi_button")), TRUE);
}

void ella_export_show_dialog(gchar* path)
{
    static gboolean first = TRUE;
    builder = gtk_builder_new();
    gtk_builder_add_from_file(builder, g_build_filename(ella_get_data_dir(), "ui/export.ui", NULL), NULL);
    gtk_builder_connect_signals(builder, NULL);
    if(first)
    {
        first = FALSE;
        g_signal_connect(canvas, "export_png_finished", G_CALLBACK(on_export_png_finished), NULL);
        g_signal_connect(canvas, "export_avi_finished", G_CALLBACK(on_export_avi_finished), NULL);
        g_signal_connect(canvas, "export_gif_finished", G_CALLBACK(on_export_gif_finished), NULL);
    }
    GtkWidget* dialog = GTK_WIDGET(gtk_builder_get_object(builder, "window1"));
    g_debug("Current filename: %s\n", path);
    GtkEntry* svg_entry, *png_entry, *gif_entry, *avi_entry;
    svg_entry = GTK_ENTRY(gtk_builder_get_object(builder, "svg_entry"));
    png_entry = GTK_ENTRY(gtk_builder_get_object(builder, "png_entry"));
    gif_entry = GTK_ENTRY(gtk_builder_get_object(builder, "gif_entry"));
    avi_entry = GTK_ENTRY(gtk_builder_get_object(builder, "avi_entry"));
    gchar* filename;
    if(path == NULL)
    {
        filename = g_build_filename(ella_config_get_value("PREFERED_FOLDER"), "Untitled", NULL);
    }
    else
    {
        gchar** split = g_strsplit(path, ".esvg", -1);
        filename = g_strdup(split[0]);
        g_strfreev(split);
    }
    gtk_entry_set_text(svg_entry, g_strdup_printf("%s.svg", filename));
    gtk_entry_set_text(png_entry, g_strdup_printf("%s ", filename));
    gtk_entry_set_text(gif_entry, g_strdup_printf("%s.gif", filename));
    gtk_entry_set_text(avi_entry, g_strdup_printf("%s.avi", filename));

    gtk_widget_show_all(GTK_WIDGET(dialog));
}

void on_export_file_chooser_activate(GtkWidget* button, GtkEntry* entry)
{
    g_return_if_fail(GTK_IS_ENTRY(entry));
    GtkWidget *dialog;
    dialog = gtk_file_chooser_dialog_new ("Save File",
                          NULL,
                          GTK_FILE_CHOOSER_ACTION_SAVE,
                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                          NULL);
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), gtk_entry_get_text(entry));
    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
        gchar *filename;
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        gtk_entry_set_text(entry, filename);
        g_free (filename);
    }
    gtk_widget_destroy (dialog);
}

void on_export_svg_clicked(GtkWidget* button, GtkEntry* entry)
{
    EllaFilesData data;
    ella_get_curent_files_data(&data);
    ella_files_write_ella_file(&data, gtk_entry_get_text(entry));
    g_debug("SVG export path; %s\n", gtk_entry_get_text(entry));
}

void on_export_avi_clicked(GtkWidget* button, GtkEntry* data)
{
    ella_canvas_export_to_avi_async(canvas, gtk_entry_get_text(data), timeline_get_movie_freq());
    gtk_widget_set_sensitive(button, FALSE);
    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(builder, "avi_loading")), TRUE);
    gtk_main_iteration();
}

void on_export_gif_clicked(GtkWidget* button, GtkEntry* data)
{
    ella_canvas_export_to_gif_async(canvas, gtk_entry_get_text(data), timeline_get_movie_freq());
    gtk_widget_set_sensitive(button, FALSE);
    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(builder, "gif_loading")), TRUE);
    gtk_main_iteration();
}

void on_export_png_clicked(GtkWidget* button, GtkEntry* data)
{
    ella_canvas_export_to_png_async(canvas, gtk_entry_get_text(data));
    gtk_widget_set_sensitive(button, FALSE);
    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(builder, "png_loading")), TRUE);
    gtk_main_iteration();
}

void on_export_close_cliked(GtkWidget* button, GtkWidget* window)
{
    gtk_widget_destroy(window);
}

