/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "filesfuncs.h"

/* constantes des effets :
 1 mouvement, 2 zoom, 4 rotation, 8 luminance,
			   * 16 alpha */

/* structure spéciale pour optimiser l'usage des fontes */
typedef struct
{
	gchar *file_font;
	gchar *nom_font;
} stack_font;
	  	  
GSList *pile_items_export;
GSList *pile_fontes_export;
GSList *pile_sound;

/* variables globales */
gint stack_object_width,stack_object_height;
gdouble stack_object_rotation;
gint snd_item_count=0;

/************************************
 * looking for an item which name
 * is known, on the stack of
 *  images ;if found, returns a
 * pointer on pixbuf, either NULL
 * *********************************/
GdkPixbuf* find_item_on_images_stack(GSList* pile, gchar* titre)
{
   gboolean trouve = FALSE;
   GSList* temp    = pile;
   GdkPixbuf* pb   = NULL;

   if(titre)
   {
      while(temp && !trouve)
      { 
         if(strcmp(((stack_images*)(temp->data))->nom, titre) == 0)
         {          	
            trouve = TRUE;
            pb = ((stack_images*)(temp->data))->pixb;
         }	
         else
         {	  
            temp = g_slist_next(temp);
         }
      }
   }
    return pb;
}
/************************************
 * looking for an item which name
 * is known, on the stack of
 *  clips ;if found, returns a
 * pointer on file, either NULL
 * *********************************/
gchar* find_item_on_clips_stack(GSList* pile, gchar* titre)
{
   gboolean trouve = FALSE;
   GSList* temp    = pile;
   gchar* pb   = NULL;

   if(titre)
   {
      while(temp && !trouve)
      { 
         if(strcmp(((stack_clips*)(temp->data))->clip_name, titre) == 0)
         {          	
            trouve = TRUE;
            pb = ((stack_clips*)(temp->data))->clip_file;
         }	
         else
         {	  
            temp = g_slist_next(temp);
         }
      }
   }
    return pb;
}
/******************************
 * conversion des couleurs
 * GOOCANVAS sur 32 bits vers 
 * chaînes couleurs
 * en 8 bits 
 * **************************/
gchar *strgdk8bits(guint32 couleur)
{
	gint16 r,v,b;
	gchar *temp=NULL;
	
	/* passage de 32 bits RGBA en 16 bits composantes */
    r=(couleur & 0xFF000000)>>24;
    v=(couleur & 0x00FF0000)>>16;
    b=(couleur & 0x0000FF00)>>8;
	/* passage de 16 bits en 8 bits par plan */
	temp=g_strdup_printf("#%.2x%.2x%.2x",r,v,b);  	
	return temp;	
}

/******************************
 * conversion vers couleurs
 * en 8 bits (on part
 * d'une GdkColor)
 * **************************/
gchar *gdk8bits(GdkColor *couleur)
{
	gchar *temp=NULL;
	
	temp=g_strdup_printf("#%.2x%.2x%.2x",(gint)(255*(couleur->red/65535.0)),(gint)(255*(couleur->green/65535.0)), 
	                (gint)(255*(couleur->blue/65535.0)));  	
	return temp;	
}
/******************************
 * fermer le fichier courant
 * 
 * ***************************/
void fermer_fichier_ella(FILE *f)
{	
 fprintf(f,"[FILEND]\n");/* obligatoire pour éviter le bug en relecture */
 fclose(f);
}


/***********************************
 * enregistrer le contenu de tous 
 * les calques ; en entrée, pointeur
 * sur la liste des calques,
 * suivi du nombre total réel de
 * calques
 * *******************************/
void sauver_donnees_ella(FILE *f, GtkWidget *pListLayers, gint nb_layers, gint maxframes)
{
 /* TODO: Deprecated */
}

/**********************************
 * tester si l'image courante est
 * une image-clé ; si OUI,
 * la fonction renvoit un ensemble
 * de bits allumés ou éteints
 * indiquant les effets activés
 * le paramètre est en coordonnées
 * PC.
 * GSList correspond à keyframes_list
 * ***********************************/
guint get_transitions_keyframe(GSList *list,guint kf)
{
   gboolean trouve = FALSE;  
   guint effets = 0;
   GSList *temp = NULL;

   temp = g_slist_nth (list, 0);
   while(temp != NULL && trouve == FALSE)
   {
      if(((effects_data_type*)(temp->data))->keyframe == kf)
      {          	
         trouve = TRUE;
         /* passer les bits : 0 rien, 1 mouvement, 2 zoom, 4 rotation,
         * 8 luminance, 16 alpha */
         if (((effects_data_type *)(temp->data))->effect_motion)
            effets |= 1;

         if (((effects_data_type*)(temp->data))->effect_zooming)
            effets |= 2;

         if (((effects_data_type*)(temp->data))->effect_rotation)
            effets |= 4;

         if (((effects_data_type*)(temp->data))->effect_lumi)
            effets |= 8;

         if (((effects_data_type*)(temp->data))->effect_alpha)
            effets |= 16;				
      }		  
      temp = g_slist_next(temp);
   }
   return effets;
}
